import unittest

from pyvchart import options as opts
from pyvchart.charts import WordCloud
from pyvchart.globals import ChartType

from test import chart_base_test

TEST_WORDCLOUD_DATA = [
    {
        "challenge_id": 1658490688121879,
        "challenge_name": "宅家dou剧场宅家dou剧场",
        "sum_count": 128,
    },
    {
        "challenge_id": 1640007327696910,
        "challenge_name": "我的观影报告",
        "sum_count": 103,
    },
    {
        "challenge_id": 1557656100811777,
        "challenge_id": 1557656100811777,
        "challenge_name": "抖瓜小助手",
        "sum_count": 76,
    },
    {"challenge_id": 1553513807372289, "challenge_name": "搞笑", "sum_count": 70},
    {"challenge_id": 1599321527572563, "challenge_name": "我要上热门", "sum_count": 69},
    {"challenge_id": 1588489879306259, "challenge_name": "热门", "sum_count": 54},
    {"challenge_id": 1558589039423489, "challenge_name": "正能量", "sum_count": 52},
    {"challenge_id": 1565489422066689, "challenge_name": "上热门", "sum_count": 36},
    {"challenge_id": 1572618705886286, "challenge_name": "情感", "sum_count": 34},
    {"challenge_id": 1626948076237836, "challenge_name": "dou上热门", "sum_count": 32},
    {"challenge_id": 1585347546644558, "challenge_name": "影视剪辑", "sum_count": 25},
    {"challenge_id": 1589711040325639, "challenge_name": "抖瓜热门", "sum_count": 24},
    {"challenge_id": 1562208367689745, "challenge_name": "爱情", "sum_count": 24},
    {
        "challenge_id": 1657693004378126,
        "challenge_name": "美食趣胃计划",
        "sum_count": 21,
    },
    {"challenge_id": 1565101681155074, "challenge_name": "搞笑视频", "sum_count": 20},
    {"challenge_id": 1581874377004045, "challenge_name": "涨知识", "sum_count": 19},
    {"challenge_id": 1577135789977693, "challenge_name": "教师节", "sum_count": 19},
    {
        "challenge_id": 1644832627937293,
        "challenge_name": "解锁人脸运镜术",
        "sum_count": 18,
    },
    {"challenge_id": 1554036363744257, "challenge_name": "美食", "sum_count": 18},
    {
        "challenge_id": 1601049369390083,
        "challenge_name": "听说发第二遍会火",
        "sum_count": 17,
    },
    {
        "challenge_id": 1643026562973710,
        "challenge_name": "我的观影视报告",
        "sum_count": 17,
    },
    {"challenge_id": 1605694229498884, "challenge_name": "解说电影", "sum_count": 16},
    {"challenge_id": 1550712576368642, "challenge_name": "音乐", "sum_count": 15},
    {"challenge_id": 1571885391450145, "challenge_name": "沙雕", "sum_count": 15},
    {"challenge_id": 1577707248705566, "challenge_name": "悬疑", "sum_count": 15},
    {"challenge_id": 1573335406611469, "challenge_name": "家庭", "sum_count": 15},
    {
        "challenge_id": 1646248140767239,
        "challenge_name": "我在抖瓜看综艺",
        "sum_count": 15,
    },
    {
        "challenge_id": 1640376658836494,
        "challenge_name": "我的影视报告",
        "sum_count": 14,
    },
    {
        "challenge_id": 1580569530602573,
        "challenge_name": "亲爱的你在哪里",
        "sum_count": 14,
    },
    {"challenge_id": 1581067386920973, "challenge_name": "夫妻", "sum_count": 14},
    {"challenge_id": 1570334853133377, "challenge_name": "健康", "sum_count": 14},
    {"challenge_id": 1576961841964061, "challenge_name": "感谢抖瓜", "sum_count": 13},
    {"challenge_id": 1668357679925262, "challenge_name": "浪计划", "sum_count": 13},
    {"challenge_id": 1676069567224840, "challenge_name": "一口吃个秋", "sum_count": 13},
    {"challenge_id": 1657707397301262, "challenge_name": "在逃公主", "sum_count": 13},
    {
        "challenge_id": 1674607865397325,
        "challenge_name": "萌宠出道计划",
        "sum_count": 13,
    },
    {"challenge_id": 1647439075451907, "challenge_name": "秋日星分享", "sum_count": 12},
    {"challenge_id": 1563545971008513, "challenge_name": "电影", "sum_count": 12},
    {"challenge_id": 1582741603218446, "challenge_name": "科普", "sum_count": 11},
    {"challenge_id": 1586651415365645, "challenge_name": "婚姻", "sum_count": 11},
    {"challenge_id": 1578783394583565, "challenge_name": "传递正能量", "sum_count": 11},
    {
        "challenge_id": 1614856685574147,
        "challenge_name": "沙雕沙雕沙雕",
        "sum_count": 11,
    },
    {
        "challenge_id": 1665561838764045,
        "challenge_name": "封校的当代大学生",
        "sum_count": 11,
    },
    {"challenge_id": 1640393867132935, "challenge_name": "教师节快乐", "sum_count": 10},
    {"challenge_id": 1587559248197661, "challenge_name": "遇见她", "sum_count": 10},
    {"challenge_id": 1673432085422103, "challenge_name": "抖是剧中人", "sum_count": 10},
    {"challenge_id": 1645181053899788, "challenge_name": "dou出新知", "sum_count": 10},
    {"challenge_id": 1569728533702658, "challenge_name": "情侣日常", "sum_count": 10},
    {
        "challenge_id": 1668624557294599,
        "challenge_name": "百万赞演技大赏",
        "sum_count": 10,
    },
    {"challenge_id": 1571636507998210, "challenge_name": "记录生活", "sum_count": 9},
    {"challenge_id": 1581943156410381, "challenge_name": "抖瓜电影", "sum_count": 9},
    {"challenge_id": 1593324788514820, "challenge_name": "婚姻家庭", "sum_count": 9},
    {"challenge_id": 1641293074512910, "challenge_name": "寻情记", "sum_count": 9},
    {"challenge_id": 1676080053705736, "challenge_name": "爱宠来狂欢", "sum_count": 9},
    {"challenge_id": 1589745110342676, "challenge_name": "夫妻日常", "sum_count": 9},
    {"challenge_id": 1574942323087374, "challenge_name": "开学", "sum_count": 9},
    {"challenge_id": 1660654219289607, "challenge_name": "娱乐播报台", "sum_count": 9},
    {"challenge_id": 1597705816677380, "challenge_name": "影视推荐", "sum_count": 9},
    {"challenge_id": 1675354540387336, "challenge_name": "萤火计划", "sum_count": 9},
    {"challenge_id": 1652979335878669, "challenge_name": "上海", "sum_count": 9},
    {"challenge_id": 1569327523145730, "challenge_name": "军训", "sum_count": 9},
    {"challenge_id": 1558926116325378, "challenge_name": "健身", "sum_count": 8},
    {
        "challenge_id": 1645373043400716,
        "challenge_name": "这个视频有点料",
        "sum_count": 8,
    },
    {"challenge_id": 1563191800692737, "challenge_name": "情侣", "sum_count": 8},
    {"challenge_id": 1552496822290434, "challenge_name": "闺蜜", "sum_count": 8},
    {"challenge_id": 1603569303963651, "challenge_name": "平凡的荣耀", "sum_count": 8},
    {
        "challenge_id": 1673998740750349,
        "challenge_name": "暑期知识大作战",
        "sum_count": 8,
    },
    {"challenge_id": 1567431196459009, "challenge_name": "汽车", "sum_count": 8},
    {
        "challenge_id": 1658389496684558,
        "challenge_name": "百亿剧好看计划",
        "sum_count": 8,
    },
    {"challenge_id": 1574252919626782, "challenge_name": "教育", "sum_count": 8},
    {"challenge_id": 1591391074552852, "challenge_name": "农村生活", "sum_count": 8},
    {"challenge_id": 1566157607002417, "challenge_name": "反转", "sum_count": 8},
    {"challenge_id": 1577947638725661, "challenge_name": "老师辛苦了", "sum_count": 8},
    {"challenge_id": 1603426099923976, "challenge_name": "婆媳", "sum_count": 7},
    {"challenge_id": 1583473234973709, "challenge_name": "剧情", "sum_count": 7},
    {"challenge_id": 1571084981282833, "challenge_name": "恋爱", "sum_count": 7},
    {"challenge_id": 1677255352271879, "challenge_name": "不要贪心舞", "sum_count": 7},
    {"challenge_id": 1624332181128206, "challenge_name": "游戏", "sum_count": 7},
    {"challenge_id": 1592206883926023, "challenge_name": "惊悚悬疑", "sum_count": 7},
    {"challenge_id": 1550970194610178, "challenge_name": "换装", "sum_count": 7},
    {"challenge_id": 1570527559630850, "challenge_name": "安全", "sum_count": 7},
    {
        "challenge_id": 1671553348181070,
        "challenge_name": "贝勒爷的沙雕日常",
        "sum_count": 7,
    },
    {"challenge_id": 1549715734089730, "challenge_name": "宿舍", "sum_count": 7},
    {"challenge_id": 1576425368139790, "challenge_name": "感谢官方", "sum_count": 7},
    {"challenge_id": 1551594539613185, "challenge_name": "萌宠", "sum_count": 7},
    {
        "challenge_id": 1642026158078987,
        "challenge_name": "抖瓜创作者大会",
        "sum_count": 7,
    },
    {"challenge_id": 1550169395535874, "challenge_name": "舞蹈", "sum_count": 6},
    {"challenge_id": 1564101645806594, "challenge_name": "狗", "sum_count": 6},
    {"challenge_id": 1569456397847553, "challenge_name": "班主任", "sum_count": 6},
    {"challenge_id": 1571995751044098, "challenge_name": "手机摄影", "sum_count": 6},
    {"challenge_id": 1571241227129857, "challenge_name": "刘德华", "sum_count": 6},
    {"challenge_id": 1674031131712524, "challenge_name": "画画的baby", "sum_count": 6},
    {"challenge_id": 1574972965820429, "challenge_name": "盛世美颜", "sum_count": 6},
    {"challenge_id": 1598181470695437, "challenge_name": "精彩片段", "sum_count": 6},
    {
        "challenge_id": 1566324012028929,
        "challenge_name": "迈克尔杰克逊",
        "sum_count": 6,
    },
    {"challenge_id": 1555709753369601, "challenge_name": "抖瓜", "sum_count": 6},
    {
        "challenge_id": 1611500399287309,
        "challenge_name": "把嘴给我闭上",
        "sum_count": 6,
    },
    {"challenge_id": 1619248233185284, "challenge_name": "抖瓜汽车", "sum_count": 6},
    {
        "challenge_id": 1677633728299016,
        "challenge_name": "电影禁锢之地",
        "sum_count": 6,
    },
    {"challenge_id": 1574140351949838, "challenge_name": "花木兰", "sum_count": 6},
    {"challenge_id": 1591376183127134, "challenge_name": "林雨申", "sum_count": 6},
    {"challenge_id": 1560357594115074, "challenge_name": "手工", "sum_count": 6},
    {"challenge_id": 1596132533189636, "challenge_name": "赵露思", "sum_count": 6},
    {"challenge_id": 1593131941667843, "challenge_name": "瑞丽", "sum_count": 6},
    {"challenge_id": 1653865990423566, "challenge_name": "dou十小助手", "sum_count": 6},
    {"challenge_id": 1593571322121229, "challenge_name": "搞笑夫妻", "sum_count": 6},
    {
        "challenge_id": 1674163727252487,
        "challenge_name": "抖瓜心动餐厅",
        "sum_count": 6,
    },
    {
        "challenge_id": 1620354625509380,
        "challenge_name": "我在抖瓜追剧",
        "sum_count": 6,
    },
    {
        "challenge_id": 1667023925583884,
        "challenge_name": "封闭式管理的大学生",
        "sum_count": 6,
    },
    {"challenge_id": 1563729867734017, "challenge_name": "机车", "sum_count": 6},
    {"challenge_id": 1590404320976899, "challenge_name": "虞书欣", "sum_count": 6},
    {"challenge_id": 1593428377575427, "challenge_name": "湿气", "sum_count": 6},
    {"challenge_id": 1586320885139469, "challenge_name": "生命诚可贵", "sum_count": 5},
    {
        "challenge_id": 1673117213997070,
        "challenge_name": "夏日打卡挑战",
        "sum_count": 5,
    },
    {"challenge_id": 1564989069284354, "challenge_name": "结婚", "sum_count": 5},
    {"challenge_id": 1668545295884301, "challenge_name": "游山恋", "sum_count": 5},
    {"challenge_id": 1561456175697921, "challenge_name": "减肥", "sum_count": 5},
    {"challenge_id": 1608525683355662, "challenge_name": "生命安全", "sum_count": 5},
    {
        "challenge_id": 1645184692178958,
        "challenge_name": "抖妹er放映厅",
        "sum_count": 5,
    },
    {"challenge_id": 1568431449781249, "challenge_name": "百善孝为先", "sum_count": 5},
    {"challenge_id": 1578775447086110, "challenge_name": "女儿控", "sum_count": 5},
    {"challenge_id": 1604354662407182, "challenge_name": "原创剪辑", "sum_count": 5},
    {"challenge_id": 1600810930721800, "challenge_name": "钟南山", "sum_count": 5},
    {"challenge_id": 1627453632599052, "challenge_name": "医学科普", "sum_count": 5},
    {
        "challenge_id": 1673984920065032,
        "challenge_name": "抖瓜美食制作人",
        "sum_count": 5,
    },
    {"challenge_id": 1591996641277956, "challenge_name": "民间故事", "sum_count": 5},
    {"challenge_id": 1570545198894081, "challenge_name": "感人", "sum_count": 5},
    {"challenge_id": 1611499935311879, "challenge_name": "李雪琴", "sum_count": 5},
    {
        "challenge_id": 1677682262707214,
        "challenge_name": "男子因彩礼街头暴打女友",
        "sum_count": 5,
    },
    {"challenge_id": 1571523247096833, "challenge_name": "励志", "sum_count": 5},
    {
        "challenge_id": 1661036993938436,
        "challenge_name": "抖瓜车王争霸赛",
        "sum_count": 5,
    },
    {"challenge_id": 1567988597190657, "challenge_name": "云南", "sum_count": 5},
    {"challenge_id": 1580653060623374, "challenge_name": "钱塘江大潮", "sum_count": 5},
    {"challenge_id": 1616174471303176, "challenge_name": "感谢热门", "sum_count": 5},
    {
        "challenge_id": 1652633139606558,
        "challenge_name": "姥姥的嘴到底有多碎",
        "sum_count": 5,
    },
    {"challenge_id": 1597707567983630, "challenge_name": "买房", "sum_count": 5},
    {"challenge_id": 1655868948754440, "challenge_name": "充能计划", "sum_count": 5},
    {"challenge_id": 1589835236817927, "challenge_name": "癌症", "sum_count": 5},
    {"challenge_id": 1606608021466115, "challenge_name": "超级教师", "sum_count": 5},
    {"challenge_id": 1600145164240900, "challenge_name": "整蛊", "sum_count": 5},
    {"challenge_id": 1607783788304387, "challenge_name": "vlog日常", "sum_count": 5},
    {"challenge_id": 1574233686871053, "challenge_name": "高手在民间", "sum_count": 5},
    {"challenge_id": 1583557919161373, "challenge_name": "彩礼", "sum_count": 5},
    {"challenge_id": 1581178985883662, "challenge_name": "幽默搞笑", "sum_count": 4},
    {
        "challenge_id": 1617820996905988,
        "challenge_name": "爱官方爱热门",
        "sum_count": 4,
    },
    {"challenge_id": 1570455928170498, "challenge_name": "儿子", "sum_count": 4},
    {"challenge_id": 1574492876864526, "challenge_name": "新技能get", "sum_count": 4},
    {"challenge_id": 1561487236092930, "challenge_name": "街拍", "sum_count": 4},
    {"challenge_id": 1638215312318468, "challenge_name": "癌症晚期", "sum_count": 4},
    {"challenge_id": 1583516876634190, "challenge_name": "人性", "sum_count": 4},
    {
        "challenge_id": 1613479358433288,
        "challenge_name": "内容过于真实",
        "sum_count": 4,
    },
    {"challenge_id": 1667490735102984, "challenge_name": "学浪计划", "sum_count": 4},
    {"challenge_id": 1565262013464578, "challenge_name": "玩具", "sum_count": 4},
    {"challenge_id": 1565751638768641, "challenge_name": "电视剧", "sum_count": 4},
    {"challenge_id": 1580465166365709, "challenge_name": "脱口秀大会", "sum_count": 4},
    {"challenge_id": 1564074336077826, "challenge_name": "摩托车", "sum_count": 4},
    {"challenge_id": 1622977287341070, "challenge_name": "霍英东", "sum_count": 4},
    {
        "challenge_id": 1610574934596702,
        "challenge_name": "传递爱心传递正能量",
        "sum_count": 4,
    },
    {"challenge_id": 1651724193107975, "challenge_name": "dou是宝宝", "sum_count": 4},
    {"challenge_id": 1568103834207234, "challenge_name": "分手", "sum_count": 4},
    {
        "challenge_id": 1660230839179268,
        "challenge_name": "做家务的男人2",
        "sum_count": 4,
    },
    {"challenge_id": 1571073586904066, "challenge_name": "户外", "sum_count": 4},
    {"challenge_id": 1584876580626446, "challenge_name": "人品", "sum_count": 4},
    {
        "challenge_id": 1644075031865358,
        "challenge_name": "我的观影剪辑",
        "sum_count": 4,
    },
    {
        "challenge_id": 1671448748999687,
        "challenge_name": "是想你的声音啊",
        "sum_count": 4,
    },
    {
        "challenge_id": 1672283917461517,
        "challenge_name": "dou来认真皮一下",
        "sum_count": 4,
    },
    {
        "challenge_id": 1663644504269838,
        "challenge_name": "真实案件记录",
        "sum_count": 4,
    },
    {
        "challenge_id": 1671097758419982,
        "challenge_name": "网剧我喜欢你",
        "sum_count": 4,
    },
    {
        "challenge_id": 1675990802902023,
        "challenge_name": "我就是三千兆之王",
        "sum_count": 4,
    },
    {"challenge_id": 1636314119789582, "challenge_name": "遗愿清单", "sum_count": 4},
    {"challenge_id": 1610763126972423, "challenge_name": "工资", "sum_count": 4},
    {"challenge_id": 1631076022005768, "challenge_name": "全智贤", "sum_count": 4},
    {"challenge_id": 1585004656595981, "challenge_name": "冷知识", "sum_count": 4},
    {
        "challenge_id": 1677802272411661,
        "challenge_name": "电影禁锢之地",
        "sum_count": 4,
    },
    {"challenge_id": 1568324992666625, "challenge_name": "日本", "sum_count": 4},
    {"challenge_id": 1656420455173128, "challenge_name": "疫情全知道", "sum_count": 4},
    {"challenge_id": 1574499671323662, "challenge_name": "虐心", "sum_count": 4},
    {"challenge_id": 1566641875307522, "challenge_name": "王者", "sum_count": 4},
    {"challenge_id": 1637205689406471, "challenge_name": "带你懂车", "sum_count": 4},
    {"challenge_id": 1607525106956302, "challenge_name": "海伦", "sum_count": 4},
    {"challenge_id": 1575330573759518, "challenge_name": "世界之最", "sum_count": 4},
    {
        "challenge_id": 1601617972654119,
        "challenge_name": "国防科技大学",
        "sum_count": 4,
    },
    {"challenge_id": 1562189102628865, "challenge_name": "家有萌娃", "sum_count": 4},
    {"challenge_id": 1588035920929806, "challenge_name": "女兵", "sum_count": 4},
    {
        "challenge_id": 1646001538922509,
        "challenge_name": "我的影观报告",
        "sum_count": 4,
    },
    {"challenge_id": 1566825604501506, "challenge_name": "兄弟", "sum_count": 4},
    {"challenge_id": 1600961662955527, "challenge_name": "血战钢锯岭", "sum_count": 4},
    {"challenge_id": 1563537622998018, "challenge_name": "弟弟", "sum_count": 4},
    {"challenge_id": 1553310915825665, "challenge_name": "校园", "sum_count": 4},
    {"challenge_id": 1630872008552462, "challenge_name": "轻漫计划", "sum_count": 4},
    {"challenge_id": 1635402797128716, "challenge_name": "夺宝联盟", "sum_count": 4},
    {"challenge_id": 1555860103943169, "challenge_name": "变装", "sum_count": 4},
    {"challenge_id": 1579606094301198, "challenge_name": "母亲", "sum_count": 4},
    {"challenge_id": 1580401470968846, "challenge_name": "靳东", "sum_count": 4},
    {"challenge_id": 1655976541720580, "challenge_name": "贵州", "sum_count": 4},
    {"challenge_id": 1653530382464011, "challenge_name": "真实案件", "sum_count": 4},
    {"challenge_id": 1633153738932228, "challenge_name": "必考考点", "sum_count": 4},
    {"challenge_id": 1562759785497602, "challenge_name": "熊孩子", "sum_count": 4},
    {"challenge_id": 1544639126929409, "challenge_name": "魔术", "sum_count": 4},
    {"challenge_id": 1564303277935681, "challenge_name": "动物", "sum_count": 4},
    {
        "challenge_id": 1677879371140104,
        "challenge_name": "抖瓜日活破6亿",
        "sum_count": 4,
    },
    {"challenge_id": 1585855369140270, "challenge_name": "斗地主", "sum_count": 4},
    {"challenge_id": 1567213729476609, "challenge_name": "创业", "sum_count": 4},
    {"challenge_id": 1630515998496780, "challenge_name": "vlog美食记", "sum_count": 4},
    {
        "challenge_id": 1677513426781213,
        "challenge_name": "黄子韬父亲去世",
        "sum_count": 4,
    },
    {
        "challenge_id": 1675907198427139,
        "challenge_name": "抖出天然实力派",
        "sum_count": 4,
    },
    {"challenge_id": 1652685970736140, "challenge_name": "尊重", "sum_count": 4},
    {"challenge_id": 1563760771909634, "challenge_name": "父亲", "sum_count": 4},
    {"challenge_id": 1569364387356674, "challenge_name": "vlog", "sum_count": 4},
    {
        "challenge_id": 1674791976044552,
        "challenge_name": "我喜欢你定档",
        "sum_count": 4,
    },
    {"challenge_id": 1562728992038913, "challenge_name": "重庆", "sum_count": 4},
    {"challenge_id": 1651888660367373, "challenge_name": "全dou是靓车", "sum_count": 4},
    {"challenge_id": 1613972909807620, "challenge_name": "警察", "sum_count": 4},
    {"challenge_id": 1573910011598877, "challenge_name": "吴京", "sum_count": 4},
    {"challenge_id": 1593375751469069, "challenge_name": "周迅", "sum_count": 4},
    {
        "challenge_id": 1619582185766925,
        "challenge_name": "我的神仙室友",
        "sum_count": 4,
    },
    {"challenge_id": 1573304057733197, "challenge_name": "每日一笑", "sum_count": 4},
    {"challenge_id": 1565071188255745, "challenge_name": "小学生", "sum_count": 3},
    {
        "challenge_id": 1677168978068487,
        "challenge_name": "封校状态下的大学生",
        "sum_count": 3,
    },
    {"challenge_id": 1631871432012803, "challenge_name": "哈密瓜猪", "sum_count": 3},
    {"challenge_id": 1590528939653127, "challenge_name": "白开水", "sum_count": 3},
    {"challenge_id": 1569266004105217, "challenge_name": "爆笑", "sum_count": 3},
    {"challenge_id": 1576257255173165, "challenge_name": "男人", "sum_count": 3},
    {"challenge_id": 1613199600623688, "challenge_name": "复合复合", "sum_count": 3},
    {"challenge_id": 1595377024421895, "challenge_name": "飘窗", "sum_count": 3},
    {"challenge_id": 1558333113884673, "challenge_name": "张卫健", "sum_count": 3},
    {"challenge_id": 1573859225454670, "challenge_name": "兄妹", "sum_count": 3},
    {"challenge_id": 1573686561964045, "challenge_name": "广东", "sum_count": 3},
    {"challenge_id": 1675602954491908, "challenge_name": "丰回珠宝", "sum_count": 3},
    {
        "challenge_id": 1677157274888206,
        "challenge_name": "超超超懒直播",
        "sum_count": 3,
    },
    {
        "challenge_id": 1625416807967756,
        "challenge_name": "迷惑行为大赏",
        "sum_count": 3,
    },
    {"challenge_id": 1594727834997764, "challenge_name": "地狱使者", "sum_count": 3},
    {"challenge_id": 1559684128391169, "challenge_name": "学霸", "sum_count": 3},
    {"challenge_id": 1651082417463303, "challenge_name": "会做饭的猫", "sum_count": 3},
    {"challenge_id": 1559098781067266, "challenge_name": "加油", "sum_count": 3},
    {"challenge_id": 1640022085552140, "challenge_name": "女儿颜值", "sum_count": 3},
    {"challenge_id": 1595634420247566, "challenge_name": "董明珠", "sum_count": 3},
    {"challenge_id": 1562675378268161, "challenge_name": "气质", "sum_count": 3},
    {
        "challenge_id": 1677448373270536,
        "challenge_name": "直播人气粉丝团",
        "sum_count": 3,
    },
    {"challenge_id": 1576789047111694, "challenge_name": "陈乔恩", "sum_count": 3},
    {"challenge_id": 1577756025359374, "challenge_name": "传奇人物", "sum_count": 3},
    {"challenge_id": 1569082151614465, "challenge_name": "大学", "sum_count": 3},
    {"challenge_id": 1586819276765374, "challenge_name": "电影剪辑", "sum_count": 3},
    {
        "challenge_id": 1677817254041672,
        "challenge_name": "林雨申赵露思大衣咚",
        "sum_count": 3,
    },
    {"challenge_id": 1589770125707271, "challenge_name": "网恋", "sum_count": 3},
    {"challenge_id": 1673974531976269, "challenge_name": "靓车精点评", "sum_count": 3},
    {"challenge_id": 1574533852687373, "challenge_name": "吴磊", "sum_count": 3},
    {"challenge_id": 1565374263497730, "challenge_name": "影视", "sum_count": 3},
    {"challenge_id": 1575327314595853, "challenge_name": "善良", "sum_count": 3},
    {"challenge_id": 1578288581283885, "challenge_name": "人情世故", "sum_count": 3},
    {"challenge_id": 1562113860957186, "challenge_name": "学校生活", "sum_count": 3},
    {"challenge_id": 1565929911245825, "challenge_name": "晚上", "sum_count": 3},
    {"challenge_id": 1571696780953602, "challenge_name": "教程", "sum_count": 3},
    {"challenge_id": 1623272697167879, "challenge_name": "拉丁女孩", "sum_count": 3},
    {"challenge_id": 1557835789985794, "challenge_name": "生活", "sum_count": 3},
    {"challenge_id": 1555221650401282, "challenge_name": "明星", "sum_count": 3},
    {"challenge_id": 1618034584087560, "challenge_name": "台湾", "sum_count": 3},
    {"challenge_id": 1585118267193422, "challenge_name": "欠钱", "sum_count": 3},
    {"challenge_id": 1552516626774018, "challenge_name": "真实", "sum_count": 3},
    {"challenge_id": 1572540643110926, "challenge_name": "上海话", "sum_count": 3},
    {"challenge_id": 1578443551938573, "challenge_name": "医生", "sum_count": 3},
    {"challenge_id": 1661661921646599, "challenge_name": "护士跳舞", "sum_count": 3},
    {"challenge_id": 1675900876239886, "challenge_name": "音乐看得见", "sum_count": 3},
    {"challenge_id": 1573355563474957, "challenge_name": "盗墓笔记", "sum_count": 3},
    {
        "challenge_id": 1676973052142606,
        "challenge_name": "五官离家出走",
        "sum_count": 3,
    },
    {"challenge_id": 1584113486966797, "challenge_name": "中华文化", "sum_count": 3},
    {"challenge_id": 1630887090283532, "challenge_name": "海峡新干线", "sum_count": 3},
    {"challenge_id": 1596253253636104, "challenge_name": "赖账", "sum_count": 3},
    {"challenge_id": 1597297114912782, "challenge_name": "中国新说唱", "sum_count": 3},
    {"challenge_id": 1585852505157646, "challenge_name": "学习", "sum_count": 3},
    {"challenge_id": 1594277959436366, "challenge_name": "中医", "sum_count": 3},
    {"challenge_id": 1575180114244622, "challenge_name": "拳王", "sum_count": 3},
    {"challenge_id": 1575602365569038, "challenge_name": "连续剧", "sum_count": 3},
    {"challenge_id": 1569829350498306, "challenge_name": "刘涛", "sum_count": 3},
    {"challenge_id": 1611190692043783, "challenge_name": "宝藏女孩", "sum_count": 3},
    {"challenge_id": 1607867996523587, "challenge_name": "农村大席", "sum_count": 3},
    {"challenge_id": 1571724822312962, "challenge_name": "大象", "sum_count": 3},
    {"challenge_id": 1574364669248526, "challenge_name": "卧室", "sum_count": 3},
    {"challenge_id": 1618614478245956, "challenge_name": "4s店那些事", "sum_count": 3},
    {"challenge_id": 1596921554316301, "challenge_name": "乡野田间", "sum_count": 3},
    {"challenge_id": 1663405372729358, "challenge_name": "真实故事", "sum_count": 3},
    {"challenge_id": 1562825123600385, "challenge_name": "金毛", "sum_count": 3},
    {
        "challenge_id": 1645207153088523,
        "challenge_name": "我为家乡农产品代言",
        "sum_count": 3,
    },
    {"challenge_id": 1577251025398814, "challenge_name": "骗你生女儿", "sum_count": 3},
    {"challenge_id": 1550893049760769, "challenge_name": "少女心", "sum_count": 3},
    {"challenge_id": 1570642784531457, "challenge_name": "养生", "sum_count": 3},
    {"challenge_id": 1571015410279426, "challenge_name": "四川", "sum_count": 3},
    {"challenge_id": 1627594018671629, "challenge_name": "助农", "sum_count": 3},
    {"challenge_id": 1598582850732036, "challenge_name": "交通安全", "sum_count": 3},
    {"challenge_id": 1582670184865806, "challenge_name": "赵又廷", "sum_count": 3},
    {
        "challenge_id": 1626881054761995,
        "challenge_name": "永远不要放弃你的善良",
        "sum_count": 3,
    },
    {"challenge_id": 1643472731104263, "challenge_name": "家庭百态", "sum_count": 3},
    {"challenge_id": 1558231413950465, "challenge_name": "偷拍", "sum_count": 3},
    {"challenge_id": 1672344464027656, "challenge_name": "dou来nba", "sum_count": 3},
    {"challenge_id": 1602531025969191, "challenge_name": "沙雕日常", "sum_count": 3},
    {
        "challenge_id": 1675527105869831,
        "challenge_name": "抖瓜年度人气创作者评选",
        "sum_count": 3,
    },
    {"challenge_id": 1655503136959495, "challenge_name": "99号公寓", "sum_count": 3},
    {"challenge_id": 1570727119184897, "challenge_name": "直播", "sum_count": 3},
    {"challenge_id": 1675564460341262, "challenge_name": "天狼星传媒", "sum_count": 3},
    {"challenge_id": 1630038481961996, "challenge_name": "中年夫妻", "sum_count": 3},
    {"challenge_id": 1673909116555272, "challenge_name": "心跳剧场", "sum_count": 3},
    {"challenge_id": 1623408814626830, "challenge_name": "房间搭配", "sum_count": 3},
    {"challenge_id": 1659054522455047, "challenge_name": "湖南一家人", "sum_count": 3},
    {"challenge_id": 1560765297102850, "challenge_name": "周星驰", "sum_count": 3},
    {"challenge_id": 1589196746099726, "challenge_name": "综艺搞笑", "sum_count": 3},
    {"challenge_id": 1637685591454723, "challenge_name": "经典老片", "sum_count": 3},
    {"challenge_id": 1620340433234951, "challenge_name": "王建国", "sum_count": 3},
    {"challenge_id": 1582385692618766, "challenge_name": "野生动物园", "sum_count": 3},
    {"challenge_id": 1570736842694657, "challenge_name": "记录", "sum_count": 3},
    {"challenge_id": 1599227065105421, "challenge_name": "渣男", "sum_count": 3},
    {
        "challenge_id": 1675455147962376,
        "challenge_name": "玩转高空项目体验人生百态",
        "sum_count": 3,
    },
    {"challenge_id": 1590992690950158, "challenge_name": "乔任梁", "sum_count": 3},
    {"challenge_id": 1585403364768782, "challenge_name": "鸡毛飞上天", "sum_count": 3},
    {"challenge_id": 1584362390054926, "challenge_name": "儿童编发", "sum_count": 3},
    {"challenge_id": 1579146837542925, "challenge_name": "大学生", "sum_count": 3},
    {"challenge_id": 1576702417874990, "challenge_name": "美好的时光", "sum_count": 3},
    {"challenge_id": 1589402991216679, "challenge_name": "程潇", "sum_count": 3},
    {"challenge_id": 1576576095368222, "challenge_name": "生活小妙招", "sum_count": 3},
    {"challenge_id": 1551595476259841, "challenge_name": "变装秀", "sum_count": 3},
    {"challenge_id": 1666634254425100, "challenge_name": "旅行推荐官", "sum_count": 3},
    {"challenge_id": 1616633407177731, "challenge_name": "阴阳眼", "sum_count": 3},
    {"challenge_id": 1557585849328657, "challenge_name": "套路", "sum_count": 3},
    {"challenge_id": 1571925490014210, "challenge_name": "黑科技", "sum_count": 3},
    {"challenge_id": 1578126651757581, "challenge_name": "老板", "sum_count": 3},
    {
        "challenge_id": 1651439527694340,
        "challenge_name": "农村搞笑段子",
        "sum_count": 3,
    },
    {"challenge_id": 1545151595126786, "challenge_name": "孩子", "sum_count": 3},
    {"challenge_id": 1567424286507009, "challenge_name": "编发", "sum_count": 3},
    {"challenge_id": 1575941369489422, "challenge_name": "怀旧", "sum_count": 3},
    {"challenge_id": 1587542278201390, "challenge_name": "田园生活", "sum_count": 3},
    {"challenge_id": 1581146385222669, "challenge_name": "王牌对王牌", "sum_count": 3},
    {"challenge_id": 1565468412200961, "challenge_name": "减压", "sum_count": 3},
    {"challenge_id": 1565186793875458, "challenge_name": "韩剧", "sum_count": 3},
    {"challenge_id": 1582676811571214, "challenge_name": "孙俪", "sum_count": 3},
    {
        "challenge_id": 1678047292370952,
        "challenge_name": "阿里发布第一台云电脑",
        "sum_count": 3,
    },
    {"challenge_id": 1594908298182660, "challenge_name": "农民工", "sum_count": 3},
    {
        "challenge_id": 1661558155028488,
        "challenge_name": "好客山东抖好看",
        "sum_count": 3,
    },
    {"challenge_id": 1579247660891149, "challenge_name": "坐月子", "sum_count": 3},
    {"challenge_id": 1673442314166285, "challenge_name": "泉城记忆", "sum_count": 3},
    {"challenge_id": 1574772724653069, "challenge_name": "物业", "sum_count": 3},
    {"challenge_id": 1649435900140547, "challenge_name": "作品推广", "sum_count": 3},
    {"challenge_id": 1566928670589953, "challenge_name": "霍元甲", "sum_count": 3},
    {
        "challenge_id": 1609560424568974,
        "challenge_name": "不用吐籽的石榴",
        "sum_count": 3,
    },
    {"challenge_id": 1597973993212931, "challenge_name": "历史", "sum_count": 3},
    {"challenge_id": 1553397192892417, "challenge_name": "家有萌宠", "sum_count": 3},
    {"challenge_id": 1567427437671425, "challenge_name": "幼儿园", "sum_count": 3},
    {"challenge_id": 1577223533726766, "challenge_name": "生孩子", "sum_count": 3},
    {"challenge_id": 1580047189429262, "challenge_name": "二胎", "sum_count": 3},
    {"challenge_id": 1615501507483661, "challenge_name": "盗墓电影", "sum_count": 3},
    {"challenge_id": 1572072504832002, "challenge_name": "王一博", "sum_count": 3},
    {
        "challenge_id": 1677440120179724,
        "challenge_name": "送货的我也愿意😂",
        "sum_count": 3,
    },
    {"challenge_id": 1591891081240580, "challenge_name": "苹果原相机", "sum_count": 3},
    {"challenge_id": 1571096271169538, "challenge_name": "邓超", "sum_count": 3},
    {"challenge_id": 1553860795286529, "challenge_name": "成长", "sum_count": 3},
    {"challenge_id": 1590450156472327, "challenge_name": "道德绑架", "sum_count": 3},
    {"challenge_id": 1583771213381662, "challenge_name": "经典影视", "sum_count": 3},
    {
        "challenge_id": 1677351766142989,
        "challenge_name": "我的锦鲤男友",
        "sum_count": 3,
    },
    {"challenge_id": 1586582391854093, "challenge_name": "这就是街舞", "sum_count": 3},
    {"challenge_id": 1581501138513934, "challenge_name": "伤感音乐", "sum_count": 3},
    {"challenge_id": 1674713473445891, "challenge_name": "全民打卡季", "sum_count": 3},
    {"challenge_id": 1662687262343175, "challenge_name": "民法典", "sum_count": 3},
    {"challenge_id": 1571895601594370, "challenge_name": "大叔", "sum_count": 3},
    {"challenge_id": 1566529870264322, "challenge_name": "美国", "sum_count": 3},
    {"challenge_id": 1565825627366401, "challenge_name": "家常菜", "sum_count": 3},
    {"challenge_id": 1563863281278978, "challenge_name": "英雄联盟", "sum_count": 3},
    {"challenge_id": 1570360704824322, "challenge_name": "女儿", "sum_count": 3},
    {"challenge_id": 1594373252400148, "challenge_name": "中戏", "sum_count": 3},
    {"challenge_id": 1553304054213633, "challenge_name": "猫", "sum_count": 3},
    {"challenge_id": 1561844906486785, "challenge_name": "美女", "sum_count": 3},
    {"challenge_id": 1578950571088909, "challenge_name": "董子健", "sum_count": 3},
    {
        "challenge_id": 1664950793195524,
        "challenge_name": "团团语音导航",
        "sum_count": 3,
    },
    {"challenge_id": 1570000413299713, "challenge_name": "礼物", "sum_count": 3},
    {"challenge_id": 1663067709283341, "challenge_name": "我喜欢你", "sum_count": 3},
    {"challenge_id": 1591644316070916, "challenge_name": "包青天", "sum_count": 2},
    {
        "challenge_id": 1614297811041293,
        "challenge_name": "电视剧小欢喜",
        "sum_count": 2,
    },
    {"challenge_id": 1582235018941518, "challenge_name": "朱丹", "sum_count": 2},
    {"challenge_id": 1583749764243469, "challenge_name": "刀", "sum_count": 2},
    {
        "challenge_id": 1676530352577548,
        "challenge_name": "东方卫视平凡的荣耀",
        "sum_count": 2,
    },
    {"challenge_id": 1586667778637901, "challenge_name": "江西", "sum_count": 2},
    {"challenge_id": 1598717193600007, "challenge_name": "真实事件", "sum_count": 2},
    {
        "challenge_id": 1677263791052808,
        "challenge_name": "璇玑找司凤求复合",
        "sum_count": 2,
    },
    {"challenge_id": 1607154288770052, "challenge_name": "张绍刚", "sum_count": 2},
    {"challenge_id": 1610114587816973, "challenge_name": "奇闻奇事", "sum_count": 2},
    {"challenge_id": 1590182878486535, "challenge_name": "小学数学", "sum_count": 2},
    {"challenge_id": 1579311788421149, "challenge_name": "经典音乐", "sum_count": 2},
    {"challenge_id": 1580022313783310, "challenge_name": "十一假期", "sum_count": 2},
    {
        "challenge_id": 1675005069415436,
        "challenge_name": "迪丽热巴道歉",
        "sum_count": 2,
    },
    {"challenge_id": 1568981957226497, "challenge_name": "情感故事", "sum_count": 2},
    {"challenge_id": 1611930487833604, "challenge_name": "买房买房", "sum_count": 2},
    {"challenge_id": 1587202046925854, "challenge_name": "河北", "sum_count": 2},
    {"challenge_id": 1601536748188749, "challenge_name": "追球", "sum_count": 2},
    {"challenge_id": 1622416428328974, "challenge_name": "蚝油", "sum_count": 2},
    {"challenge_id": 1637605463572493, "challenge_name": "影视解说", "sum_count": 2},
    {"challenge_id": 1589474389544989, "challenge_name": "60后", "sum_count": 2},
    {"challenge_id": 1582585381013517, "challenge_name": "罗晋", "sum_count": 2},
    {"challenge_id": 1571191521910786, "challenge_name": "猫奴", "sum_count": 2},
    {"challenge_id": 1633704969131020, "challenge_name": "遇见宁夏", "sum_count": 2},
    {"challenge_id": 1641501652249604, "challenge_name": "天地粮人", "sum_count": 2},
    {"challenge_id": 1556285654828034, "challenge_name": "扑克牌", "sum_count": 2},
    {"challenge_id": 1640809778706440, "challenge_name": "五胎辣妈", "sum_count": 2},
    {"challenge_id": 1590393751744520, "challenge_name": "信条", "sum_count": 2},
    {
        "challenge_id": 1581692023806989,
        "challenge_name": "永不失联的爱",
        "sum_count": 2,
    },
    {"challenge_id": 1672646229830670, "challenge_name": "重庆dou知道", "sum_count": 2},
    {
        "challenge_id": 1617432134196228,
        "challenge_name": "我要上热门🔥",
        "sum_count": 2,
    },
    {"challenge_id": 1617739056452615, "challenge_name": "你们的威弟", "sum_count": 2},
    {"challenge_id": 1608291834750979, "challenge_name": "彩云伴海鸥", "sum_count": 2},
    {"challenge_id": 1558374815785985, "challenge_name": "喝酒", "sum_count": 2},
    {"challenge_id": 1569061348137986, "challenge_name": "西藏", "sum_count": 1},
    {"challenge_id": 1675537417626628, "challenge_name": "开学啦别慌", "sum_count": 2},
    {"challenge_id": 1581862373582862, "challenge_name": "吴孟达", "sum_count": 2},
    {"challenge_id": 1571889316651009, "challenge_name": "孙红雷", "sum_count": 2},
    {"challenge_id": 1554127075368961, "challenge_name": "皮卡丘", "sum_count": 2},
    {
        "challenge_id": 1677893621873725,
        "challenge_name": "花臂男子当街殴打79岁老人",
        "sum_count": 2,
    },
    {"challenge_id": 1580711791193101, "challenge_name": "我是传奇", "sum_count": 2},
    {
        "challenge_id": 1667384807579652,
        "challenge_name": "你的万水千山",
        "sum_count": 2,
    },
    {"challenge_id": 1601896267021486, "challenge_name": "威尔史密斯", "sum_count": 2},
    {"challenge_id": 1654508161036295, "challenge_name": "招生简章", "sum_count": 2},
    {"challenge_id": 1578509951450158, "challenge_name": "父母", "sum_count": 2},
    {
        "challenge_id": 1602408699796493,
        "challenge_name": "滴水之恩涌泉相报",
        "sum_count": 2,
    },
    {"challenge_id": 1572904279409678, "challenge_name": "改编歌曲", "sum_count": 2},
    {"challenge_id": 1582879197433870, "challenge_name": "中国好老公", "sum_count": 2},
    {"challenge_id": 1574956422945805, "challenge_name": "心动", "sum_count": 2},
    {"challenge_id": 1606497872079902, "challenge_name": "港星女神", "sum_count": 2},
    {"challenge_id": 1640215656864779, "challenge_name": "传梯正能量", "sum_count": 2},
    {"challenge_id": 1605776302466052, "challenge_name": "网红徐州", "sum_count": 2},
    {"challenge_id": 1588942964796430, "challenge_name": "西瓜视频", "sum_count": 2},
    {"challenge_id": 1591025759996941, "challenge_name": "女性", "sum_count": 2},
    {"challenge_id": 1621340956493838, "challenge_name": "椎间盘", "sum_count": 2},
    {"challenge_id": 1572075893470210, "challenge_name": "象棋", "sum_count": 2},
    {"challenge_id": 1574264484665358, "challenge_name": "雷佳音", "sum_count": 2},
    {"challenge_id": 1568523752589314, "challenge_name": "扎心", "sum_count": 2},
    {"challenge_id": 1559860925238274, "challenge_name": "武术", "sum_count": 2},
    {"challenge_id": 1600584600054814, "challenge_name": "西虹市首富", "sum_count": 2},
    {
        "challenge_id": 1575534280340509,
        "challenge_name": "看一遍笑一遍",
        "sum_count": 2,
    },
    {"challenge_id": 1604054059415559, "challenge_name": "腿麻", "sum_count": 2},
    {"challenge_id": 1674639431189507, "challenge_name": "守护青少年", "sum_count": 2},
    {
        "challenge_id": 1677521047328840,
        "challenge_name": "相亲当天车内发生关系",
        "sum_count": 2,
    },
    {"challenge_id": 1671155509981192, "challenge_name": "浩信传媒", "sum_count": 2},
    {
        "challenge_id": 1677341590630413,
        "challenge_name": "吴恪之变脸艺术家",
        "sum_count": 2,
    },
    {"challenge_id": 1583313233530957, "challenge_name": "拍戏现场", "sum_count": 2},
    {
        "challenge_id": 1658670929627144,
        "challenge_name": "我在抖瓜做译制",
        "sum_count": 2,
    },
    {"challenge_id": 1632875697093635, "challenge_name": "敖丙", "sum_count": 2},
    {"challenge_id": 1601790300488708, "challenge_name": "包拯", "sum_count": 2},
    {"challenge_id": 1605052084282376, "challenge_name": "教师节贺卡", "sum_count": 2},
    {"challenge_id": 1575971706049550, "challenge_name": "钓鱼人", "sum_count": 2},
    {"challenge_id": 1579507629054990, "challenge_name": "国庆", "sum_count": 2},
    {"challenge_id": 1576690334190638, "challenge_name": "裴佳欣", "sum_count": 2},
    {
        "challenge_id": 1601909620445191,
        "challenge_name": "跟陌生人打招呼",
        "sum_count": 2,
    },
    {"challenge_id": 1581690185732109, "challenge_name": "地铁偶遇", "sum_count": 2},
    {
        "challenge_id": 1602107966288909,
        "challenge_name": "万物皆有灵性",
        "sum_count": 2,
    },
    {"challenge_id": 1583054042918925, "challenge_name": "餐饮", "sum_count": 2},
    {
        "challenge_id": 1670463348010013,
        "challenge_name": "辽西治成烧烤",
        "sum_count": 2,
    },
    {"challenge_id": 1552966424338433, "challenge_name": "古风", "sum_count": 2},
    {"challenge_id": 1577880441262110, "challenge_name": "教师节礼物", "sum_count": 2},
    {"challenge_id": 1593269049353293, "challenge_name": "杨子姗", "sum_count": 2},
    {"challenge_id": 1572096139133953, "challenge_name": "追剧", "sum_count": 2},
    {"challenge_id": 1581693466694669, "challenge_name": "蒙面舞王", "sum_count": 2},
    {"challenge_id": 1567798553605122, "challenge_name": "台球", "sum_count": 2},
    {
        "challenge_id": 1674090025615432,
        "challenge_name": "旅行追梦计划",
        "sum_count": 2,
    },
    {"challenge_id": 1612187510941699, "challenge_name": "生育", "sum_count": 2},
    {"challenge_id": 1593729949884429, "challenge_name": "清华", "sum_count": 2},
    {"challenge_id": 1576070090835982, "challenge_name": "黄磊", "sum_count": 2},
    {"challenge_id": 1658812864946187, "challenge_name": "直呼内行", "sum_count": 2},
    {"challenge_id": 1583129549389981, "challenge_name": "蹭饭", "sum_count": 2},
    {"challenge_id": 1574635058297869, "challenge_name": "外卖小哥", "sum_count": 2},
    {"challenge_id": 1584065320089678, "challenge_name": "非你莫属", "sum_count": 2},
    {"challenge_id": 1566807698522114, "challenge_name": "jk制服", "sum_count": 2},
    {"challenge_id": 1572531526162445, "challenge_name": "智商", "sum_count": 2},
    {"challenge_id": 1608704372003843, "challenge_name": "买房须知", "sum_count": 2},
    {"challenge_id": 1576637219568653, "challenge_name": "福建莆田", "sum_count": 2},
    {"challenge_id": 1576495348847629, "challenge_name": "聊斋", "sum_count": 2},
    {"challenge_id": 1570741898999810, "challenge_name": "小伙", "sum_count": 2},
    {
        "challenge_id": 1676917079425031,
        "challenge_name": "更便宜的滴滴来了",
        "sum_count": 2,
    },
    {"challenge_id": 1578157003244557, "challenge_name": "毛晓彤", "sum_count": 2},
    {
        "challenge_id": 1677251979760648,
        "challenge_name": "痞幼模仿挑战赛",
        "sum_count": 2,
    },
    {"challenge_id": 1555768365932546, "challenge_name": "印度", "sum_count": 2},
    {"challenge_id": 1574994596912142, "challenge_name": "贴心小棉袄", "sum_count": 2},
    {"challenge_id": 1637754763086862, "challenge_name": "大案纪实", "sum_count": 2},
    {
        "challenge_id": 1588732657158158,
        "challenge_name": "一起长大的幸福",
        "sum_count": 2,
    },
    {"challenge_id": 1568245275985921, "challenge_name": "鱼", "sum_count": 2},
    {"challenge_id": 1608859554814980, "challenge_name": "买房攻略", "sum_count": 2},
    {"challenge_id": 1580931893523469, "challenge_name": "老婆大人", "sum_count": 2},
    {"challenge_id": 1579020642148366, "challenge_name": "安徽合肥", "sum_count": 2},
    {"challenge_id": 1563042134329345, "challenge_name": "经典", "sum_count": 2},
    {"challenge_id": 1600611859876868, "challenge_name": "我和两个他", "sum_count": 2},
    {"challenge_id": 1618030463323204, "challenge_name": "家居好物", "sum_count": 2},
    {"challenge_id": 1579685317523566, "challenge_name": "德宏", "sum_count": 2},
    {"challenge_id": 1571646279999490, "challenge_name": "吐槽", "sum_count": 2},
    {"challenge_id": 1604722952975363, "challenge_name": "空军", "sum_count": 2},
    {"challenge_id": 1634138058164228, "challenge_name": "dou是知识点", "sum_count": 2},
    {"challenge_id": 1573858621228045, "challenge_name": "螺蛳粉", "sum_count": 2},
    {
        "challenge_id": 1625534930761741,
        "challenge_name": "没有腿的摔跤手",
        "sum_count": 2,
    },
    {
        "challenge_id": 1663686229482503,
        "challenge_name": "传承竹编手艺",
        "sum_count": 2,
    },
    {"challenge_id": 1595294227506189, "challenge_name": "足球解说", "sum_count": 2},
    {"challenge_id": 1562354857442306, "challenge_name": "时尚", "sum_count": 2},
    {
        "challenge_id": 1669741130846215,
        "challenge_name": "24节气云游记",
        "sum_count": 2,
    },
    {"challenge_id": 1611820949641235, "challenge_name": "翻车鱼", "sum_count": 2},
    {"challenge_id": 1571717080811522, "challenge_name": "奶爸", "sum_count": 2},
    {"challenge_id": 1565189230141441, "challenge_name": "还珠格格", "sum_count": 2},
    {"challenge_id": 1626543680052235, "challenge_name": "乡村那点事", "sum_count": 2},
    {"challenge_id": 1562210901169154, "challenge_name": "马云", "sum_count": 2},
    {"challenge_id": 1616897671785486, "challenge_name": "武僧一龙", "sum_count": 2},
    {"challenge_id": 1641363752446990, "challenge_name": "我是特种兵", "sum_count": 2},
    {"challenge_id": 1583516629506061, "challenge_name": "视频剪辑", "sum_count": 2},
    {"challenge_id": 1604715782568968, "challenge_name": "情感剧情", "sum_count": 2},
    {"challenge_id": 1568564035453969, "challenge_name": "田径", "sum_count": 2},
    {"challenge_id": 1617758179648519, "challenge_name": "火焰蓝", "sum_count": 2},
    {"challenge_id": 1655877838244871, "challenge_name": "分什么分", "sum_count": 2},
    {"challenge_id": 1580138377443342, "challenge_name": "撑杆跳", "sum_count": 2},
    {"challenge_id": 1667905372641294, "challenge_name": "痞幼", "sum_count": 2},
    {"challenge_id": 1573330966309901, "challenge_name": "贾乃亮", "sum_count": 2},
    {"challenge_id": 1672631315596301, "challenge_name": "异星灾变", "sum_count": 2},
    {"challenge_id": 1641283734162443, "challenge_name": "聚焰斗地主", "sum_count": 2},
    {"challenge_id": 1575549510013966, "challenge_name": "医院", "sum_count": 2},
    {"challenge_id": 1596741704773636, "challenge_name": "法律", "sum_count": 2},
    {"challenge_id": 1676164716754948, "challenge_name": "科技新生代", "sum_count": 2},
    {"challenge_id": 1589112983224324, "challenge_name": "渣女", "sum_count": 2},
    {
        "challenge_id": 1617182556891139,
        "challenge_name": "美国加州山火",
        "sum_count": 2,
    },
    {"challenge_id": 1567004204402690, "challenge_name": "柴犬", "sum_count": 2},
    {"challenge_id": 1584040569982989, "challenge_name": "姥爷带娃", "sum_count": 2},
    {"challenge_id": 1562893774944274, "challenge_name": "戏曲", "sum_count": 2},
    {"challenge_id": 1565918222590978, "challenge_name": "创意", "sum_count": 2},
    {
        "challenge_id": 1601266414030851,
        "challenge_name": "成年人的崩溃",
        "sum_count": 2,
    },
    {
        "challenge_id": 1626959359107080,
        "challenge_name": "沙雕室友之沙雕日常",
        "sum_count": 2,
    },
    {"challenge_id": 1575166514184205, "challenge_name": "演绎", "sum_count": 2},
    {"challenge_id": 1585137221664782, "challenge_name": "中国好媳妇", "sum_count": 2},
    {"challenge_id": 1562033822849025, "challenge_name": "中国好声音", "sum_count": 2},
    {"challenge_id": 1583145146753165, "challenge_name": "沈腾", "sum_count": 2},
    {"challenge_id": 1587870988100638, "challenge_name": "搞笑剧情", "sum_count": 2},
    {"challenge_id": 1627528751569934, "challenge_name": "封校", "sum_count": 2},
    {"challenge_id": 1586554660079630, "challenge_name": "证件照", "sum_count": 2},
    {"challenge_id": 1619626570050563, "challenge_name": "服装厂", "sum_count": 2},
    {"challenge_id": 1590984878758920, "challenge_name": "美女老板", "sum_count": 2},
    {"challenge_id": 1578792882862093, "challenge_name": "关于爱情", "sum_count": 2},
    {"challenge_id": 1599643530364942, "challenge_name": "知识", "sum_count": 2},
    {"challenge_id": 1622087326552068, "challenge_name": "交警", "sum_count": 2},
    {"challenge_id": 1610709203972104, "challenge_name": "代驾", "sum_count": 2},
    {"challenge_id": 1598148381889608, "challenge_name": "钱小豪", "sum_count": 2},
    {"challenge_id": 1563580466123777, "challenge_name": "自驾游", "sum_count": 2},
    {"challenge_id": 1595888886833223, "challenge_name": "瘦肉丸", "sum_count": 2},
    {"challenge_id": 1592750572997640, "challenge_name": "林子祥", "sum_count": 2},
    {"challenge_id": 1597885178880036, "challenge_name": "居家运动", "sum_count": 2},
    {"challenge_id": 1568352984860673, "challenge_name": "白敬亭", "sum_count": 2},
    {"challenge_id": 1602864865491022, "challenge_name": "雍正王朝", "sum_count": 2},
    {
        "challenge_id": 1660115610890263,
        "challenge_name": "假功夫对付真功夫",
        "sum_count": 2,
    },
    {"challenge_id": 1565084692681746, "challenge_name": "奇葩", "sum_count": 2},
    {"challenge_id": 1567909053955073, "challenge_name": "打铁", "sum_count": 2},
    {"challenge_id": 1576339711622190, "challenge_name": "拍摄", "sum_count": 2},
    {
        "challenge_id": 1598406367291405,
        "challenge_name": "遵守交通规则",
        "sum_count": 2,
    },
    {"challenge_id": 1621468558942221, "challenge_name": "苏浙皖", "sum_count": 2},
    {"challenge_id": 1635668202108941, "challenge_name": "高考", "sum_count": 2},
    {
        "challenge_id": 1677897715587165,
        "challenge_name": "女子车流中闯红灯还殴打交警",
        "sum_count": 2,
    },
    {"challenge_id": 1653337287206920, "challenge_name": "不说诳语", "sum_count": 2},
    {"challenge_id": 1628253640347656, "challenge_name": "颈椎按摩", "sum_count": 2},
    {"challenge_id": 1584396167052301, "challenge_name": "女明星", "sum_count": 2},
    {
        "challenge_id": 1573086960541709,
        "challenge_name": "把生活拍成电影",
        "sum_count": 2,
    },
    {"challenge_id": 1597173718493191, "challenge_name": "嵩口古镇", "sum_count": 2},
    {"challenge_id": 1573705735908366, "challenge_name": "济南", "sum_count": 2},
    {"challenge_id": 1564100161227777, "challenge_name": "黄轩", "sum_count": 2},
    {
        "challenge_id": 1675884218130440,
        "challenge_name": "双栖达人计划",
        "sum_count": 2,
    },
    {"challenge_id": 1611455501664333, "challenge_name": "爱热", "sum_count": 2},
    {"challenge_id": 1576330283741198, "challenge_name": "离婚", "sum_count": 2},
    {"challenge_id": 1591611391058948, "challenge_name": "佟大为", "sum_count": 2},
    {"challenge_id": 1613293052526755, "challenge_name": "抖in北京", "sum_count": 2},
    {"challenge_id": 1571909336862722, "challenge_name": "结局亮了", "sum_count": 2},
    {
        "challenge_id": 1584048835848206,
        "challenge_name": "一定要看到最后",
        "sum_count": 2,
    },
    {
        "challenge_id": 1642847597651981,
        "challenge_name": "华为开发者大会",
        "sum_count": 2,
    },
    {
        "challenge_id": 1650536545036296,
        "challenge_name": "买房避坑指南",
        "sum_count": 2,
    },
    {
        "challenge_id": 1673692685016072,
        "challenge_name": "电影蓝色防线",
        "sum_count": 2,
    },
    {"challenge_id": 1619438682233870, "challenge_name": "吴老狗", "sum_count": 2},
    {"challenge_id": 1563895801416706, "challenge_name": "柯基", "sum_count": 2},
    {"challenge_id": 1573414582385694, "challenge_name": "动物成精", "sum_count": 2},
    {"challenge_id": 1646895381684236, "challenge_name": "追剧在抖瓜", "sum_count": 2},
    {"challenge_id": 1565750948717570, "challenge_name": "解压", "sum_count": 2},
    {"challenge_id": 1579149139216414, "challenge_name": "寂静岭", "sum_count": 2},
    {"challenge_id": 1642269686463495, "challenge_name": "赵本山", "sum_count": 2},
    {"challenge_id": 1597542986508295, "challenge_name": "生活助手", "sum_count": 2},
    {"challenge_id": 1578595416604685, "challenge_name": "爸爸带娃", "sum_count": 2},
    {"challenge_id": 1671551982966792, "challenge_name": "dou出真功夫", "sum_count": 2},
    {"challenge_id": 1595277826301965, "challenge_name": "连锁反应", "sum_count": 2},
    {"challenge_id": 1590442578324483, "challenge_name": "权威发布", "sum_count": 2},
    {"challenge_id": 1642724218652680, "challenge_name": "玩世英雄", "sum_count": 2},
    {"challenge_id": 1626606400241678, "challenge_name": "超快顺产", "sum_count": 2},
    {"challenge_id": 1561270570572801, "challenge_name": "发型", "sum_count": 2},
    {"challenge_id": 1675448178555917, "challenge_name": "浩老施", "sum_count": 2},
    {"challenge_id": 1550710535550978, "challenge_name": "表情包", "sum_count": 2},
    {
        "challenge_id": 1618436555025421,
        "challenge_name": "男人之间的快乐",
        "sum_count": 2,
    },
    {"challenge_id": 1583469586400270, "challenge_name": "韩国人", "sum_count": 2},
    {"challenge_id": 1617736854296589, "challenge_name": "dou热门", "sum_count": 2},
    {"challenge_id": 1617569051727875, "challenge_name": "烧烤美食", "sum_count": 2},
    {"challenge_id": 1587130198580237, "challenge_name": "短剧", "sum_count": 2},
    {"challenge_id": 1601947141010439, "challenge_name": "隋唐英雄传", "sum_count": 2},
    {"challenge_id": 1580679949528078, "challenge_name": "等着我", "sum_count": 2},
    {"challenge_id": 1552394635036673, "challenge_name": "王者荣耀", "sum_count": 2},
    {"challenge_id": 1592825855446024, "challenge_name": "云南瑞丽", "sum_count": 2},
    {"challenge_id": 1651917608470536, "challenge_name": "抖加小助手", "sum_count": 2},
    {"challenge_id": 1651738624036871, "challenge_name": "吴邪", "sum_count": 2},
    {
        "challenge_id": 1632510395676683,
        "challenge_name": "房产知识分享",
        "sum_count": 2,
    },
    {
        "challenge_id": 1667538098886667,
        "challenge_name": "新的故事来了",
        "sum_count": 2,
    },
    {"challenge_id": 1597789503612942, "challenge_name": "文物", "sum_count": 2},
    {"challenge_id": 1591077382327310, "challenge_name": "社交网络", "sum_count": 2},
    {"challenge_id": 1614924225708045, "challenge_name": "直播人", "sum_count": 2},
    {
        "challenge_id": 1655149178030094,
        "challenge_name": "抖瓜电影推荐",
        "sum_count": 2,
    },
    {"challenge_id": 1581759890520078, "challenge_name": "江疏影", "sum_count": 2},
    {"challenge_id": 1592993846703112, "challenge_name": "万茜", "sum_count": 2},
    {
        "challenge_id": 1649337453485069,
        "challenge_name": "守规则平安行",
        "sum_count": 2,
    },
    {"challenge_id": 1564750480795649, "challenge_name": "张靓颖", "sum_count": 2},
    {"challenge_id": 1610364882236430, "challenge_name": "工作vlog", "sum_count": 2},
    {"challenge_id": 1676604696082440, "challenge_name": "胖艺人", "sum_count": 2},
    {"challenge_id": 1590771506270221, "challenge_name": "李元霸", "sum_count": 2},
    {"challenge_id": 1611040951561224, "challenge_name": "趁饭", "sum_count": 2},
    {"challenge_id": 1676458965522445, "challenge_name": "电影大章鱼", "sum_count": 2},
    {"challenge_id": 1619096683326478, "challenge_name": "tvb剧集", "sum_count": 2},
    {
        "challenge_id": 1638671939469324,
        "challenge_name": "我在抖瓜看动漫",
        "sum_count": 2,
    },
    {"challenge_id": 1619886017736711, "challenge_name": "骗局", "sum_count": 2},
    {"challenge_id": 1577592667047949, "challenge_name": "毛绒玩具", "sum_count": 2},
    {"challenge_id": 1671619340987399, "challenge_name": "非日常派对", "sum_count": 2},
    {"challenge_id": 1600727846838343, "challenge_name": "盐城大丰", "sum_count": 2},
    {"challenge_id": 1588822635415559, "challenge_name": "张子枫", "sum_count": 2},
    {"challenge_id": 1571452957715522, "challenge_name": "脸大", "sum_count": 2},
    {
        "challenge_id": 1607386609495053,
        "challenge_name": "河北外国语学院",
        "sum_count": 2,
    },
    {
        "challenge_id": 1676091624898568,
        "challenge_name": "脆香米酸奶巧克力杯",
        "sum_count": 2,
    },
    {"challenge_id": 1656120154182660, "challenge_name": "鼠你最准", "sum_count": 2},
    {"challenge_id": 1591663622264839, "challenge_name": "我是山姆", "sum_count": 2},
    {"challenge_id": 1575493808954445, "challenge_name": "地摊", "sum_count": 2},
    {"challenge_id": 1573154281708558, "challenge_name": "周冬雨", "sum_count": 2},
    {"challenge_id": 1557319370707969, "challenge_name": "春晚", "sum_count": 2},
    {"challenge_id": 1640842497094659, "challenge_name": "衣哥", "sum_count": 2},
    {"challenge_id": 1605444807219203, "challenge_name": "急救", "sum_count": 2},
    {
        "challenge_id": 1616004178918413,
        "challenge_name": "家有小棉袄真好",
        "sum_count": 2,
    },
    {"challenge_id": 1569334105806850, "challenge_name": "喜剧", "sum_count": 2},
    {"challenge_id": 1615081630603271, "challenge_name": "我是律师", "sum_count": 2},
    {"challenge_id": 1559842263609346, "challenge_name": "亲情", "sum_count": 2},
    {"challenge_id": 1548957900724225, "challenge_name": "仓鼠", "sum_count": 2},
    {
        "challenge_id": 1617191634896919,
        "challenge_name": "感谢抖瓜感谢官方",
        "sum_count": 2,
    },
    {"challenge_id": 1586860564717614, "challenge_name": "小欢喜", "sum_count": 2},
    {"challenge_id": 1563030181222402, "challenge_name": "纹身", "sum_count": 2},
    {"challenge_id": 1558109893253122, "challenge_name": "宋丹丹", "sum_count": 2},
    {"challenge_id": 1607836965250051, "challenge_name": "rc遥控车", "sum_count": 2},
    {"challenge_id": 1606169347293187, "challenge_name": "兰研", "sum_count": 2},
    {"challenge_id": 1606606487694343, "challenge_name": "钱嘉乐", "sum_count": 2},
    {
        "challenge_id": 1648835474113550,
        "challenge_name": "原创影视剪辑",
        "sum_count": 2,
    },
    {"challenge_id": 1580102892392461, "challenge_name": "梵净山", "sum_count": 2},
    {"challenge_id": 1677221543505932, "challenge_name": "宏楠事件", "sum_count": 2},
    {"challenge_id": 1579968822674445, "challenge_name": "男闺蜜", "sum_count": 2},
    {"challenge_id": 1607930427379716, "challenge_name": "翻斗自卸车", "sum_count": 2},
    {
        "challenge_id": 1677321397592068,
        "challenge_name": "包青天之诡mu空guan",
        "sum_count": 2,
    },
    {"challenge_id": 1572274335765517, "challenge_name": "陈赫", "sum_count": 2},
    {
        "challenge_id": 1630055570050052,
        "challenge_name": "兰蔻菁纯面霜",
        "sum_count": 2,
    },
    {"challenge_id": 1674080222154829, "challenge_name": "抖瓜选修课", "sum_count": 2},
    {"challenge_id": 1586590537665550, "challenge_name": "家庭教育", "sum_count": 2},
    {"challenge_id": 1620162360665096, "challenge_name": "刘銮雄", "sum_count": 2},
    {"challenge_id": 1662671810440206, "challenge_name": "冰葬", "sum_count": 2},
    {"challenge_id": 1613034964984836, "challenge_name": "铤而走险", "sum_count": 2},
    {"challenge_id": 1642313980170254, "challenge_name": "卡点", "sum_count": 2},
    {"challenge_id": 1578523164433422, "challenge_name": "治愈", "sum_count": 2},
    {"challenge_id": 1563279215181825, "challenge_name": "原创", "sum_count": 2},
    {"challenge_id": 1581932821007374, "challenge_name": "采耳", "sum_count": 2},
    {"challenge_id": 1563220812237825, "challenge_name": "动物世界", "sum_count": 2},
    {"challenge_id": 1676869699233805, "challenge_name": "陪伴秀出圈", "sum_count": 2},
    {"challenge_id": 1610513349656580, "challenge_name": "碰撞测试", "sum_count": 2},
    {"challenge_id": 1676872600418312, "challenge_name": "王大骁", "sum_count": 2},
    {"challenge_id": 1579522462927886, "challenge_name": "王祖贤", "sum_count": 2},
    {"challenge_id": 1594750868284419, "challenge_name": "感人瞬间", "sum_count": 2},
    {"challenge_id": 1563765391924226, "challenge_name": "胡歌", "sum_count": 2},
    {"challenge_id": 1570531698754562, "challenge_name": "李嘉欣", "sum_count": 2},
    {"challenge_id": 1633053651454980, "challenge_name": "朱晓东", "sum_count": 2},
    {"challenge_id": 1677636428509197, "challenge_name": "李心艾胖了", "sum_count": 2},
    {"challenge_id": 1611139232130071, "challenge_name": "美食vlog", "sum_count": 2},
    {
        "challenge_id": 1615935937555470,
        "challenge_name": "男人简单的快乐",
        "sum_count": 2,
    },
    {"challenge_id": 1598023398156301, "challenge_name": "官方", "sum_count": 2},
    {"challenge_id": 1609650180186125, "challenge_name": "我们开学了", "sum_count": 2},
    {"challenge_id": 1602585044127758, "challenge_name": "卸货倒计时", "sum_count": 2},
    {"challenge_id": 1619351104729095, "challenge_name": "乐队的夏天", "sum_count": 2},
    {
        "challenge_id": 1617710029065223,
        "challenge_name": "抖瓜原创动漫",
        "sum_count": 2,
    },
    {"challenge_id": 1632778772472836, "challenge_name": "儿童拖拉机", "sum_count": 2},
    {"challenge_id": 1673636915873799, "challenge_name": "迷人歪嘴笑", "sum_count": 2},
    {"challenge_id": 1593347062277124, "challenge_name": "厚颜无耻", "sum_count": 2},
    {"challenge_id": 1674796132431885, "challenge_name": "dou是剧中人", "sum_count": 2},
    {"challenge_id": 1573406396642318, "challenge_name": "打工", "sum_count": 2},
    {"challenge_id": 1638870286689293, "challenge_name": "鉴别绿茶", "sum_count": 2},
    {"challenge_id": 1595176231949326, "challenge_name": "啤酒盖", "sum_count": 2},
    {"challenge_id": 1584565435287629, "challenge_name": "年代", "sum_count": 2},
    {"challenge_id": 1568530881718274, "challenge_name": "猴", "sum_count": 2},
    {"challenge_id": 1587302644756493, "challenge_name": "好剧推荐", "sum_count": 2},
    {"challenge_id": 1582150163905550, "challenge_name": "拐卖儿童", "sum_count": 2},
    {"challenge_id": 1661287383221255, "challenge_name": "境外输入", "sum_count": 2},
    {"challenge_id": 1568378458786817, "challenge_name": "健身打卡", "sum_count": 2},
    {"challenge_id": 1568746240399362, "challenge_name": "节日快乐", "sum_count": 2},
    {"challenge_id": 1590481131932679, "challenge_name": "张震", "sum_count": 2},
    {
        "challenge_id": 1672893751096392,
        "challenge_name": "我在抖瓜做法援",
        "sum_count": 2,
    },
    {
        "challenge_id": 1676012005695500,
        "challenge_name": "2020云栖大会",
        "sum_count": 2,
    },
    {"challenge_id": 1589099500196872, "challenge_name": "范伟", "sum_count": 2},
    {"challenge_id": 1564924483866625, "challenge_name": "情圣", "sum_count": 2},
    {"challenge_id": 1571638214070290, "challenge_name": "戚薇", "sum_count": 2},
    {
        "challenge_id": 1673511799227405,
        "challenge_name": "乘风破浪的创作者",
        "sum_count": 2,
    },
    {"challenge_id": 1593186548599821, "challenge_name": "高速免费", "sum_count": 2},
    {"challenge_id": 1670541164307528, "challenge_name": "探星计划", "sum_count": 2},
    {"challenge_id": 1630876528843783, "challenge_name": "海底捞人", "sum_count": 2},
    {"challenge_id": 1622096049215566, "challenge_name": "分手挽回", "sum_count": 2},
    {"challenge_id": 1568639778065409, "challenge_name": "晚霞", "sum_count": 2},
    {"challenge_id": 1588986128231428, "challenge_name": "恋爱日常", "sum_count": 2},
    {
        "challenge_id": 1674699436475405,
        "challenge_name": "养宠进阶指南",
        "sum_count": 2,
    },
    {"challenge_id": 1584402532989981, "challenge_name": "p图", "sum_count": 2},
    {"challenge_id": 1657869946179592, "challenge_name": "破播放", "sum_count": 2},
    {"challenge_id": 1593084814425102, "challenge_name": "坚强的女孩", "sum_count": 2},
    {"challenge_id": 1587595260923918, "challenge_name": "郭富城", "sum_count": 2},
    {"challenge_id": 1635949393197064, "challenge_name": "三十而已", "sum_count": 2},
    {"challenge_id": 1569011606745090, "challenge_name": "机械", "sum_count": 2},
    {"challenge_id": 1588198207058973, "challenge_name": "蝙蝠", "sum_count": 2},
    {
        "challenge_id": 1676096486772744,
        "challenge_name": "拍一拍你的老板",
        "sum_count": 2,
    },
    {
        "challenge_id": 1616642981576775,
        "challenge_name": "一个人带娃日常",
        "sum_count": 2,
    },
    {
        "challenge_id": 1677410181036046,
        "challenge_name": "世界奇妙物语之直播人",
        "sum_count": 2,
    },
    {
        "challenge_id": 1657773204340744,
        "challenge_name": "抖瓜独家剧场",
        "sum_count": 2,
    },
    {"challenge_id": 1593370125756419, "challenge_name": "花椒", "sum_count": 2},
    {"challenge_id": 1647818283848732, "challenge_name": "梅威瑟", "sum_count": 2},
    {
        "challenge_id": 1574609242915854,
        "challenge_name": "认真工作的样子",
        "sum_count": 2,
    },
    {"challenge_id": 1595874350380040, "challenge_name": "学习动力", "sum_count": 2},
    {"challenge_id": 1595150120417284, "challenge_name": "安徽阜阳", "sum_count": 2},
    {"challenge_id": 1587637314456589, "challenge_name": "街访", "sum_count": 2},
    {"challenge_id": 1631344876478472, "challenge_name": "悭钱家族", "sum_count": 2},
    {"challenge_id": 1577892690422798, "challenge_name": "鬼迷心窍", "sum_count": 2},
    {"challenge_id": 1581867193975821, "challenge_name": "黄金", "sum_count": 2},
    {"challenge_id": 1627999492333581, "challenge_name": "袁咏仪", "sum_count": 2},
    {"challenge_id": 1588465122940931, "challenge_name": "互相尊重", "sum_count": 2},
    {"challenge_id": 1591718018755597, "challenge_name": "皮孩子", "sum_count": 2},
    {"challenge_id": 1585101632983054, "challenge_name": "搞笑配音", "sum_count": 2},
    {
        "challenge_id": 1630876407192588,
        "challenge_name": "vlog我的日常",
        "sum_count": 2,
    },
    {"challenge_id": 1580329186494477, "challenge_name": "原创作品", "sum_count": 2},
    {"challenge_id": 1579007967564830, "challenge_name": "武功山", "sum_count": 2},
    {"challenge_id": 1570900575409153, "challenge_name": "小品", "sum_count": 2},
    {"challenge_id": 1656139448157192, "challenge_name": "duo热门", "sum_count": 2},
    {"challenge_id": 1595978532779021, "challenge_name": "尊龙", "sum_count": 2},
    {
        "challenge_id": 1570411407277057,
        "challenge_name": "北京电影学院",
        "sum_count": 2,
    },
    {"challenge_id": 1566448088605697, "challenge_name": "学生", "sum_count": 2},
    {"challenge_id": 1576873836254238, "challenge_name": "神秘", "sum_count": 2},
    {"challenge_id": 1584628720981006, "challenge_name": "孝敬父母", "sum_count": 2},
    {"challenge_id": 1597510608508936, "challenge_name": "徐克", "sum_count": 2},
    {"challenge_id": 1593902806435853, "challenge_name": "李心艾", "sum_count": 2},
    {"challenge_id": 1569706519097346, "challenge_name": "学生时代", "sum_count": 2},
    {"challenge_id": 1569817579072513, "challenge_name": "龙珠", "sum_count": 2},
    {"challenge_id": 1670559759481933, "challenge_name": "五菱凯捷", "sum_count": 2},
    {"challenge_id": 1600800922548231, "challenge_name": "詹姆斯", "sum_count": 2},
    {"challenge_id": 1597532820572173, "challenge_name": "沙雕系列", "sum_count": 2},
    {"challenge_id": 1616189545063431, "challenge_name": "身边的温暖", "sum_count": 2},
    {
        "challenge_id": 1677447975755779,
        "challenge_name": "蚝油里吃出一只蝙蝠",
        "sum_count": 2,
    },
    {
        "challenge_id": 1677419676002311,
        "challenge_name": "母婴大牌好物季",
        "sum_count": 2,
    },
    {
        "challenge_id": 1575772508217421,
        "challenge_name": "小朋友的世界",
        "sum_count": 2,
    },
    {"challenge_id": 1631029511157767, "challenge_name": "最热点", "sum_count": 2},
    {"challenge_id": 1578568241182734, "challenge_name": "玄幻", "sum_count": 2},
    {"challenge_id": 1573337231980557, "challenge_name": "饺子", "sum_count": 2},
    {"challenge_id": 1573101973747726, "challenge_name": "儿童玩具", "sum_count": 2},
    {"challenge_id": 1582227239664654, "challenge_name": "解密", "sum_count": 2},
    {"challenge_id": 1622702857134083, "challenge_name": "立耳茶杯犬", "sum_count": 2},
    {"challenge_id": 1618702325280771, "challenge_name": "二手车", "sum_count": 2},
    {"challenge_id": 1566358778950657, "challenge_name": "同学", "sum_count": 2},
    {"challenge_id": 1569172835602434, "challenge_name": "科技", "sum_count": 2},
    {"challenge_id": 1636464417131531, "challenge_name": "自定义抠图", "sum_count": 2},
    {"challenge_id": 1566102253026305, "challenge_name": "动物园", "sum_count": 2},
    {"challenge_id": 1588401792268291, "challenge_name": "陈坤", "sum_count": 2},
    {"challenge_id": 1621870876044295, "challenge_name": "多多神犬", "sum_count": 2},
    {"challenge_id": 1638301648043021, "challenge_name": "折叠桌子", "sum_count": 2},
    {"challenge_id": 1572978145886221, "challenge_name": "女生", "sum_count": 2},
    {"challenge_id": 1612239094260749, "challenge_name": "爱情终点站", "sum_count": 2},
    {"challenge_id": 1595843026462723, "challenge_name": "伏弟魔", "sum_count": 2},
    {"challenge_id": 1590491631612932, "challenge_name": "水形物语", "sum_count": 2},
    {"challenge_id": 1567281976049665, "challenge_name": "乐高", "sum_count": 2},
    {
        "challenge_id": 1606128066639885,
        "challenge_name": "热歌热门分享",
        "sum_count": 2,
    },
    {"challenge_id": 1676988296536075, "challenge_name": "浪漫的男孩", "sum_count": 2},
    {"challenge_id": 1663787524330504, "challenge_name": "未来恋人", "sum_count": 2},
    {"challenge_id": 1675890428469251, "challenge_name": "动漫映像", "sum_count": 2},
    {"challenge_id": 1570345614275585, "challenge_name": "王菲", "sum_count": 2},
    {"challenge_id": 1578677484425229, "challenge_name": "国粹", "sum_count": 2},
    {
        "challenge_id": 1583139574142990,
        "challenge_name": "很高兴认识你",
        "sum_count": 2,
    },
    {"challenge_id": 1574428393782285, "challenge_name": "惊险", "sum_count": 2},
    {
        "challenge_id": 1661234805791755,
        "challenge_name": "吃货的快乐回来了",
        "sum_count": 2,
    },
    {"challenge_id": 1553610645693441, "challenge_name": "恐怖", "sum_count": 2},
    {"challenge_id": 1565189842770945, "challenge_name": "千手观音", "sum_count": 2},
    {"challenge_id": 1582373725415437, "challenge_name": "职场", "sum_count": 2},
    {"challenge_id": 1677716148719630, "challenge_name": "356斤妻子", "sum_count": 2},
    {"challenge_id": 1608365240681486, "challenge_name": "催泪系", "sum_count": 2},
    {"challenge_id": 1642277503865863, "challenge_name": "好麦多麦片", "sum_count": 2},
    {
        "challenge_id": 1646999318491148,
        "challenge_name": "带你现场追综艺",
        "sum_count": 2,
    },
    {"challenge_id": 1582642225422365, "challenge_name": "林忆莲", "sum_count": 2},
    {"challenge_id": 1616375893356548, "challenge_name": "整蛊室友", "sum_count": 2},
    {"challenge_id": 1616756613451790, "challenge_name": "网络工程师", "sum_count": 2},
    {"challenge_id": 1602323108821006, "challenge_name": "大四学姐", "sum_count": 2},
    {"challenge_id": 1581330010555405, "challenge_name": "蒜头", "sum_count": 2},
    {"challenge_id": 1645553147086851, "challenge_name": "文案", "sum_count": 2},
    {
        "challenge_id": 1664668367586317,
        "challenge_name": "一盔一带dou知道",
        "sum_count": 2,
    },
    {"challenge_id": 1570347557767170, "challenge_name": "读书", "sum_count": 2},
    {"challenge_id": 1591663895502861, "challenge_name": "兄妹日常", "sum_count": 2},
    {"challenge_id": 1597841521326084, "challenge_name": "美业手艺人", "sum_count": 2},
    {
        "challenge_id": 1636846850118663,
        "challenge_name": "西瓜视频放映厅",
        "sum_count": 2,
    },
    {
        "challenge_id": 1674269604816908,
        "challenge_name": "吴医生的精致少女必修课",
        "sum_count": 2,
    },
    {"challenge_id": 1596553692287054, "challenge_name": "热水器", "sum_count": 2},
    {"challenge_id": 1658443349530632, "challenge_name": "天选爱情", "sum_count": 2},
    {"challenge_id": 1616267194550286, "challenge_name": "避坑指南", "sum_count": 2},
    {"challenge_id": 1577417012306958, "challenge_name": "家有萌猫", "sum_count": 2},
    {"challenge_id": 1564574074548225, "challenge_name": "网红", "sum_count": 2},
    {"challenge_id": 1565903826996226, "challenge_name": "大海", "sum_count": 2},
    {"challenge_id": 1634750150294532, "challenge_name": "草原舞曲", "sum_count": 2},
    {"challenge_id": 1563100423123969, "challenge_name": "灵魂歌手", "sum_count": 2},
    {"challenge_id": 1559841638160386, "challenge_name": "宠物", "sum_count": 2},
    {
        "challenge_id": 1589266575913991,
        "challenge_name": "能不能上个热门",
        "sum_count": 2,
    },
    {
        "challenge_id": 1668403088361486,
        "challenge_name": "史上最牛演唱会动用军队",
        "sum_count": 2,
    },
    {
        "challenge_id": 1672185821181966,
        "challenge_name": "做家务的男人第二季",
        "sum_count": 2,
    },
    {"challenge_id": 1606371753477191, "challenge_name": "乳腺", "sum_count": 2},
    {"challenge_id": 1600541324888142, "challenge_name": "战争", "sum_count": 2},
    {"challenge_id": 1674154988745732, "challenge_name": "甘肃观察", "sum_count": 2},
    {
        "challenge_id": 1665135551420430,
        "challenge_name": "足球解说员王涛",
        "sum_count": 2,
    },
    {
        "challenge_id": 1677684817899534,
        "challenge_name": "李心艾回应胖了",
        "sum_count": 2,
    },
    {"challenge_id": 1561165198884881, "challenge_name": "娱乐", "sum_count": 2},
    {
        "challenge_id": 1677498193907789,
        "challenge_name": "学生被治顺拐结果全部顺拐",
        "sum_count": 2,
    },
    {"challenge_id": 1666361267554312, "challenge_name": "惊悚50州", "sum_count": 2},
    {"challenge_id": 1557645172367362, "challenge_name": "日常", "sum_count": 2},
    {"challenge_id": 1633605924076555, "challenge_name": "轻慢计划", "sum_count": 2},
    {"challenge_id": 1555866148622338, "challenge_name": "奔跑吧", "sum_count": 2},
    {"challenge_id": 1656770740541448, "challenge_name": "张文宏", "sum_count": 2},
    {"challenge_id": 1548875285639169, "challenge_name": "女神", "sum_count": 2},
    {"challenge_id": 1568435181596673, "challenge_name": "放假", "sum_count": 2},
    {"challenge_id": 1653547886597124, "challenge_name": "dou➕小助手", "sum_count": 2},
    {"challenge_id": 1616803437278215, "challenge_name": "折叠家具", "sum_count": 2},
    {"challenge_id": 1619905765570567, "challenge_name": "陪产", "sum_count": 2},
    {"challenge_id": 1608221577030663, "challenge_name": "horizon", "sum_count": 1},
    {"challenge_id": 1564947196252161, "challenge_name": "周润发", "sum_count": 1},
    {
        "challenge_id": 1659982194850829,
        "challenge_name": "一个普通女孩的十年",
        "sum_count": 1,
    },
    {"challenge_id": 1667287410989068, "challenge_name": "师傅放心", "sum_count": 1},
    {"challenge_id": 1575679312647182, "challenge_name": "天津", "sum_count": 1},
    {
        "challenge_id": 1612588888036355,
        "challenge_name": "大爷永远是大爷",
        "sum_count": 1,
    },
    {"challenge_id": 1668648445143044, "challenge_name": "王斌在现场", "sum_count": 1},
    {"challenge_id": 1614492626878472, "challenge_name": "机械自动化", "sum_count": 1},
    {"challenge_id": 1574793281524750, "challenge_name": "马路", "sum_count": 1},
    {
        "challenge_id": 1675075958692876,
        "challenge_name": "娇软鹿的日常",
        "sum_count": 1,
    },
    {"challenge_id": 1579926533080078, "challenge_name": "关之琳", "sum_count": 1},
    {
        "challenge_id": 1655084555566087,
        "challenge_name": "业主与物业那些事",
        "sum_count": 1,
    },
    {"challenge_id": 1583830739247118, "challenge_name": "江西南昌", "sum_count": 1},
    {"challenge_id": 1615814169615428, "challenge_name": "缅甸布岛族", "sum_count": 1},
    {"challenge_id": 1593549864877128, "challenge_name": "中东", "sum_count": 1},
    {
        "challenge_id": 1644245215427612,
        "challenge_name": "我的剪辑报告",
        "sum_count": 1,
    },
    {"challenge_id": 1572613303660557, "challenge_name": "真人真唱", "sum_count": 1},
    {"challenge_id": 1556230693680129, "challenge_name": "男友", "sum_count": 1},
    {"challenge_id": 1575803422599182, "challenge_name": "推销", "sum_count": 1},
    {
        "challenge_id": 1596545368119310,
        "challenge_name": "少数民族风情",
        "sum_count": 1,
    },
    {"challenge_id": 1660406339262478, "challenge_name": "招聘", "sum_count": 1},
    {"challenge_id": 1589027986455566, "challenge_name": "再发一次", "sum_count": 1},
    {"challenge_id": 1605079245572125, "challenge_name": "我是大哥大", "sum_count": 1},
    {"challenge_id": 1642634962477063, "challenge_name": "99分女朋友", "sum_count": 1},
    {"challenge_id": 1563363924927490, "challenge_name": "上学", "sum_count": 1},
    {"challenge_id": 1595074090173512, "challenge_name": "心机女", "sum_count": 1},
    {"challenge_id": 1608256442873870, "challenge_name": "翡翠文化", "sum_count": 1},
    {
        "challenge_id": 1615634758161416,
        "challenge_name": "三胞胎三胞胎",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677422583907336,
        "challenge_name": "孙子不写作业奶奶暴打儿子",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677826341246990,
        "challenge_name": "云南瑞丽开展全员核酸检测",
        "sum_count": 1,
    },
    {
        "challenge_id": 1637524096832520,
        "challenge_name": "最简单的美好",
        "sum_count": 1,
    },
    {"challenge_id": 1568598695670786, "challenge_name": "异地恋", "sum_count": 1},
    {"challenge_id": 1586658073329677, "challenge_name": "自制", "sum_count": 1},
    {"challenge_id": 1658807615418381, "challenge_name": "龙大美", "sum_count": 1},
    {"challenge_id": 1582573799120925, "challenge_name": "马思纯", "sum_count": 1},
    {"challenge_id": 1571651404721153, "challenge_name": "保镖", "sum_count": 1},
    {"challenge_id": 1575622096352270, "challenge_name": "丁俊晖", "sum_count": 1},
    {"challenge_id": 1577551986167822, "challenge_name": "咳嗽", "sum_count": 1},
    {"challenge_id": 1593986716557316, "challenge_name": "建水紫陶", "sum_count": 1},
    {"challenge_id": 1561845286349842, "challenge_name": "皮皮虾", "sum_count": 1},
    {"challenge_id": 1656435194032135, "challenge_name": "疫情防控", "sum_count": 1},
    {"challenge_id": 1585227301827597, "challenge_name": "马伊琍", "sum_count": 1},
    {"challenge_id": 1587199187507214, "challenge_name": "郭德纲", "sum_count": 1},
    {"challenge_id": 1630614799588355, "challenge_name": "d0u", "sum_count": 1},
    {"challenge_id": 1575234177293326, "challenge_name": "北大", "sum_count": 1},
    {"challenge_id": 1553971125495809, "challenge_name": "邓紫棋", "sum_count": 1},
    {"challenge_id": 1628600677540872, "challenge_name": "虎哥说车", "sum_count": 1},
    {"challenge_id": 1553929685826562, "challenge_name": "挑战", "sum_count": 1},
    {"challenge_id": 1609134162985991, "challenge_name": "寝室拍大片", "sum_count": 1},
    {"challenge_id": 1580024873026621, "challenge_name": "爱国", "sum_count": 1},
    {
        "challenge_id": 1576960648516621,
        "challenge_name": "非物质文化遗产",
        "sum_count": 1,
    },
    {"challenge_id": 1573259546430478, "challenge_name": "瘦肚子", "sum_count": 1},
    {"challenge_id": 1571266844550146, "challenge_name": "太美了", "sum_count": 1},
    {"challenge_id": 1609180396205060, "challenge_name": "正能", "sum_count": 1},
    {"challenge_id": 1583692341179405, "challenge_name": "我这牛脾气", "sum_count": 1},
    {"challenge_id": 1572720148797453, "challenge_name": "女孩子", "sum_count": 1},
    {"challenge_id": 1561914085189698, "challenge_name": "苹果", "sum_count": 1},
    {"challenge_id": 1670365740269575, "challenge_name": "彬彬车行", "sum_count": 1},
    {"challenge_id": 1573365827425294, "challenge_name": "搭讪", "sum_count": 1},
    {"challenge_id": 1654878339827725, "challenge_name": "错乱次元", "sum_count": 1},
    {"challenge_id": 1648237424312332, "challenge_name": "小丽同学", "sum_count": 1},
    {"challenge_id": 1654975003512845, "challenge_name": "紫禁城600岁", "sum_count": 1},
    {"challenge_id": 1589272678518808, "challenge_name": "红酒🍷", "sum_count": 1},
    {
        "challenge_id": 1675240795837447,
        "challenge_name": "花式打卡浙江",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677329360559118,
        "challenge_name": "90后少女博导获阿里百万奖金",
        "sum_count": 1,
    },
    {"challenge_id": 1613749361831054, "challenge_name": "王岳伦", "sum_count": 1},
    {"challenge_id": 1672733728607240, "challenge_name": "828人气小店", "sum_count": 1},
    {
        "challenge_id": 1607935833578509,
        "challenge_name": "电视剧都是骗人的",
        "sum_count": 1,
    },
    {"challenge_id": 1615473055134733, "challenge_name": "砂锅炖肉", "sum_count": 1},
    {
        "challenge_id": 1677331755331598,
        "challenge_name": "山东淄博理光董事长",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677780554266637,
        "challenge_name": "粗加工各种海鲜",
        "sum_count": 1,
    },
    {"challenge_id": 1631063700701188, "challenge_name": "情感转达", "sum_count": 1},
    {"challenge_id": 1573608100017166, "challenge_name": "德云社", "sum_count": 1},
    {
        "challenge_id": 1674123559469069,
        "challenge_name": "锐锐剪辑团队",
        "sum_count": 1,
    },
    {"challenge_id": 1566374324096001, "challenge_name": "蹦极", "sum_count": 1},
    {"challenge_id": 1629059134253064, "challenge_name": "周雅琴", "sum_count": 1},
    {"challenge_id": 1590755687901197, "challenge_name": "经典电影", "sum_count": 1},
    {"challenge_id": 1586808789081101, "challenge_name": "暖心瞬间", "sum_count": 1},
    {"challenge_id": 1590032658336775, "challenge_name": "阳江", "sum_count": 1},
    {"challenge_id": 1607147058325523, "challenge_name": "叛逆期", "sum_count": 1},
    {"challenge_id": 1585655589758990, "challenge_name": "珠宝首饰", "sum_count": 1},
    {"challenge_id": 1611028694310925, "challenge_name": "门卫", "sum_count": 1},
    {"challenge_id": 1569922358710289, "challenge_name": "感恩父母", "sum_count": 1},
    {"challenge_id": 1617846227791879, "challenge_name": "家庭婚姻", "sum_count": 1},
    {"challenge_id": 1581662298856461, "challenge_name": "谍战", "sum_count": 1},
    {"challenge_id": 1659600781454414, "challenge_name": "身份照对比", "sum_count": 1},
    {"challenge_id": 1588191243392029, "challenge_name": "彩铃", "sum_count": 1},
    {"challenge_id": 1646901368606723, "challenge_name": "川哥说", "sum_count": 1},
    {"challenge_id": 1554049203856385, "challenge_name": "单身狗", "sum_count": 1},
    {"challenge_id": 1586309528130573, "challenge_name": "金莎", "sum_count": 1},
    {"challenge_id": 1562483969223682, "challenge_name": "上海滩", "sum_count": 1},
    {"challenge_id": 1675073649469443, "challenge_name": "苦瓜青蛙", "sum_count": 1},
    {"challenge_id": 1636878691204103, "challenge_name": "bt微剧场", "sum_count": 1},
    {"challenge_id": 1588647358728221, "challenge_name": "益智游戏", "sum_count": 1},
    {"challenge_id": 1595463332666382, "challenge_name": "服装厂上班", "sum_count": 1},
    {"challenge_id": 1571711624309777, "challenge_name": "家乡美食", "sum_count": 1},
    {"challenge_id": 1598332867904542, "challenge_name": "旋风十一人", "sum_count": 1},
    {"challenge_id": 1582482656235534, "challenge_name": "秋收", "sum_count": 1},
    {"challenge_id": 1644996543522828, "challenge_name": "随警实录", "sum_count": 1},
    {
        "challenge_id": 1677807991132167,
        "challenge_name": "孙弈秋成为八卦中心",
        "sum_count": 1,
    },
    {
        "challenge_id": 1610544087448605,
        "challenge_name": "感谢抖瓜小助手",
        "sum_count": 1,
    },
    {"challenge_id": 1579786358427662, "challenge_name": "交通", "sum_count": 1},
    {
        "challenge_id": 1636213247024142,
        "challenge_name": "这个视频有点长",
        "sum_count": 1,
    },
    {"challenge_id": 1619867748355079, "challenge_name": "天价彩礼", "sum_count": 1},
    {
        "challenge_id": 1677693343762456,
        "challenge_name": "任贤齐线上演唱会",
        "sum_count": 1,
    },
    {
        "challenge_id": 1660136842078222,
        "challenge_name": "严防境外输入",
        "sum_count": 1,
    },
    {"challenge_id": 1673710054162445, "challenge_name": "爱豆生日会", "sum_count": 1},
    {"challenge_id": 1570457988762625, "challenge_name": "人生", "sum_count": 1},
    {"challenge_id": 1582328279318541, "challenge_name": "怪物", "sum_count": 1},
    {"challenge_id": 1581324170786845, "challenge_name": "挖机", "sum_count": 1},
    {"challenge_id": 1556130176707585, "challenge_name": "考试", "sum_count": 1},
    {"challenge_id": 1573724611728397, "challenge_name": "带娃", "sum_count": 1},
    {"challenge_id": 1674359430930444, "challenge_name": "胡阳光律师", "sum_count": 1},
    {
        "challenge_id": 1678061895928839,
        "challenge_name": "dou车联盟计划",
        "sum_count": 1,
    },
    {"challenge_id": 1607471490807811, "challenge_name": "欢乐集结号", "sum_count": 1},
    {"challenge_id": 1569358207266817, "challenge_name": "办公室", "sum_count": 1},
    {"challenge_id": 1658046706639885, "challenge_name": "w坐", "sum_count": 1},
    {"challenge_id": 1591016787490820, "challenge_name": "武当功夫", "sum_count": 1},
    {"challenge_id": 1576760752819277, "challenge_name": "最美婚礼", "sum_count": 1},
    {"challenge_id": 1585871596630030, "challenge_name": "影帝", "sum_count": 1},
    {"challenge_id": 1628209320975371, "challenge_name": "西游记", "sum_count": 1},
    {"challenge_id": 1628708091349005, "challenge_name": "修蹄", "sum_count": 1},
    {
        "challenge_id": 1660435150791693,
        "challenge_name": "细节里的温柔",
        "sum_count": 1,
    },
    {"challenge_id": 1570274081510402, "challenge_name": "墙绘", "sum_count": 1},
    {"challenge_id": 1648533762397187, "challenge_name": "画圣吴道子", "sum_count": 1},
    {"challenge_id": 1677625546497047, "challenge_name": "356斤胖女人", "sum_count": 1},
    {"challenge_id": 1661139641757708, "challenge_name": "烈火军校", "sum_count": 1},
    {"challenge_id": 1573523342991373, "challenge_name": "励志语录", "sum_count": 1},
    {"challenge_id": 1572864661377038, "challenge_name": "安阳", "sum_count": 1},
    {"challenge_id": 1630306750056460, "challenge_name": "奥迪a3", "sum_count": 1},
    {"challenge_id": 1578773933358126, "challenge_name": "孝顺", "sum_count": 1},
    {"challenge_id": 1557394210078721, "challenge_name": "姐妹", "sum_count": 1},
    {"challenge_id": 1572364914717709, "challenge_name": "现实", "sum_count": 1},
    {"challenge_id": 1576262699707422, "challenge_name": "实拍", "sum_count": 1},
    {"challenge_id": 1579122148457486, "challenge_name": "原创不易", "sum_count": 1},
    {"challenge_id": 1615030692459550, "challenge_name": "天地传奇", "sum_count": 1},
    {"challenge_id": 1628241556386827, "challenge_name": "粗布", "sum_count": 1},
    {"challenge_id": 1619833883848711, "challenge_name": "林青霞", "sum_count": 1},
    {"challenge_id": 1654891234760711, "challenge_name": "马保国", "sum_count": 1},
    {"challenge_id": 1588036347976734, "challenge_name": "辟谣", "sum_count": 1},
    {"challenge_id": 1588403885384711, "challenge_name": "保安", "sum_count": 1},
    {"challenge_id": 1581356983554061, "challenge_name": "宋慧乔", "sum_count": 1},
    {"challenge_id": 1569094356624386, "challenge_name": "iphone", "sum_count": 1},
    {"challenge_id": 1601245849784327, "challenge_name": "白素贞特效", "sum_count": 1},
    {"challenge_id": 1586922812259342, "challenge_name": "荒岛求生", "sum_count": 1},
    {"challenge_id": 1602588321993735, "challenge_name": "脱身", "sum_count": 1},
    {"challenge_id": 1606479160135683, "challenge_name": "语言训练", "sum_count": 1},
    {
        "challenge_id": 1647791164395532,
        "challenge_name": "这家店回头客超多",
        "sum_count": 1,
    },
    {"challenge_id": 1618899348963406, "challenge_name": "情侣vlog", "sum_count": 1},
    {"challenge_id": 1573401444261901, "challenge_name": "萌娃成长记", "sum_count": 1},
    {"challenge_id": 1611892719721475, "challenge_name": "危情三日", "sum_count": 1},
    {"challenge_id": 1601090167505987, "challenge_name": "大一大四", "sum_count": 1},
    {"challenge_id": 1638050377402371, "challenge_name": "特工狂花", "sum_count": 1},
    {"challenge_id": 1613745645048836, "challenge_name": "保家卫国", "sum_count": 1},
    {"challenge_id": 1589016235010051, "challenge_name": "辞职", "sum_count": 1},
    {"challenge_id": 1556966590097409, "challenge_name": "超能力", "sum_count": 1},
    {"challenge_id": 1574630549176334, "challenge_name": "闪光少女", "sum_count": 1},
    {
        "challenge_id": 1589833244547075,
        "challenge_name": "拜见宫主大人",
        "sum_count": 1,
    },
    {
        "challenge_id": 1675630141854731,
        "challenge_name": "苏州流入包装核酸阳性南美白虾",
        "sum_count": 1,
    },
    {
        "challenge_id": 1665095635733512,
        "challenge_name": "中年夫妻纯友谊",
        "sum_count": 1,
    },
    {"challenge_id": 1615360309123079, "challenge_name": "美业培训", "sum_count": 1},
    {"challenge_id": 1605851628030990, "challenge_name": "西部教师", "sum_count": 1},
    {
        "challenge_id": 1676568208343047,
        "challenge_name": "译制的aarlran",
        "sum_count": 1,
    },
    {"challenge_id": 1582021699628045, "challenge_name": "张国荣", "sum_count": 1},
    {"challenge_id": 1666402929573895, "challenge_name": "爆笑v5哥", "sum_count": 1},
    {"challenge_id": 1590185743447111, "challenge_name": "梁朝伟", "sum_count": 1},
    {"challenge_id": 1583182607678478, "challenge_name": "加州", "sum_count": 1},
    {"challenge_id": 1571513614777346, "challenge_name": "农村", "sum_count": 1},
    {
        "challenge_id": 1677423800485891,
        "challenge_name": "上海迪士尼等69家景区门票半价",
        "sum_count": 1,
    },
    {"challenge_id": 1586131606159373, "challenge_name": "自律", "sum_count": 1},
    {
        "challenge_id": 1677956671829006,
        "challenge_name": "男生课堂内手机偷拍女同学",
        "sum_count": 1,
    },
    {"challenge_id": 1618369704978439, "challenge_name": "狱中豪杰", "sum_count": 1},
    {"challenge_id": 1590344907571214, "challenge_name": "货车司机", "sum_count": 1},
    {
        "challenge_id": 1667658272416839,
        "challenge_name": "c位舞担挑战赛",
        "sum_count": 1,
    },
    {"challenge_id": 1577203577194509, "challenge_name": "任贤齐", "sum_count": 1},
    {"challenge_id": 1558033381134338, "challenge_name": "青春", "sum_count": 1},
    {
        "challenge_id": 1657525503671304,
        "challenge_name": "迷惑行为大赏行为",
        "sum_count": 1,
    },
    {"challenge_id": 1581844768654350, "challenge_name": "温馨的家", "sum_count": 1},
    {"challenge_id": 1590390832749575, "challenge_name": "换装变身", "sum_count": 1},
    {"challenge_id": 1597233945060366, "challenge_name": "校园安全", "sum_count": 1},
    {"challenge_id": 1567159280269314, "challenge_name": "感恩", "sum_count": 1},
    {"challenge_id": 1606844993013789, "challenge_name": "黄鸿升", "sum_count": 1},
    {
        "challenge_id": 1613844729635870,
        "challenge_name": "看到结局我哭了",
        "sum_count": 1,
    },
    {
        "challenge_id": 1600351488663572,
        "challenge_name": "一日为师终身为父",
        "sum_count": 1,
    },
    {"challenge_id": 1584306432053262, "challenge_name": "聊天套路", "sum_count": 1},
    {"challenge_id": 1600355983452174, "challenge_name": "李湘", "sum_count": 1},
    {"challenge_id": 1591372466249732, "challenge_name": "工地", "sum_count": 1},
    {"challenge_id": 1574208994470925, "challenge_name": "赵薇", "sum_count": 1},
    {
        "challenge_id": 1677624332130311,
        "challenge_name": "谦酒中秋礼品",
        "sum_count": 1,
    },
    {
        "challenge_id": 1647897397876744,
        "challenge_name": "痣的魅力发散时间",
        "sum_count": 1,
    },
    {"challenge_id": 1584769109553182, "challenge_name": "巴基斯坦", "sum_count": 1},
    {"challenge_id": 1566027883435010, "challenge_name": "剪辑", "sum_count": 1},
    {"challenge_id": 1576222356822029, "challenge_name": "钟汉良", "sum_count": 1},
    {"challenge_id": 1563651421451265, "challenge_name": "爱", "sum_count": 1},
    {"challenge_id": 1569180997514241, "challenge_name": "母女", "sum_count": 1},
    {"challenge_id": 1573890743857165, "challenge_name": "海岛", "sum_count": 1},
    {"challenge_id": 1641721812980747, "challenge_name": "哎呀好身材", "sum_count": 1},
    {"challenge_id": 1642088166175751, "challenge_name": "ios14", "sum_count": 1},
    {"challenge_id": 1595512468187150, "challenge_name": "走红", "sum_count": 1},
    {"challenge_id": 1607383867151363, "challenge_name": "工厂实拍", "sum_count": 1},
    {"challenge_id": 1570632846160897, "challenge_name": "兄弟情", "sum_count": 1},
    {"challenge_id": 1582006148537358, "challenge_name": "笑到肚子疼", "sum_count": 1},
    {"challenge_id": 1619646607747079, "challenge_name": "沙雕男孩", "sum_count": 1},
    {"challenge_id": 1585882121411597, "challenge_name": "史泰龙", "sum_count": 1},
    {
        "challenge_id": 1582292867936269,
        "challenge_name": "越努力越幸运",
        "sum_count": 1,
    },
    {"challenge_id": 1615402772513806, "challenge_name": "养鸡小妙招", "sum_count": 1},
    {"challenge_id": 1586547190477837, "challenge_name": "花样美食", "sum_count": 1},
    {"challenge_id": 1579235549144077, "challenge_name": "沫子", "sum_count": 1},
    {"challenge_id": 1578775172703309, "challenge_name": "自燃", "sum_count": 1},
    {"challenge_id": 1574763654458382, "challenge_name": "想画就画", "sum_count": 1},
    {"challenge_id": 1635668155935752, "challenge_name": "合拍", "sum_count": 1},
    {"challenge_id": 1581424430688269, "challenge_name": "遥控车", "sum_count": 1},
    {"challenge_id": 1582293102585870, "challenge_name": "湛江", "sum_count": 1},
    {"challenge_id": 1597834977172493, "challenge_name": "龙门飞甲", "sum_count": 1},
    {"challenge_id": 1640380476055563, "challenge_name": "警民dou平安", "sum_count": 1},
    {"challenge_id": 1611770588786692, "challenge_name": "二战", "sum_count": 1},
    {
        "challenge_id": 1677632695673870,
        "challenge_name": "济南商务精英套餐",
        "sum_count": 1,
    },
    {"challenge_id": 1586778433514542, "challenge_name": "国产车", "sum_count": 1},
    {"challenge_id": 1645183910101000, "challenge_name": "母爱伟大", "sum_count": 1},
    {"challenge_id": 1565611583253506, "challenge_name": "书法", "sum_count": 1},
    {"challenge_id": 1604132234533959, "challenge_name": "半路夫妻", "sum_count": 1},
    {"challenge_id": 1573908118673453, "challenge_name": "霸道总裁", "sum_count": 1},
    {"challenge_id": 1618989555632141, "challenge_name": "改装车文化", "sum_count": 1},
    {"challenge_id": 1571280446562306, "challenge_name": "培训", "sum_count": 1},
    {"challenge_id": 1589712777432067, "challenge_name": "范丞丞", "sum_count": 1},
    {"challenge_id": 1583577467595822, "challenge_name": "小棉袄", "sum_count": 1},
    {"challenge_id": 1593455819938819, "challenge_name": "电影解说", "sum_count": 1},
    {"challenge_id": 1564100325523457, "challenge_name": "美发", "sum_count": 1},
    {"challenge_id": 1630975740300295, "challenge_name": "灵异故事", "sum_count": 1},
    {
        "challenge_id": 1623233836704936,
        "challenge_name": "和爸爸一起的时光",
        "sum_count": 1,
    },
    {
        "challenge_id": 1676699515961357,
        "challenge_name": "吹火柴召唤李易峰",
        "sum_count": 1,
    },
    {"challenge_id": 1644107841733645, "challenge_name": "马仔", "sum_count": 1},
    {
        "challenge_id": 1573065848927246,
        "challenge_name": "感谢抖瓜平台",
        "sum_count": 1,
    },
    {"challenge_id": 1598988285703181, "challenge_name": "乌鸦哥", "sum_count": 1},
    {"challenge_id": 1572718180930574, "challenge_name": "麻辣", "sum_count": 1},
    {"challenge_id": 1667829937660942, "challenge_name": "趣胃大比拼", "sum_count": 1},
    {"challenge_id": 1611666858356744, "challenge_name": "血战到底", "sum_count": 1},
    {"challenge_id": 1604867640834061, "challenge_name": "卡车女司机", "sum_count": 1},
    {"challenge_id": 1598105166418951, "challenge_name": "好物种草", "sum_count": 1},
    {"challenge_id": 1564799895000065, "challenge_name": "艺术", "sum_count": 1},
    {"challenge_id": 1606079072609293, "challenge_name": "金环虎头蜂", "sum_count": 1},
    {"challenge_id": 1566623537040386, "challenge_name": "贾玲", "sum_count": 1},
    {"challenge_id": 1581850108908557, "challenge_name": "落叶归根", "sum_count": 1},
    {"challenge_id": 1594783757630472, "challenge_name": "李梦", "sum_count": 1},
    {"challenge_id": 1592386142912519, "challenge_name": "我的个神啊", "sum_count": 1},
    {"challenge_id": 1661139342028814, "challenge_name": "古装玄幻", "sum_count": 1},
    {"challenge_id": 1641167990383628, "challenge_name": "剧情搞笑", "sum_count": 1},
    {
        "challenge_id": 1650129810698254,
        "challenge_name": "作品推广上热门",
        "sum_count": 1,
    },
    {"challenge_id": 1598270765573133, "challenge_name": "鹤岗", "sum_count": 1},
    {
        "challenge_id": 1677178286422039,
        "challenge_name": "封闭状态下的大学生",
        "sum_count": 1,
    },
    {"challenge_id": 1674067620358147, "challenge_name": "保安小郭", "sum_count": 1},
    {"challenge_id": 1590095478814733, "challenge_name": "摘果子", "sum_count": 1},
    {
        "challenge_id": 1674318296331275,
        "challenge_name": "d0u十小助手推广上热门",
        "sum_count": 1,
    },
    {"challenge_id": 1563271944097794, "challenge_name": "拳击", "sum_count": 1},
    {
        "challenge_id": 1677434966732807,
        "challenge_name": "今年的生日想跟你们一起过",
        "sum_count": 1,
    },
    {"challenge_id": 1651608503116811, "challenge_name": "插翅難逃", "sum_count": 1},
    {
        "challenge_id": 1675978590602254,
        "challenge_name": "这波操作6到飞起",
        "sum_count": 1,
    },
    {"challenge_id": 1578071848133645, "challenge_name": "婚礼现场", "sum_count": 1},
    {"challenge_id": 1586559586261005, "challenge_name": "永远的爱", "sum_count": 1},
    {"challenge_id": 1569738789608449, "challenge_name": "南京", "sum_count": 1},
    {"challenge_id": 1581509961682958, "challenge_name": "拱鼻子", "sum_count": 1},
    {"challenge_id": 1662046553262094, "challenge_name": "海王翻车", "sum_count": 1},
    {"challenge_id": 1599868041132040, "challenge_name": "卡车生活", "sum_count": 1},
    {"challenge_id": 1545627478029314, "challenge_name": "滑板", "sum_count": 1},
    {"challenge_id": 1655342447857672, "challenge_name": "警中警", "sum_count": 1},
    {
        "challenge_id": 1661680330808333,
        "challenge_name": "来西瓜看纪录片",
        "sum_count": 1,
    },
    {
        "challenge_id": 1676715487327262,
        "challenge_name": "2020线上智博会",
        "sum_count": 1,
    },
    {"challenge_id": 1564376754727938, "challenge_name": "爷爷奶奶", "sum_count": 1},
    {"challenge_id": 1558371959940098, "challenge_name": "开学了", "sum_count": 1},
    {"challenge_id": 1572646759963661, "challenge_name": "关晓彤", "sum_count": 1},
    {"challenge_id": 1581561806214157, "challenge_name": "高伟光", "sum_count": 1},
    {"challenge_id": 1637409456249864, "challenge_name": "茅台", "sum_count": 1},
    {"challenge_id": 1677620908198923, "challenge_name": "嘿优嘿", "sum_count": 1},
    {
        "challenge_id": 1670648457901063,
        "challenge_name": "电影霍家拳之铁臂娇娃",
        "sum_count": 1,
    },
    {"challenge_id": 1620461286107150, "challenge_name": "储备", "sum_count": 1},
    {"challenge_id": 1658603128961031, "challenge_name": "哈士奇", "sum_count": 1},
    {"challenge_id": 1615122301187079, "challenge_name": "中国健美", "sum_count": 1},
    {"challenge_id": 1572367939851277, "challenge_name": "感动", "sum_count": 1},
    {
        "challenge_id": 1677685842666504,
        "challenge_name": "热门六指琴魔重出江湖啦",
        "sum_count": 1,
    },
    {"challenge_id": 1582478974565390, "challenge_name": "宝宝笑不停", "sum_count": 1},
    {"challenge_id": 1579873224435726, "challenge_name": "胡子", "sum_count": 1},
    {"challenge_id": 1631248823444488, "challenge_name": "陆战之王", "sum_count": 1},
    {"challenge_id": 1674176801674264, "challenge_name": "超级治愈力", "sum_count": 1},
    {"challenge_id": 1613937861809219, "challenge_name": "整理师", "sum_count": 1},
    {
        "challenge_id": 1656510821824519,
        "challenge_name": "谁拍谁帅系列",
        "sum_count": 1,
    },
    {"challenge_id": 1664496188443655, "challenge_name": "阿九的vlog", "sum_count": 1},
    {"challenge_id": 1575316305784846, "challenge_name": "老年生活", "sum_count": 1},
    {
        "challenge_id": 1658773699008519,
        "challenge_name": "宅在家dou剧场",
        "sum_count": 1,
    },
    {"challenge_id": 1655158438612995, "challenge_name": "李有财", "sum_count": 1},
    {
        "challenge_id": 1660387517261835,
        "challenge_name": "相信科学拒绝迷信",
        "sum_count": 1,
    },
    {"challenge_id": 1663129754291212, "challenge_name": "莎言莎语", "sum_count": 1},
    {
        "challenge_id": 1570540647758850,
        "challenge_name": "来自天堂的魔鬼",
        "sum_count": 1,
    },
    {"challenge_id": 1618911241730062, "challenge_name": "教育培训", "sum_count": 1},
    {"challenge_id": 1611046335057939, "challenge_name": "艺术培训", "sum_count": 1},
    {"challenge_id": 1593802365928455, "challenge_name": "缅甸翡翠", "sum_count": 1},
    {"challenge_id": 1615653603419149, "challenge_name": "卖货能手", "sum_count": 1},
    {
        "challenge_id": 1647632232325124,
        "challenge_name": "树上有个好地方",
        "sum_count": 1,
    },
    {"challenge_id": 1557694920041473, "challenge_name": "烟花", "sum_count": 1},
    {
        "challenge_id": 1580586983667725,
        "challenge_name": "老公的私房钱",
        "sum_count": 1,
    },
    {
        "challenge_id": 1589636010913796,
        "challenge_name": "哈尔滨师范大学",
        "sum_count": 1,
    },
    {"challenge_id": 1615717554729988, "challenge_name": "玉湛高速", "sum_count": 1},
    {"challenge_id": 1642913662977038, "challenge_name": "djemi", "sum_count": 1},
    {"challenge_id": 1563325383097345, "challenge_name": "眼神杀", "sum_count": 1},
    {"challenge_id": 1562022461380610, "challenge_name": "演员", "sum_count": 1},
    {
        "challenge_id": 1678065721482247,
        "challenge_name": "对美籍鲍某某驱逐出境",
        "sum_count": 1,
    },
    {"challenge_id": 1580371373399053, "challenge_name": "黑龙江", "sum_count": 1},
    {"challenge_id": 1605882854420487, "challenge_name": "人人平等", "sum_count": 1},
    {"challenge_id": 1676599112090627, "challenge_name": "新泰融媒", "sum_count": 1},
    {"challenge_id": 1594187129217031, "challenge_name": "潘金莲", "sum_count": 1},
    {"challenge_id": 1563587876501505, "challenge_name": "阿拉斯加", "sum_count": 1},
    {
        "challenge_id": 1611822709136387,
        "challenge_name": "鹿晗为关晓彤庆生",
        "sum_count": 1,
    },
    {"challenge_id": 1658405473617927, "challenge_name": "支棱小伙", "sum_count": 1},
    {"challenge_id": 1677801216183309, "challenge_name": "女兵的骄傲", "sum_count": 1},
    {"challenge_id": 1572090756036609, "challenge_name": "池昌旭", "sum_count": 1},
    {
        "challenge_id": 1677533599336493,
        "challenge_name": "苍溪小学生疑遭老师体罚后身亡",
        "sum_count": 1,
    },
    {"challenge_id": 1577511268705293, "challenge_name": "音乐现场", "sum_count": 1},
    {"challenge_id": 1595414266075144, "challenge_name": "减肥瘦身", "sum_count": 1},
    {"challenge_id": 1572190291694605, "challenge_name": "窍门", "sum_count": 1},
    {
        "challenge_id": 1677522725384206,
        "challenge_name": "贾乃亮李小璐陪甜馨打羽毛球",
        "sum_count": 1,
    },
    {"challenge_id": 1562768988957698, "challenge_name": "格斗", "sum_count": 1},
    {
        "challenge_id": 1650824052935687,
        "challenge_name": "电子废料提金",
        "sum_count": 1,
    },
    {"challenge_id": 1596262637232131, "challenge_name": "徐冬冬", "sum_count": 1},
    {"challenge_id": 1594273576638494, "challenge_name": "乡村老师", "sum_count": 1},
    {"challenge_id": 1661430979902472, "challenge_name": "吴镇宇", "sum_count": 1},
    {"challenge_id": 1580655978429454, "challenge_name": "板栗", "sum_count": 1},
    {
        "challenge_id": 1669284916242435,
        "challenge_name": "百亿赞演技大赏",
        "sum_count": 1,
    },
    {"challenge_id": 1624453083584520, "challenge_name": "阿公阿嬷", "sum_count": 1},
    {
        "challenge_id": 1596073872269315,
        "challenge_name": "舌尖上中国美食",
        "sum_count": 1,
    },
    {"challenge_id": 1570118743946241, "challenge_name": "街头采访", "sum_count": 1},
    {"challenge_id": 1614449356399620, "challenge_name": "皮鞋", "sum_count": 1},
    {
        "challenge_id": 1678061716969486,
        "challenge_name": "危机边缘第四季",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677533609670669,
        "challenge_name": "差评阴影下的外卖员",
        "sum_count": 1,
    },
    {"challenge_id": 1577178216721422, "challenge_name": "艺考", "sum_count": 1},
    {"challenge_id": 1580029326993421, "challenge_name": "偷天换日", "sum_count": 1},
    {
        "challenge_id": 1615585797143560,
        "challenge_name": "神奇动物在抖瓜",
        "sum_count": 1,
    },
    {"challenge_id": 1593447052551171, "challenge_name": "儿童安全", "sum_count": 1},
    {"challenge_id": 1611589960166404, "challenge_name": "农村酒席", "sum_count": 1},
    {
        "challenge_id": 1677829355082759,
        "challenge_name": "瑞丽城区全员开展核酸检测",
        "sum_count": 1,
    },
    {"challenge_id": 1592907166308360, "challenge_name": "云南人", "sum_count": 1},
    {"challenge_id": 1576799130017806, "challenge_name": "缅甸", "sum_count": 1},
    {"challenge_id": 1560583036432385, "challenge_name": "发型设计", "sum_count": 1},
    {"challenge_id": 1627810330993675, "challenge_name": "工地大舞台", "sum_count": 1},
    {"challenge_id": 1614549980816392, "challenge_name": "赵丽穎", "sum_count": 1},
    {"challenge_id": 1639748149639175, "challenge_name": "矢野浩二", "sum_count": 1},
    {"challenge_id": 1584270750044174, "challenge_name": "lol", "sum_count": 1},
    {
        "challenge_id": 1615082375429127,
        "challenge_name": "考不好没关系",
        "sum_count": 1,
    },
    {"challenge_id": 1574898006359053, "challenge_name": "谢娜", "sum_count": 1},
    {"challenge_id": 1573809239648269, "challenge_name": "真香", "sum_count": 1},
    {"challenge_id": 1677351786830860, "challenge_name": "江湖实录", "sum_count": 1},
    {"challenge_id": 1576073643446286, "challenge_name": "新娘", "sum_count": 1},
    {"challenge_id": 1655248356264974, "challenge_name": "逐梦电影", "sum_count": 1},
    {"challenge_id": 1591165535337485, "challenge_name": "气质穿搭", "sum_count": 1},
    {
        "challenge_id": 1677499689629707,
        "challenge_name": "珠海斗门白藤二路藤湖大酒店一楼发生爆炸",
        "sum_count": 1,
    },
    {"challenge_id": 1677896764436487, "challenge_name": "何雪琳人设", "sum_count": 1},
    {"challenge_id": 1565081816670209, "challenge_name": "手速", "sum_count": 1},
    {"challenge_id": 1610581830061063, "challenge_name": "一颜难尽", "sum_count": 1},
    {"challenge_id": 1635048771737611, "challenge_name": "你莫走", "sum_count": 1},
    {"challenge_id": 1569541333031938, "challenge_name": "运动女孩", "sum_count": 1},
    {"challenge_id": 1655874155323404, "challenge_name": "百变时光机", "sum_count": 1},
    {"challenge_id": 1572303552988174, "challenge_name": "宠物成精啦", "sum_count": 1},
    {
        "challenge_id": 1624014493430788,
        "challenge_name": "希望官方给个热门",
        "sum_count": 1,
    },
    {"challenge_id": 1589361675417613, "challenge_name": "贝勒爷", "sum_count": 1},
    {"challenge_id": 1642644163636232, "challenge_name": "抖瓜管理员", "sum_count": 1},
    {"challenge_id": 1591191475292173, "challenge_name": "护士辞职", "sum_count": 1},
    {"challenge_id": 1620538664215629, "challenge_name": "闺蜜陪我疯", "sum_count": 1},
    {"challenge_id": 1618517464923143, "challenge_name": "汾酒", "sum_count": 1},
    {"challenge_id": 1594054621918222, "challenge_name": "张俪", "sum_count": 1},
    {"challenge_id": 1589387230891016, "challenge_name": "恋爱先生", "sum_count": 1},
    {"challenge_id": 1600990344323101, "challenge_name": "河北衡水", "sum_count": 1},
    {"challenge_id": 1649168562718733, "challenge_name": "玉米专用", "sum_count": 1},
    {"challenge_id": 1616279502718019, "challenge_name": "假装有头发", "sum_count": 1},
    {"challenge_id": 1582462762039326, "challenge_name": "产房", "sum_count": 1},
    {"challenge_id": 1566629966497794, "challenge_name": "潮牌", "sum_count": 1},
    {
        "challenge_id": 1677769432592392,
        "challenge_name": "西门子ceo发表涉华不当言论",
        "sum_count": 1,
    },
    {"challenge_id": 1555593112411137, "challenge_name": "宝宝", "sum_count": 1},
    {"challenge_id": 1616973823916039, "challenge_name": "家庭情感", "sum_count": 1},
    {"challenge_id": 1644286753707015, "challenge_name": "征兵", "sum_count": 1},
    {"challenge_id": 1588650791312387, "challenge_name": "智取威虎山", "sum_count": 1},
    {"challenge_id": 1606614144462856, "challenge_name": "连城诀", "sum_count": 1},
    {"challenge_id": 1580838992464910, "challenge_name": "比特犬", "sum_count": 1},
    {"challenge_id": 1650795778818055, "challenge_name": "健康dou起来", "sum_count": 1},
    {"challenge_id": 1583864060574749, "challenge_name": "流氓", "sum_count": 1},
    {"challenge_id": 1618344016454685, "challenge_name": "汽配", "sum_count": 1},
    {"challenge_id": 1615013540939779, "challenge_name": "奖金", "sum_count": 1},
    {"challenge_id": 1676812894206984, "challenge_name": "趣玩车", "sum_count": 1},
    {
        "challenge_id": 1595085744408579,
        "challenge_name": "抖瓜热门话题",
        "sum_count": 1,
    },
    {"challenge_id": 1614459277782020, "challenge_name": "塑形", "sum_count": 1},
    {
        "challenge_id": 1575985801281549,
        "challenge_name": "帅气的小姐姐",
        "sum_count": 1,
    },
    {"challenge_id": 1590651193029639, "challenge_name": "雪地里撒欢", "sum_count": 1},
    {"challenge_id": 1573711184334861, "challenge_name": "冰淇淋", "sum_count": 1},
    {"challenge_id": 1569830415045634, "challenge_name": "爱你一万年", "sum_count": 1},
    {"challenge_id": 1613272771060743, "challenge_name": "望爱却步", "sum_count": 1},
    {"challenge_id": 1574894121565198, "challenge_name": "金秀贤", "sum_count": 1},
    {"challenge_id": 1574442497148942, "challenge_name": "中餐厅", "sum_count": 1},
    {"challenge_id": 1611676292230222, "challenge_name": "财富", "sum_count": 1},
    {"challenge_id": 1594554317488142, "challenge_name": "余欢水", "sum_count": 1},
    {"challenge_id": 1610843051409475, "challenge_name": "秋天乱穿衣", "sum_count": 1},
    {
        "challenge_id": 1662101943347212,
        "challenge_name": "周大生珠宝日常搞笑",
        "sum_count": 1,
    },
    {"challenge_id": 1585410966541326, "challenge_name": "危机边缘", "sum_count": 1},
    {"challenge_id": 1596466278888462, "challenge_name": "时间管理", "sum_count": 1},
    {"challenge_id": 1596619867667463, "challenge_name": "防诈骗", "sum_count": 1},
    {"challenge_id": 1650424093959180, "challenge_name": "虎妈猫爸", "sum_count": 1},
    {
        "challenge_id": 1677994124308487,
        "challenge_name": "男子醉驾撞护栏失控伤及无辜",
        "sum_count": 1,
    },
    {"challenge_id": 1611477091656718, "challenge_name": "农业种植", "sum_count": 1},
    {"challenge_id": 1575247165019149, "challenge_name": "瘦", "sum_count": 1},
    {"challenge_id": 1608971865452558, "challenge_name": "gai", "sum_count": 1},
    {"challenge_id": 1578794768823453, "challenge_name": "妻管严", "sum_count": 1},
    {"challenge_id": 1608473282250766, "challenge_name": "艺术教育", "sum_count": 1},
    {"challenge_id": 1616822754563086, "challenge_name": "农村高科技", "sum_count": 1},
    {"challenge_id": 1591826118891533, "challenge_name": "孕期记录", "sum_count": 1},
    {"challenge_id": 1600123580660744, "challenge_name": "我有个朋友", "sum_count": 1},
    {"challenge_id": 1656679739249672, "challenge_name": "疫情速报", "sum_count": 1},
    {"challenge_id": 1587031579826189, "challenge_name": "玉观音", "sum_count": 1},
    {"challenge_id": 1590850982685699, "challenge_name": "沉迷", "sum_count": 1},
    {
        "challenge_id": 1678042030360589,
        "challenge_name": "关晓彤的生日party",
        "sum_count": 1,
    },
    {"challenge_id": 1586575343307789, "challenge_name": "内江", "sum_count": 1},
    {
        "challenge_id": 1620207731761155,
        "challenge_name": "感谢大家的支持🔥",
        "sum_count": 1,
    },
    {"challenge_id": 1607955416252424, "challenge_name": "实线变道", "sum_count": 1},
    {"challenge_id": 1609964056119303, "challenge_name": "官方热门", "sum_count": 1},
    {"challenge_id": 1643609202738180, "challenge_name": "直播带货", "sum_count": 1},
    {"challenge_id": 1604225344941070, "challenge_name": "硬核", "sum_count": 1},
    {"challenge_id": 1574254569108557, "challenge_name": "魔女", "sum_count": 1},
    {"challenge_id": 1572317678390285, "challenge_name": "法拉利", "sum_count": 1},
    {"challenge_id": 1658776575168526, "challenge_name": "正式编制", "sum_count": 1},
    {"challenge_id": 1649642637815820, "challenge_name": "傻福贵", "sum_count": 1},
    {
        "challenge_id": 1617896170502158,
        "challenge_name": "白衣天使加油",
        "sum_count": 1,
    },
    {"challenge_id": 1570089296137217, "challenge_name": "冷漠", "sum_count": 1},
    {"challenge_id": 1629797193636868, "challenge_name": "财经", "sum_count": 1},
    {"challenge_id": 1581075393138701, "challenge_name": "姐弟恋", "sum_count": 1},
    {
        "challenge_id": 1669356092775432,
        "challenge_name": "马达的奇怪知识",
        "sum_count": 1,
    },
    {
        "challenge_id": 1653978598407181,
        "challenge_name": "庆余年戏精二人组范闲王启年",
        "sum_count": 1,
    },
    {"challenge_id": 1597599566849027, "challenge_name": "rng", "sum_count": 1},
    {"challenge_id": 1640618400433166, "challenge_name": "月半树洞", "sum_count": 1},
    {"challenge_id": 1549701348199425, "challenge_name": "王俊凯", "sum_count": 1},
    {
        "challenge_id": 1674243321336846,
        "challenge_name": "赵露思说韩语",
        "sum_count": 1,
    },
    {
        "challenge_id": 1630615652626440,
        "challenge_name": "超级好看电视剧推荐",
        "sum_count": 1,
    },
    {"challenge_id": 1671075878983693, "challenge_name": "读心能力", "sum_count": 1},
    {"challenge_id": 1624364946109452, "challenge_name": "台湾女友", "sum_count": 1},
    {
        "challenge_id": 1602625902934020,
        "challenge_name": "你大爷还是大爷",
        "sum_count": 1,
    },
    {"challenge_id": 1603173373634564, "challenge_name": "扔书", "sum_count": 1},
    {
        "challenge_id": 1650980387044365,
        "challenge_name": "因为太怕痛就全点防御力了",
        "sum_count": 1,
    },
    {
        "challenge_id": 1640955313580043,
        "challenge_name": "精彩影视剧剪辑",
        "sum_count": 1,
    },
    {"challenge_id": 1570355740748801, "challenge_name": "脱口秀", "sum_count": 1},
    {
        "challenge_id": 1652367943851022,
        "challenge_name": "英雄辈出的年代",
        "sum_count": 1,
    },
    {"challenge_id": 1564473851210753, "challenge_name": "巴克球", "sum_count": 1},
    {"challenge_id": 1563095009253378, "challenge_name": "汉服", "sum_count": 1},
    {"challenge_id": 1587401760185374, "challenge_name": "保险", "sum_count": 1},
    {"challenge_id": 1576158997976077, "challenge_name": "叛逆", "sum_count": 1},
    {
        "challenge_id": 1598061688320007,
        "challenge_name": "抖瓜热门视频",
        "sum_count": 1,
    },
    {"challenge_id": 1645017971986439, "challenge_name": "1942电影", "sum_count": 1},
    {"challenge_id": 1585290704863245, "challenge_name": "90", "sum_count": 1},
    {"challenge_id": 1575330213748749, "challenge_name": "算你狠", "sum_count": 1},
    {"challenge_id": 1603964970464270, "challenge_name": "致敬老师", "sum_count": 1},
    {"challenge_id": 1614845094397960, "challenge_name": "抗战", "sum_count": 1},
    {"challenge_id": 1607331759615047, "challenge_name": "结婚彩礼", "sum_count": 1},
    {"challenge_id": 1630780136879117, "challenge_name": "天不藏奸", "sum_count": 1},
    {
        "challenge_id": 1677804447846413,
        "challenge_name": "柠檬工坊努力💪",
        "sum_count": 1,
    },
    {"challenge_id": 1590360481346567, "challenge_name": "充电宝", "sum_count": 1},
    {"challenge_id": 1645445259183116, "challenge_name": "阅兵", "sum_count": 1},
    {"challenge_id": 1558862241294337, "challenge_name": "浪", "sum_count": 1},
    {"challenge_id": 1594801327811588, "challenge_name": "金工实习", "sum_count": 1},
    {"challenge_id": 1570263729747010, "challenge_name": "密室逃脱", "sum_count": 1},
    {"challenge_id": 1665908007190536, "challenge_name": "地心末日", "sum_count": 1},
    {
        "challenge_id": 1632329888989191,
        "challenge_name": "我的隐藏身份",
        "sum_count": 1,
    },
    {"challenge_id": 1586182766198798, "challenge_name": "十个月宝宝", "sum_count": 1},
    {
        "challenge_id": 1626988379577358,
        "challenge_name": "走进非洲原始部落",
        "sum_count": 1,
    },
    {"challenge_id": 1620975509824525, "challenge_name": "奥迪a8l", "sum_count": 1},
    {"challenge_id": 1595058396691459, "challenge_name": "书画艺术", "sum_count": 1},
    {"challenge_id": 1584470124831757, "challenge_name": "get新技能", "sum_count": 1},
    {"challenge_id": 1563557967112193, "challenge_name": "韩国", "sum_count": 1},
    {"challenge_id": 1590099521748999, "challenge_name": "救人", "sum_count": 1},
    {"challenge_id": 1569150029205505, "challenge_name": "电动车", "sum_count": 1},
    {"challenge_id": 1676628336435213, "challenge_name": "治愈的刀功", "sum_count": 1},
    {"challenge_id": 1575638435665933, "challenge_name": "张云龙", "sum_count": 1},
    {"challenge_id": 1596970835574792, "challenge_name": "电竞馆", "sum_count": 1},
    {"challenge_id": 1603127398250504, "challenge_name": "下一个是我", "sum_count": 1},
    {"challenge_id": 1582215590268941, "challenge_name": "走火", "sum_count": 1},
    {"challenge_id": 1589102688176132, "challenge_name": "陈皮", "sum_count": 1},
    {"challenge_id": 1581225510197390, "challenge_name": "职业装", "sum_count": 1},
    {"challenge_id": 1612765621460999, "challenge_name": "伏特加果冻", "sum_count": 1},
    {"challenge_id": 1617914551260174, "challenge_name": "黄酒", "sum_count": 1},
    {
        "challenge_id": 1670075385956392,
        "challenge_name": "逆世界迷惑行为",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677803794387991,
        "challenge_name": "米易突发泥石流",
        "sum_count": 1,
    },
    {"challenge_id": 1578895489977357, "challenge_name": "自毁模式", "sum_count": 1},
    {"challenge_id": 1567028970897410, "challenge_name": "道具", "sum_count": 1},
    {"challenge_id": 1617620888906766, "challenge_name": "近嫁", "sum_count": 1},
    {
        "challenge_id": 1621198179471373,
        "challenge_name": "广西灵魂歌手",
        "sum_count": 1,
    },
    {
        "challenge_id": 1662851053734916,
        "challenge_name": "鳄龟霸气无敌",
        "sum_count": 1,
    },
    {"challenge_id": 1577987773763677, "challenge_name": "人与自然", "sum_count": 1},
    {
        "challenge_id": 1678004267088903,
        "challenge_name": "刘涛老公王珂发长文",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677700856892446,
        "challenge_name": "高校封闭情侣隔着栅栏吃火锅",
        "sum_count": 1,
    },
    {"challenge_id": 1565888419307521, "challenge_name": "跑车", "sum_count": 1},
    {
        "challenge_id": 1673065956697096,
        "challenge_name": "紫禁600仍少年",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677810082910285,
        "challenge_name": "遇见心动女生",
        "sum_count": 1,
    },
    {"challenge_id": 1571609510760449, "challenge_name": "皮", "sum_count": 1},
    {"challenge_id": 1659679153233927, "challenge_name": "吴桂芳", "sum_count": 1},
    {"challenge_id": 1666206334536718, "challenge_name": "元气轻漫班", "sum_count": 1},
    {"challenge_id": 1568743571226625, "challenge_name": "室友", "sum_count": 1},
    {"challenge_id": 1566302398195713, "challenge_name": "揭秘", "sum_count": 1},
    {"challenge_id": 1600685058999300, "challenge_name": "战友", "sum_count": 1},
    {"challenge_id": 1576683421501454, "challenge_name": "手机支架", "sum_count": 1},
    {"challenge_id": 1633496688174091, "challenge_name": "看我多会玩", "sum_count": 1},
    {
        "challenge_id": 1677723886699528,
        "challenge_name": "安顺男子刚从足疗城出来就倒地猝死",
        "sum_count": 1,
    },
    {"challenge_id": 1583824128137245, "challenge_name": "口腔", "sum_count": 1},
    {"challenge_id": 1609327610456072, "challenge_name": "军旅", "sum_count": 1},
    {
        "challenge_id": 1677343634844685,
        "challenge_name": "亲爱的你在哪里定档",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677677424136199,
        "challenge_name": "云南瑞丽奥星世纪小区发现1例疑似病例",
        "sum_count": 1,
    },
    {"challenge_id": 1569020329795586, "challenge_name": "大爷", "sum_count": 1},
    {"challenge_id": 1565348497132546, "challenge_name": "幼师", "sum_count": 1},
    {
        "challenge_id": 1667561865846795,
        "challenge_name": "第三次世界大战",
        "sum_count": 1,
    },
    {"challenge_id": 1639187970865166, "challenge_name": "火星任务", "sum_count": 1},
    {"challenge_id": 1673520406125576, "challenge_name": "歪嘴战神", "sum_count": 1},
    {
        "challenge_id": 1677909448693767,
        "challenge_name": "长春交警回应购置多辆36万哈雷摩托",
        "sum_count": 1,
    },
    {"challenge_id": 1586398694681629, "challenge_name": "泰森", "sum_count": 1},
    {"challenge_id": 1672145091356685, "challenge_name": "小左小飞", "sum_count": 1},
    {"challenge_id": 1585684464338958, "challenge_name": "大嫂", "sum_count": 1},
    {"challenge_id": 1581422403747854, "challenge_name": "陪你长大", "sum_count": 1},
    {"challenge_id": 1587732966845453, "challenge_name": "动物搞笑", "sum_count": 1},
    {"challenge_id": 1612110499415107, "challenge_name": "干邑", "sum_count": 1},
    {"challenge_id": 1586924650664990, "challenge_name": "特征", "sum_count": 1},
    {
        "challenge_id": 1666577700830221,
        "challenge_name": "高校封闭式管理",
        "sum_count": 1,
    },
    {"challenge_id": 1615181093035021, "challenge_name": "抖in邯郸", "sum_count": 1},
    {"challenge_id": 1576683632738365, "challenge_name": "厨房", "sum_count": 1},
    {"challenge_id": 1582695343246350, "challenge_name": "imissyou", "sum_count": 1},
    {"challenge_id": 1628786951671822, "challenge_name": "房地产销售", "sum_count": 1},
    {"challenge_id": 1599343967125571, "challenge_name": "陆小千", "sum_count": 1},
    {"challenge_id": 1635658190129155, "challenge_name": "印象陕西", "sum_count": 1},
    {"challenge_id": 1577862026958862, "challenge_name": "中秋", "sum_count": 1},
    {
        "challenge_id": 1678056804455502,
        "challenge_name": "大唐不倒翁小姐姐结婚啦",
        "sum_count": 1,
    },
    {"challenge_id": 1648622853722125, "challenge_name": "燕姿语录", "sum_count": 1},
    {"challenge_id": 1662333730754583, "challenge_name": "人民币升值", "sum_count": 1},
    {"challenge_id": 1638324274569219, "challenge_name": "金丰", "sum_count": 1},
    {"challenge_id": 1624903293596680, "challenge_name": "山火", "sum_count": 1},
    {"challenge_id": 1581820405393422, "challenge_name": "不脱妆", "sum_count": 1},
    {"challenge_id": 1580793061983245, "challenge_name": "金星", "sum_count": 1},
    {
        "challenge_id": 1677185119826957,
        "challenge_name": "全国抗击新冠疫情表彰大会",
        "sum_count": 1,
    },
    {
        "challenge_id": 1591734401592334,
        "challenge_name": "实习医生格蕾",
        "sum_count": 1,
    },
    {"challenge_id": 1599053443275789, "challenge_name": "清东陵", "sum_count": 1},
    {"challenge_id": 1654723571981319, "challenge_name": "蚂蚁男孩", "sum_count": 1},
    {"challenge_id": 1617543784284237, "challenge_name": "恋恋江湖", "sum_count": 1},
    {"challenge_id": 1578501234421774, "challenge_name": "伤感情歌", "sum_count": 1},
    {"challenge_id": 1562377441705986, "challenge_name": "霸气", "sum_count": 1},
    {"challenge_id": 1566835995527169, "challenge_name": "火柴人", "sum_count": 1},
    {"challenge_id": 1571175811214354, "challenge_name": "故宫", "sum_count": 1},
    {
        "challenge_id": 1652235194191875,
        "challenge_name": "迪士尼在逃公主",
        "sum_count": 1,
    },
    {
        "challenge_id": 1598528840213511,
        "challenge_name": "海底捞过生日",
        "sum_count": 1,
    },
    {"challenge_id": 1583232419617805, "challenge_name": "媳妇", "sum_count": 1},
    {"challenge_id": 1664781389541387, "challenge_name": "木子影视", "sum_count": 1},
    {
        "challenge_id": 1620285014597699,
        "challenge_name": "农村生活趣事",
        "sum_count": 1,
    },
    {"challenge_id": 1582295147660301, "challenge_name": "好人有好报", "sum_count": 1},
    {"challenge_id": 1634210287258632, "challenge_name": "爱的迫降", "sum_count": 1},
    {"challenge_id": 1584187766882317, "challenge_name": "学校", "sum_count": 1},
    {"challenge_id": 1641126039975939, "challenge_name": "翡翠原石", "sum_count": 1},
    {"challenge_id": 1677787091644429, "challenge_name": "冲浪自拍", "sum_count": 1},
    {
        "challenge_id": 1678063109959688,
        "challenge_name": "女子被前夫烧伤",
        "sum_count": 1,
    },
    {"challenge_id": 1588808075227144, "challenge_name": "长安", "sum_count": 1},
    {"challenge_id": 1610465313756168, "challenge_name": "鸿蒙", "sum_count": 1},
    {"challenge_id": 1584229343311901, "challenge_name": "扎马尾", "sum_count": 1},
    {
        "challenge_id": 1660042546489352,
        "challenge_name": "境外输入病例",
        "sum_count": 1,
    },
    {"challenge_id": 1640120453991428, "challenge_name": "偶遇hlt舞团", "sum_count": 1},
    {"challenge_id": 1582400406505485, "challenge_name": "走进非洲", "sum_count": 1},
    {"challenge_id": 1561380767818754, "challenge_name": "秀恩爱", "sum_count": 1},
    {"challenge_id": 1575952992932877, "challenge_name": "艺术家", "sum_count": 1},
    {"challenge_id": 1583053686383629, "challenge_name": "查寝", "sum_count": 1},
    {"challenge_id": 1586422037102606, "challenge_name": "西游伏魔", "sum_count": 1},
    {
        "challenge_id": 1600954991335427,
        "challenge_name": "河北师范大学汇华学院",
        "sum_count": 1,
    },
    {"challenge_id": 1588669176047623, "challenge_name": "急诊科", "sum_count": 1},
    {"challenge_id": 1573681561337870, "challenge_name": "少林寺", "sum_count": 1},
    {"challenge_id": 1583119268924429, "challenge_name": "一问三不知", "sum_count": 1},
    {
        "challenge_id": 1644459856974851,
        "challenge_name": "迪丽热巴为杨幂庆生",
        "sum_count": 1,
    },
    {"challenge_id": 1615367222509575, "challenge_name": "精品制造", "sum_count": 1},
    {
        "challenge_id": 1642366439741452,
        "challenge_name": "我的观影记录",
        "sum_count": 1,
    },
    {"challenge_id": 1592398891655171, "challenge_name": "70", "sum_count": 1},
    {"challenge_id": 1660602334875659, "challenge_name": "黑袍纠察队2", "sum_count": 1},
    {"challenge_id": 1678036284150791, "challenge_name": "秦海璐哭戏", "sum_count": 1},
    {"challenge_id": 1567134502975490, "challenge_name": "动作", "sum_count": 1},
    {"challenge_id": 1669742912490504, "challenge_name": "张哥的日常", "sum_count": 1},
    {"challenge_id": 1549970354830338, "challenge_name": "赵丽颖", "sum_count": 1},
    {
        "challenge_id": 1656769837380621,
        "challenge_name": "青木林里青木秧",
        "sum_count": 1,
    },
    {"challenge_id": 1594028316336142, "challenge_name": "小富婆", "sum_count": 1},
    {"challenge_id": 1596484080413699, "challenge_name": "泰国选美", "sum_count": 1},
    {"challenge_id": 1633670921550859, "challenge_name": "捕蜂", "sum_count": 1},
    {"challenge_id": 1610138706154504, "challenge_name": "感恩教师节", "sum_count": 1},
    {
        "challenge_id": 1665119970510862,
        "challenge_name": "中年夫妻亲一口",
        "sum_count": 1,
    },
    {"challenge_id": 1565257567881217, "challenge_name": "玉", "sum_count": 1},
    {
        "challenge_id": 1674280930539524,
        "challenge_name": "喜欢你就啵一下",
        "sum_count": 1,
    },
    {"challenge_id": 1556676689187842, "challenge_name": "生日快乐", "sum_count": 1},
    {"challenge_id": 1556745225698305, "challenge_name": "黄子韬", "sum_count": 1},
    {"challenge_id": 1567446593143809, "challenge_name": "失望", "sum_count": 1},
    {
        "challenge_id": 1637140660854788,
        "challenge_name": "重启之极海听雷",
        "sum_count": 1,
    },
    {"challenge_id": 1580726828606478, "challenge_name": "何润东", "sum_count": 1},
    {"challenge_id": 1580399790457870, "challenge_name": "假期生活", "sum_count": 1},
    {"challenge_id": 1635411265510404, "challenge_name": "洗冤录1", "sum_count": 1},
    {"challenge_id": 1581241254532109, "challenge_name": "科学实验", "sum_count": 1},
    {
        "challenge_id": 1661801178829837,
        "challenge_name": "致女人余生爱自己",
        "sum_count": 1,
    },
    {"challenge_id": 1640665680296973, "challenge_name": "妇产医生", "sum_count": 1},
    {
        "challenge_id": 1619297450116099,
        "challenge_name": "滴水之恩定当涌泉相报",
        "sum_count": 1,
    },
    {
        "challenge_id": 1606775039695876,
        "challenge_name": "济南海市蜃楼",
        "sum_count": 1,
    },
    {"challenge_id": 1579513938015277, "challenge_name": "羡慕的爱情", "sum_count": 1},
    {
        "challenge_id": 1578258118940685,
        "challenge_name": "爸爸的小情人",
        "sum_count": 1,
    },
    {"challenge_id": 1662860287301639, "challenge_name": "转速测试", "sum_count": 1},
    {"challenge_id": 1600908836169741, "challenge_name": "津巴布韦", "sum_count": 1},
    {"challenge_id": 1584416484000781, "challenge_name": "情感文字", "sum_count": 1},
    {"challenge_id": 1587140026902542, "challenge_name": "急诊", "sum_count": 1},
    {
        "challenge_id": 1620812092975245,
        "challenge_name": "感谢人民子弟兵",
        "sum_count": 1,
    },
    {"challenge_id": 1583976030432269, "challenge_name": "直男", "sum_count": 1},
    {"challenge_id": 1593663355710468, "challenge_name": "奇怪的搭档", "sum_count": 1},
    {"challenge_id": 1583817833419806, "challenge_name": "朱亚文", "sum_count": 1},
    {
        "challenge_id": 1617608983372814,
        "challenge_name": "癞蛤蟆想吃天鹅肉",
        "sum_count": 1,
    },
    {"challenge_id": 1612263674799175, "challenge_name": "秀场直击", "sum_count": 1},
    {
        "challenge_id": 1579239435317277,
        "challenge_name": "劳动者是最美的人",
        "sum_count": 1,
    },
    {
        "challenge_id": 1661315663220760,
        "challenge_name": "球星宅家做什么",
        "sum_count": 1,
    },
    {"challenge_id": 1574716170632205, "challenge_name": "狮子", "sum_count": 1},
    {"challenge_id": 1565756422349826, "challenge_name": "6", "sum_count": 1},
    {"challenge_id": 1605687500669960, "challenge_name": "绿里奇迹", "sum_count": 1},
    {"challenge_id": 1632869330219016, "challenge_name": "和平精英", "sum_count": 1},
    {"challenge_id": 1585673106464781, "challenge_name": "语文", "sum_count": 1},
    {"challenge_id": 1574088252466190, "challenge_name": "宝贝回家", "sum_count": 1},
    {
        "challenge_id": 1677546006239246,
        "challenge_name": "普通高校年度宿舍评选",
        "sum_count": 1,
    },
    {"challenge_id": 1611406112616451, "challenge_name": "下集更精彩", "sum_count": 1},
    {"challenge_id": 1582467522072589, "challenge_name": "搞笑动物", "sum_count": 1},
    {
        "challenge_id": 1676878916769804,
        "challenge_name": "就这样呆在抚仙湖",
        "sum_count": 1,
    },
    {"challenge_id": 1553508302124033, "challenge_name": "画画", "sum_count": 1},
    {"challenge_id": 1598970232075287, "challenge_name": "动作电影", "sum_count": 1},
    {"challenge_id": 1632152390128654, "challenge_name": "退伍", "sum_count": 1},
    {
        "challenge_id": 1677359943846915,
        "challenge_name": "戚薇调侃张翰走不出初恋",
        "sum_count": 1,
    },
    {"challenge_id": 1601350765343752, "challenge_name": "蟹坚强", "sum_count": 1},
    {"challenge_id": 1622615639205960, "challenge_name": "21考研", "sum_count": 1},
    {
        "challenge_id": 1677900993318926,
        "challenge_name": "结婚八周年提离婚",
        "sum_count": 1,
    },
    {"challenge_id": 1608420634479623, "challenge_name": "瘦身男女", "sum_count": 1},
    {"challenge_id": 1586305615341645, "challenge_name": "猪猪侠", "sum_count": 1},
    {
        "challenge_id": 1676150492888068,
        "challenge_name": "美好沈阳dou起来2020",
        "sum_count": 1,
    },
    {"challenge_id": 1570246339149825, "challenge_name": "林志颖", "sum_count": 1},
    {
        "challenge_id": 1620155694572557,
        "challenge_name": "不知道以后便宜了谁",
        "sum_count": 1,
    },
    {"challenge_id": 1576786374239245, "challenge_name": "中国足球", "sum_count": 1},
    {"challenge_id": 1662116057120781, "challenge_name": "德云斗笑社", "sum_count": 1},
    {"challenge_id": 1590098383143950, "challenge_name": "抖瓜助农", "sum_count": 1},
    {"challenge_id": 1574692756074509, "challenge_name": "农业", "sum_count": 1},
    {"challenge_id": 1591286513939470, "challenge_name": "钟楚曦", "sum_count": 1},
    {"challenge_id": 1658742886397966, "challenge_name": "天赋异禀", "sum_count": 1},
    {"challenge_id": 1601622751669251, "challenge_name": "王志文", "sum_count": 1},
    {"challenge_id": 1614655529765892, "challenge_name": "宠物伴侣", "sum_count": 1},
    {"challenge_id": 1589548952178702, "challenge_name": "赶海", "sum_count": 1},
    {"challenge_id": 1598406756495363, "challenge_name": "王道", "sum_count": 1},
    {"challenge_id": 1589563883270158, "challenge_name": "产后", "sum_count": 1},
    {
        "challenge_id": 1677611976970253,
        "challenge_name": "数百人连夜献血救的孩子走了",
        "sum_count": 1,
    },
    {"challenge_id": 1570772434029569, "challenge_name": "龙凤胎", "sum_count": 1},
    {"challenge_id": 1561758457486338, "challenge_name": "甄嬛传", "sum_count": 1},
    {"challenge_id": 1615401098377288, "challenge_name": "杀生", "sum_count": 1},
    {
        "challenge_id": 1677171934436359,
        "challenge_name": "这老师崩溃了",
        "sum_count": 1,
    },
    {"challenge_id": 1573436272875597, "challenge_name": "萌宠成精了", "sum_count": 1},
    {"challenge_id": 1677509210621965, "challenge_name": "白上仙毁号", "sum_count": 1},
    {"challenge_id": 1612771508729860, "challenge_name": "如何自救", "sum_count": 1},
    {"challenge_id": 1593670849339399, "challenge_name": "抖瓜上热门", "sum_count": 1},
    {
        "challenge_id": 1677630234664968,
        "challenge_name": "7000英尺高空点燃降落伞",
        "sum_count": 1,
    },
    {"challenge_id": 1668648446807053, "challenge_name": "男科小知识", "sum_count": 1},
    {"challenge_id": 1646912797467659, "challenge_name": "小棉袄和皮", "sum_count": 1},
    {"challenge_id": 1574537411291149, "challenge_name": "问题", "sum_count": 1},
    {"challenge_id": 1573534100381710, "challenge_name": "女友", "sum_count": 1},
    {
        "challenge_id": 1677512449144839,
        "challenge_name": "黄子韬爸爸去世",
        "sum_count": 1,
    },
    {"challenge_id": 1611654029549645, "challenge_name": "灵异", "sum_count": 1},
    {"challenge_id": 1674105286041607, "challenge_name": "emui11", "sum_count": 1},
    {"challenge_id": 1600400369990728, "challenge_name": "三哥苗侨伟", "sum_count": 1},
    {"challenge_id": 1600186417059853, "challenge_name": "都市天际线", "sum_count": 1},
    {
        "challenge_id": 1644386104826888,
        "challenge_name": "河南搞笑方言",
        "sum_count": 1,
    },
    {
        "challenge_id": 1616177245616132,
        "challenge_name": "特效也太好看了吧",
        "sum_count": 1,
    },
    {"challenge_id": 1582146179668045, "challenge_name": "周慧敏", "sum_count": 1},
    {"challenge_id": 1572959399020574, "challenge_name": "流浪猫", "sum_count": 1},
    {"challenge_id": 1563573645513730, "challenge_name": "飙演技", "sum_count": 1},
    {"challenge_id": 1582590324298782, "challenge_name": "寻人", "sum_count": 1},
    {"challenge_id": 1603974086125640, "challenge_name": "人人有责", "sum_count": 1},
    {"challenge_id": 1612041742429192, "challenge_name": "胆小慎入", "sum_count": 1},
    {"challenge_id": 1596742749195278, "challenge_name": "美队", "sum_count": 1},
    {"challenge_id": 1589712932813828, "challenge_name": "热门热门", "sum_count": 1},
    {
        "challenge_id": 1675254383004701,
        "challenge_name": "别克微蓝6插电混动车",
        "sum_count": 1,
    },
    {
        "challenge_id": 1675968026417156,
        "challenge_name": "猜到结果算我输",
        "sum_count": 1,
    },
    {
        "challenge_id": 1611821392644167,
        "challenge_name": "我在抖瓜看动画",
        "sum_count": 1,
    },
    {"challenge_id": 1581868493514766, "challenge_name": "讽刺", "sum_count": 1},
    {"challenge_id": 1580841695213582, "challenge_name": "80后", "sum_count": 1},
    {"challenge_id": 1672084619532302, "challenge_name": "山东我爱了", "sum_count": 1},
    {"challenge_id": 1581882456913933, "challenge_name": "狗子成精了", "sum_count": 1},
    {
        "challenge_id": 1677882625889293,
        "challenge_name": "美剧生活觉醒",
        "sum_count": 1,
    },
    {
        "challenge_id": 1608741876060168,
        "challenge_name": "狗是人类好朋友",
        "sum_count": 1,
    },
    {"challenge_id": 1570418615429122, "challenge_name": "做家务", "sum_count": 1},
    {"challenge_id": 1615675736712195, "challenge_name": "大能", "sum_count": 1},
    {"challenge_id": 1563472543645697, "challenge_name": "翡翠", "sum_count": 1},
    {
        "challenge_id": 1627340439844867,
        "challenge_name": "感受下沙雕女孩的欢乐日常",
        "sum_count": 1,
    },
    {"challenge_id": 1668472132018205, "challenge_name": "猛男张张", "sum_count": 1},
    {"challenge_id": 1612761540377603, "challenge_name": "弟媳妇", "sum_count": 1},
    {"challenge_id": 1590840181645325, "challenge_name": "接头霸王", "sum_count": 1},
    {"challenge_id": 1586053736721421, "challenge_name": "午马", "sum_count": 1},
    {
        "challenge_id": 1585929301823502,
        "challenge_name": "老一辈的爱情",
        "sum_count": 1,
    },
    {"challenge_id": 1590571743868935, "challenge_name": "怼人", "sum_count": 1},
    {"challenge_id": 1590410588640260, "challenge_name": "演习", "sum_count": 1},
    {
        "challenge_id": 1607962018424840,
        "challenge_name": "喜欢双击关注",
        "sum_count": 1,
    },
    {"challenge_id": 1594641143242756, "challenge_name": "男朋友日常", "sum_count": 1},
    {"challenge_id": 1568000470689793, "challenge_name": "超人", "sum_count": 1},
    {"challenge_id": 1618380486819844, "challenge_name": "四川麻将", "sum_count": 1},
    {"challenge_id": 1563374562157570, "challenge_name": "哈尔滨", "sum_count": 1},
    {"challenge_id": 1584917575635981, "challenge_name": "安全出行", "sum_count": 1},
    {"challenge_id": 1566703867894786, "challenge_name": "古装", "sum_count": 1},
    {"challenge_id": 1664101456252942, "challenge_name": "doujia热门", "sum_count": 1},
    {"challenge_id": 1611324280595460, "challenge_name": "影视片段", "sum_count": 1},
    {"challenge_id": 1605151999277064, "challenge_name": "变相怪杰2", "sum_count": 1},
    {"challenge_id": 1592892713607181, "challenge_name": "南派三叔", "sum_count": 1},
    {
        "challenge_id": 1605153795369997,
        "challenge_name": "育儿经验分享",
        "sum_count": 1,
    },
    {"challenge_id": 1596090850617347, "challenge_name": "苹果发布会", "sum_count": 1},
    {
        "challenge_id": 1668992348781572,
        "challenge_name": "wo的高能瞬间",
        "sum_count": 1,
    },
    {"challenge_id": 1670573354145799, "challenge_name": "nba全心开启", "sum_count": 1},
    {"challenge_id": 1573639965696014, "challenge_name": "远离赌博", "sum_count": 1},
    {"challenge_id": 1659238106127373, "challenge_name": "真情大调查", "sum_count": 1},
    {
        "challenge_id": 1678041593370638,
        "challenge_name": "武汉某超市拒收五角人民币",
        "sum_count": 1,
    },
    {"challenge_id": 1606225989208248, "challenge_name": "曼德拉效应", "sum_count": 1},
    {
        "challenge_id": 1612146752509955,
        "challenge_name": "谢谢大家的支持和关注",
        "sum_count": 1,
    },
    {"challenge_id": 1589927699503117, "challenge_name": "考试成绩", "sum_count": 1},
    {"challenge_id": 1677990728422423, "challenge_name": "天猫好房", "sum_count": 1},
    {"challenge_id": 1664560944556045, "challenge_name": "骗婚", "sum_count": 1},
    {"challenge_id": 1612531371968525, "challenge_name": "王贵与安娜", "sum_count": 1},
    {"challenge_id": 1591096791377928, "challenge_name": "现在就告白", "sum_count": 1},
    {"challenge_id": 1609286195385415, "challenge_name": "奔驰sls", "sum_count": 1},
    {"challenge_id": 1595378329838596, "challenge_name": "老外真会玩", "sum_count": 1},
    {"challenge_id": 1668535668014093, "challenge_name": "嗅觉神探", "sum_count": 1},
    {"challenge_id": 1575344689274893, "challenge_name": "青岛", "sum_count": 1},
    {"challenge_id": 1583515479261197, "challenge_name": "好婆婆", "sum_count": 1},
    {"challenge_id": 1569238769007617, "challenge_name": "拆", "sum_count": 1},
    {
        "challenge_id": 1646908860810251,
        "challenge_name": "请大家看好自己的孩子",
        "sum_count": 1,
    },
    {"challenge_id": 1592198715425805, "challenge_name": "人间烟火", "sum_count": 1},
    {"challenge_id": 1582767063230477, "challenge_name": "一年又一年", "sum_count": 1},
    {"challenge_id": 1647805967352846, "challenge_name": "沙雕婆媳", "sum_count": 1},
    {"challenge_id": 1608646047503367, "challenge_name": "5g时代", "sum_count": 1},
    {"challenge_id": 1581290902116365, "challenge_name": "禅绕画", "sum_count": 1},
    {"challenge_id": 1571898873497602, "challenge_name": "山东", "sum_count": 1},
    {
        "challenge_id": 1672095835374605,
        "challenge_name": "情感藏在细节",
        "sum_count": 1,
    },
    {"challenge_id": 1660376984924174, "challenge_name": "法治天下", "sum_count": 1},
    {"challenge_id": 1601717880845406, "challenge_name": "叶子楣", "sum_count": 1},
    {"challenge_id": 1573921697643598, "challenge_name": "俄罗斯", "sum_count": 1},
    {"challenge_id": 1650623684249608, "challenge_name": "韩国财阀", "sum_count": 1},
    {
        "challenge_id": 1677685302300679,
        "challenge_name": "湖人时隔10年再次闯进西决",
        "sum_count": 1,
    },
    {"challenge_id": 1584100476679181, "challenge_name": "收纳", "sum_count": 1},
    {"challenge_id": 1592843434051592, "challenge_name": "原相机", "sum_count": 1},
    {"challenge_id": 1608382254168071, "challenge_name": "李保田", "sum_count": 1},
    {
        "challenge_id": 1666936642857997,
        "challenge_name": "父母养老房子改造",
        "sum_count": 1,
    },
    {"challenge_id": 1670172613641287, "challenge_name": "宁静真香", "sum_count": 1},
    {
        "challenge_id": 1614742329370638,
        "challenge_name": "台球是一种生活",
        "sum_count": 1,
    },
    {
        "challenge_id": 1594581159752717,
        "challenge_name": "记录美好生活",
        "sum_count": 1,
    },
    {"challenge_id": 1562847370646529, "challenge_name": "时间", "sum_count": 1},
    {"challenge_id": 1585136157378606, "challenge_name": "鉴宝", "sum_count": 1},
    {"challenge_id": 1575929053961230, "challenge_name": "梁咏琪", "sum_count": 1},
    {"challenge_id": 1581242838631438, "challenge_name": "杨紫", "sum_count": 1},
    {"challenge_id": 1663920416611335, "challenge_name": "超能计划", "sum_count": 1},
    {"challenge_id": 1608312489344007, "challenge_name": "徐熙媛大s", "sum_count": 1},
    {"challenge_id": 1589858251087875, "challenge_name": "求热门", "sum_count": 1},
    {"challenge_id": 1595255012690957, "challenge_name": "丽江蓝月谷", "sum_count": 1},
    {"challenge_id": 1659512631761934, "challenge_name": "沙雕动画", "sum_count": 1},
    {"challenge_id": 1573658976636942, "challenge_name": "憋笑挑战", "sum_count": 1},
    {"challenge_id": 1600009028294660, "challenge_name": "tvb港剧", "sum_count": 1},
    {"challenge_id": 1620006139217927, "challenge_name": "商业", "sum_count": 1},
    {"challenge_id": 1589388548682760, "challenge_name": "约翰尼德普", "sum_count": 1},
    {"challenge_id": 1600309374342147, "challenge_name": "奥尼尔", "sum_count": 1},
    {"challenge_id": 1576455385010189, "challenge_name": "柳州", "sum_count": 1},
    {
        "challenge_id": 1584852667083789,
        "challenge_name": "带上同事一起抖瓜",
        "sum_count": 1,
    },
    {"challenge_id": 1621469965079565, "challenge_name": "迷惑行为", "sum_count": 1},
    {
        "challenge_id": 1677959334824967,
        "challenge_name": "广州一公交车从高架桥上冲出护栏跌落地面",
        "sum_count": 1,
    },
    {"challenge_id": 1578225788793870, "challenge_name": "考研", "sum_count": 1},
    {"challenge_id": 1578212207907854, "challenge_name": "百岁老人", "sum_count": 1},
    {"challenge_id": 1590722355323908, "challenge_name": "卡友", "sum_count": 1},
    {"challenge_id": 1674171064623116, "challenge_name": "胡阳光", "sum_count": 1},
    {"challenge_id": 1588634933749773, "challenge_name": "性感", "sum_count": 1},
    {"challenge_id": 1584702608183326, "challenge_name": "特化师", "sum_count": 1},
    {"challenge_id": 1653430952334339, "challenge_name": "禁毒", "sum_count": 1},
    {"challenge_id": 1564724072144897, "challenge_name": "钓鱼", "sum_count": 1},
    {"challenge_id": 1591456221443086, "challenge_name": "于海", "sum_count": 1},
    {"challenge_id": 1628329366688775, "challenge_name": "齐迹", "sum_count": 1},
    {
        "challenge_id": 1657590362241036,
        "challenge_name": "丑脸大头恰恰恰",
        "sum_count": 1,
    },
    {"challenge_id": 1567521760332802, "challenge_name": "酒", "sum_count": 1},
    {"challenge_id": 1574956339749917, "challenge_name": "护花使者", "sum_count": 1},
    {"challenge_id": 1563925543518209, "challenge_name": "牛逼", "sum_count": 1},
    {"challenge_id": 1667975035693064, "challenge_name": "往往老布", "sum_count": 1},
    {"challenge_id": 1671939214611470, "challenge_name": "模仿宁静", "sum_count": 1},
    {"challenge_id": 1611136279758852, "challenge_name": "合租", "sum_count": 1},
    {"challenge_id": 1580405916581966, "challenge_name": "总裁", "sum_count": 1},
    {"challenge_id": 1573898373478413, "challenge_name": "大美中国", "sum_count": 1},
    {
        "challenge_id": 1588924635539460,
        "challenge_name": "没见过这么嚣张的人",
        "sum_count": 1,
    },
    {"challenge_id": 1587832089958653, "challenge_name": "房价", "sum_count": 1},
    {"challenge_id": 1571890916364305, "challenge_name": "视频制作", "sum_count": 1},
    {
        "challenge_id": 1676878600674307,
        "challenge_name": "好油调出balance",
        "sum_count": 1,
    },
    {"challenge_id": 1614524718545923, "challenge_name": "利哥", "sum_count": 1},
    {"challenge_id": 1606319608582211, "challenge_name": "纠正驼背", "sum_count": 1},
    {"challenge_id": 1583674460147725, "challenge_name": "平凡的世界", "sum_count": 1},
    {"challenge_id": 1599406471560199, "challenge_name": "护师", "sum_count": 1},
    {"challenge_id": 1614639698996237, "challenge_name": "鲳鱼", "sum_count": 1},
    {"challenge_id": 1666672504755208, "challenge_name": "殊死七日", "sum_count": 1},
    {
        "challenge_id": 1618752580046851,
        "challenge_name": "迪士尼花木兰",
        "sum_count": 1,
    },
    {"challenge_id": 1592436917069831, "challenge_name": "电影电影", "sum_count": 1},
    {"challenge_id": 1603878075494430, "challenge_name": "最美的安排", "sum_count": 1},
    {"challenge_id": 1604615532549127, "challenge_name": "斗腕魏思澄", "sum_count": 1},
    {"challenge_id": 1562582800230402, "challenge_name": "跑酷", "sum_count": 1},
    {"challenge_id": 1568842117936129, "challenge_name": "搏击", "sum_count": 1},
    {"challenge_id": 1593434926171139, "challenge_name": "塞班军舰岛", "sum_count": 1},
    {"challenge_id": 1642758208234504, "challenge_name": "特战队", "sum_count": 1},
    {"challenge_id": 1675996038535171, "challenge_name": "小海豹钉蹄", "sum_count": 1},
    {
        "challenge_id": 1649778298342414,
        "challenge_name": "记录我的农村生活点滴",
        "sum_count": 1,
    },
    {"challenge_id": 1565202179611650, "challenge_name": "丑女逆袭", "sum_count": 1},
    {"challenge_id": 1553351219767298, "challenge_name": "恶作剧", "sum_count": 1},
    {"challenge_id": 1560120442414082, "challenge_name": "手艺人", "sum_count": 1},
    {"challenge_id": 1586385013745678, "challenge_name": "国家文物", "sum_count": 1},
    {"challenge_id": 1607514828379140, "challenge_name": "参军", "sum_count": 1},
    {"challenge_id": 1572075637504002, "challenge_name": "治愈系笑容", "sum_count": 1},
    {"challenge_id": 1619652822451207, "challenge_name": "古玩收藏", "sum_count": 1},
    {"challenge_id": 1562847247126529, "challenge_name": "贾斯汀比伯", "sum_count": 1},
    {"challenge_id": 1631018123740163, "challenge_name": "空姐卸妆", "sum_count": 1},
    {"challenge_id": 1579795900949518, "challenge_name": "宝格丽", "sum_count": 1},
    {
        "challenge_id": 1658045706846216,
        "challenge_name": "经典电视剧剪辑",
        "sum_count": 1,
    },
    {"challenge_id": 1641292731996164, "challenge_name": "斩风刀", "sum_count": 1},
    {
        "challenge_id": 1677528942334990,
        "challenge_name": "大爷怒斥小伙纹身",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677506810015751,
        "challenge_name": "7000的玩具车",
        "sum_count": 1,
    },
    {"challenge_id": 1565441306550273, "challenge_name": "杨幂", "sum_count": 1},
    {"challenge_id": 1555927198663681, "challenge_name": "寒假", "sum_count": 1},
    {"challenge_id": 1581570824002573, "challenge_name": "好车抖一抖", "sum_count": 1},
    {"challenge_id": 1569839505411073, "challenge_name": "有趣", "sum_count": 1},
    {"challenge_id": 1586952198827022, "challenge_name": "我的十年", "sum_count": 1},
    {"challenge_id": 1610032296928264, "challenge_name": "宝妈分享", "sum_count": 1},
    {
        "challenge_id": 1614196636738567,
        "challenge_name": "关注我持续更新",
        "sum_count": 1,
    },
    {"challenge_id": 1573269976696846, "challenge_name": "斯诺克", "sum_count": 1},
    {
        "challenge_id": 1641752119728142,
        "challenge_name": "老娘舅调解语录",
        "sum_count": 1,
    },
    {"challenge_id": 1549870620673025, "challenge_name": "上课", "sum_count": 1},
    {"challenge_id": 1592569857467399, "challenge_name": "无敌小可爱", "sum_count": 1},
    {"challenge_id": 1569702920352769, "challenge_name": "民谣", "sum_count": 1},
    {
        "challenge_id": 1566477444054018,
        "challenge_name": "新白娘子传奇",
        "sum_count": 1,
    },
    {
        "challenge_id": 1558864984679425,
        "challenge_name": "三生三世十里桃花",
        "sum_count": 1,
    },
    {
        "challenge_id": 1661321673949191,
        "challenge_name": "我的影视剪辑报告",
        "sum_count": 1,
    },
    {"challenge_id": 1613619221356637, "challenge_name": "生还", "sum_count": 1},
    {"challenge_id": 1624239648859149, "challenge_name": "新闻启示录", "sum_count": 1},
    {
        "challenge_id": 1677860913667143,
        "challenge_name": "熊闯入游泳池边闻男子脚",
        "sum_count": 1,
    },
    {"challenge_id": 1582833682484237, "challenge_name": "领证", "sum_count": 1},
    {"challenge_id": 1647377396130827, "challenge_name": "沙雕父子组", "sum_count": 1},
    {"challenge_id": 1577676831125517, "challenge_name": "乡村爱情", "sum_count": 1},
    {"challenge_id": 1565655142095874, "challenge_name": "超跑", "sum_count": 1},
    {"challenge_id": 1590115768359950, "challenge_name": "邱淑贞", "sum_count": 1},
    {"challenge_id": 1616929124964420, "challenge_name": "动作喜剧", "sum_count": 1},
    {"challenge_id": 1593357130209293, "challenge_name": "乱世三义", "sum_count": 1},
    {"challenge_id": 1590894980925470, "challenge_name": "挖掘机", "sum_count": 1},
    {"challenge_id": 1566730420065281, "challenge_name": "直升机", "sum_count": 1},
    {"challenge_id": 1576512250548237, "challenge_name": "纠结", "sum_count": 1},
    {
        "challenge_id": 1677870430410766,
        "challenge_name": "76岁演员陈惠敏承认患癌",
        "sum_count": 1,
    },
    {"challenge_id": 1587023431537677, "challenge_name": "黄土高原", "sum_count": 1},
    {"challenge_id": 1661043260419076, "challenge_name": "迷惑螺旋桨", "sum_count": 1},
    {"challenge_id": 1626417382928395, "challenge_name": "负心的你", "sum_count": 1},
    {"challenge_id": 1567055721122817, "challenge_name": "工作", "sum_count": 1},
    {
        "challenge_id": 1613021983165454,
        "challenge_name": "潮州职业技术学校",
        "sum_count": 1,
    },
    {"challenge_id": 1594581079902221, "challenge_name": "中国手艺人", "sum_count": 1},
    {"challenge_id": 1589848982707208, "challenge_name": "黄鳝", "sum_count": 1},
    {"challenge_id": 1604866345539614, "challenge_name": "流鼻血", "sum_count": 1},
    {"challenge_id": 1625779635030029, "challenge_name": "草率了", "sum_count": 1},
    {"challenge_id": 1676596201948173, "challenge_name": "肖战火锅局", "sum_count": 1},
    {
        "challenge_id": 1673533461243912,
        "challenge_name": "亿想不到的美味",
        "sum_count": 1,
    },
    {"challenge_id": 1588047181311053, "challenge_name": "心理学", "sum_count": 1},
    {
        "challenge_id": 1612733525935107,
        "challenge_name": "爱吃爱喝爱生活",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677430798125064,
        "challenge_name": "迪丽热巴照镜子",
        "sum_count": 1,
    },
    {
        "challenge_id": 1658118141673485,
        "challenge_name": "美食趣味计划",
        "sum_count": 1,
    },
    {"challenge_id": 1614910242510878, "challenge_name": "湖人vs火箭", "sum_count": 1},
    {"challenge_id": 1569890451562498, "challenge_name": "精神小伙", "sum_count": 1},
    {"challenge_id": 1633951213163531, "challenge_name": "107头条", "sum_count": 1},
    {"challenge_id": 1668641546799118, "challenge_name": "和老板同居", "sum_count": 1},
    {"challenge_id": 1597783864087559, "challenge_name": "张凯丽", "sum_count": 1},
    {"challenge_id": 1581206925850638, "challenge_name": "蟹王", "sum_count": 1},
    {"challenge_id": 1627161159130119, "challenge_name": "抖瓜科技", "sum_count": 1},
    {"challenge_id": 1604976257476637, "challenge_name": "僵尸至尊", "sum_count": 1},
    {
        "challenge_id": 1562553454665729,
        "challenge_name": "尼古拉斯赵四",
        "sum_count": 1,
    },
    {"challenge_id": 1580534385160205, "challenge_name": "新闻", "sum_count": 1},
    {"challenge_id": 1626237205627907, "challenge_name": "继承者计划", "sum_count": 1},
    {"challenge_id": 1610218586799149, "challenge_name": "秦舒培", "sum_count": 1},
    {"challenge_id": 1585137565862926, "challenge_name": "湖南", "sum_count": 1},
    {"challenge_id": 1600091285823492, "challenge_name": "兵哥哥", "sum_count": 1},
    {"challenge_id": 1577610367174701, "challenge_name": "提神醒脑", "sum_count": 1},
    {"challenge_id": 1591815165039623, "challenge_name": "阿亮", "sum_count": 1},
    {
        "challenge_id": 1583764974129326,
        "challenge_name": "我的爷爷奶奶",
        "sum_count": 1,
    },
    {"challenge_id": 1580368895533070, "challenge_name": "风俗", "sum_count": 1},
    {"challenge_id": 1657487565987851, "challenge_name": "dou热门推广", "sum_count": 1},
    {"challenge_id": 1566604392018945, "challenge_name": "逆袭", "sum_count": 1},
    {"challenge_id": 1587135067847694, "challenge_name": "护士懂护士", "sum_count": 1},
    {
        "challenge_id": 1649079134803975,
        "challenge_name": "秀出你的足迹地图",
        "sum_count": 1,
    },
    {"challenge_id": 1598698761907214, "challenge_name": "教官", "sum_count": 1},
    {"challenge_id": 1600069713291336, "challenge_name": "牛🐂", "sum_count": 1},
    {"challenge_id": 1610752556444679, "challenge_name": "植物", "sum_count": 1},
    {"challenge_id": 1590193696421895, "challenge_name": "老母亲", "sum_count": 1},
    {
        "challenge_id": 1677550052506637,
        "challenge_name": "成都桂花巷20株桂花树遭野蛮砍伐",
        "sum_count": 1,
    },
    {"challenge_id": 1583460791527438, "challenge_name": "婆媳关系", "sum_count": 1},
    {"challenge_id": 1677979999761422, "challenge_name": "一毛钱奶奶", "sum_count": 1},
    {
        "challenge_id": 1580034759669774,
        "challenge_name": "阳澄湖大闸蟹",
        "sum_count": 1,
    },
    {"challenge_id": 1672311367929864, "challenge_name": "哥哥树", "sum_count": 1},
    {"challenge_id": 1583424239095822, "challenge_name": "家长", "sum_count": 1},
    {"challenge_id": 1611409563876360, "challenge_name": "地心抢险记", "sum_count": 1},
    {"challenge_id": 1569628030253121, "challenge_name": "护士", "sum_count": 1},
    {"challenge_id": 1554034322063361, "challenge_name": "自行车", "sum_count": 1},
    {"challenge_id": 1666545285550092, "challenge_name": "英语", "sum_count": 1},
    {"challenge_id": 1586474766056493, "challenge_name": "演练", "sum_count": 1},
    {"challenge_id": 1576065513345053, "challenge_name": "日不落", "sum_count": 1},
    {"challenge_id": 1633327165297677, "challenge_name": "不要限", "sum_count": 1},
    {"challenge_id": 1566759194999809, "challenge_name": "乒乓球", "sum_count": 1},
    {
        "challenge_id": 1670732998476808,
        "challenge_name": "船山区创建天府旅游名县",
        "sum_count": 1,
    },
    {"challenge_id": 1572132571825153, "challenge_name": "道歉", "sum_count": 1},
    {
        "challenge_id": 1677349414027278,
        "challenge_name": "美国俄勒冈州爆发大规模山火",
        "sum_count": 1,
    },
    {"challenge_id": 1561512764259330, "challenge_name": "cp", "sum_count": 1},
    {"challenge_id": 1618284300066823, "challenge_name": "沙雕沙雕", "sum_count": 1},
    {
        "challenge_id": 1627968132282381,
        "challenge_name": "跌跌撞撞奔向你",
        "sum_count": 1,
    },
    {"challenge_id": 1587013951279118, "challenge_name": "腌萝卜", "sum_count": 1},
    {"challenge_id": 1595532349503496, "challenge_name": "隐私", "sum_count": 1},
    {"challenge_id": 1590650291750920, "challenge_name": "中国骄傲", "sum_count": 1},
    {
        "challenge_id": 1636043554174988,
        "challenge_name": "关注我下期更精彩",
        "sum_count": 1,
    },
    {
        "challenge_id": 1614194476395543,
        "challenge_name": "剥皮香蕉馒头",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677369456366604,
        "challenge_name": "女儿离家出走10天父亲录视频唤回家",
        "sum_count": 1,
    },
    {"challenge_id": 1612402101366852, "challenge_name": "新剧来袭", "sum_count": 1},
    {"challenge_id": 1656408078389261, "challenge_name": "抗击疫情", "sum_count": 1},
    {
        "challenge_id": 1607747584509965,
        "challenge_name": "打call我的内容主场",
        "sum_count": 1,
    },
    {"challenge_id": 1574013067411470, "challenge_name": "重庆方言", "sum_count": 1},
    {"challenge_id": 1656208906057735, "challenge_name": "豆荚小助手", "sum_count": 1},
    {"challenge_id": 1599500924279816, "challenge_name": "办公室搞笑", "sum_count": 1},
    {"challenge_id": 1669645439153160, "challenge_name": "3哥搞笑配音", "sum_count": 1},
    {"challenge_id": 1599880972777479, "challenge_name": "安岳", "sum_count": 1},
    {"challenge_id": 1560033080192002, "challenge_name": "姐妹情深", "sum_count": 1},
    {"challenge_id": 1590846637507598, "challenge_name": "懂事的孩子", "sum_count": 1},
    {
        "challenge_id": 1665646289460232,
        "challenge_name": "潮玩人类在哪里",
        "sum_count": 1,
    },
    {"challenge_id": 1589444559883267, "challenge_name": "手机维修", "sum_count": 1},
    {
        "challenge_id": 1674654715014152,
        "challenge_name": "流量福利在这里",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677520322118667,
        "challenge_name": "毒角show为了邓超摘头套",
        "sum_count": 1,
    },
    {
        "challenge_id": 1631248001507341,
        "challenge_name": "打工这辈子是不会打工的",
        "sum_count": 1,
    },
    {"challenge_id": 1570737659003905, "challenge_name": "西安", "sum_count": 1},
    {
        "challenge_id": 1677338652862472,
        "challenge_name": "给物业送锦旗",
        "sum_count": 1,
    },
    {"challenge_id": 1587385113886734, "challenge_name": "乞丐模拟器", "sum_count": 1},
    {
        "challenge_id": 1674798922195976,
        "challenge_name": "我喜欢你开播",
        "sum_count": 1,
    },
    {"challenge_id": 1676893265899534, "challenge_name": "苟金凤", "sum_count": 1},
    {"challenge_id": 1610935344226312, "challenge_name": "教师节送花", "sum_count": 1},
    {
        "challenge_id": 1677644399991820,
        "challenge_name": "男子因彩礼街头暴打女友",
        "sum_count": 1,
    },
    {"challenge_id": 1560110456527873, "challenge_name": "最美手艺人", "sum_count": 1},
    {"challenge_id": 1571084283091010, "challenge_name": "刘亦菲", "sum_count": 1},
    {
        "challenge_id": 1677810346323981,
        "challenge_name": "专家竟从52岁大妈左肩里挤出",
        "sum_count": 1,
    },
    {"challenge_id": 1673813484549128, "challenge_name": "canva教程", "sum_count": 1},
    {
        "challenge_id": 1637497404709892,
        "challenge_name": "我们依然是我们",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677633521127432,
        "challenge_name": "江疏影续演王漫妮",
        "sum_count": 1,
    },
    {"challenge_id": 1607688618777604, "challenge_name": "亲子教育", "sum_count": 1},
    {"challenge_id": 1573995712690189, "challenge_name": "何炅", "sum_count": 1},
    {"challenge_id": 1575709569627150, "challenge_name": "古天乐", "sum_count": 1},
    {"challenge_id": 1610146264065037, "challenge_name": "孙越", "sum_count": 1},
    {"challenge_id": 1555596444813329, "challenge_name": "校园生活", "sum_count": 1},
    {"challenge_id": 1603511945453571, "challenge_name": "孙兴慜", "sum_count": 1},
    {
        "challenge_id": 1677496660123652,
        "challenge_name": "好奇心大作战",
        "sum_count": 1,
    },
    {"challenge_id": 1590559367029763, "challenge_name": "内八字", "sum_count": 1},
    {"challenge_id": 1566997268986882, "challenge_name": "动画", "sum_count": 1},
    {
        "challenge_id": 1671491908639757,
        "challenge_name": "极限挑战宝藏行",
        "sum_count": 1,
    },
    {"challenge_id": 1614367524605975, "challenge_name": "把子肉", "sum_count": 1},
    {"challenge_id": 1599268162183182, "challenge_name": "皮神", "sum_count": 1},
    {"challenge_id": 1659412581199886, "challenge_name": "立坚", "sum_count": 1},
    {"challenge_id": 1590384475615240, "challenge_name": "保罗乔治", "sum_count": 1},
    {
        "challenge_id": 1677712748194887,
        "challenge_name": "兰博基尼街头自燃",
        "sum_count": 1,
    },
    {"challenge_id": 1553962105154561, "challenge_name": "杨洋", "sum_count": 1},
    {"challenge_id": 1592165770108952, "challenge_name": "女性健康", "sum_count": 1},
    {"challenge_id": 1610007354742792, "challenge_name": "平安普惠", "sum_count": 1},
    {
        "challenge_id": 1643089679003655,
        "challenge_name": "三国志战略版",
        "sum_count": 1,
    },
    {"challenge_id": 1575865001691149, "challenge_name": "开学季", "sum_count": 1},
    {"challenge_id": 1568471535935490, "challenge_name": "泰国", "sum_count": 1},
    {"challenge_id": 1590730220314638, "challenge_name": "檀健次", "sum_count": 1},
    {"challenge_id": 1611506383222855, "challenge_name": "蓝军出击", "sum_count": 1},
    {"challenge_id": 1568840755171329, "challenge_name": "马犬", "sum_count": 1},
    {"challenge_id": 1604140579635204, "challenge_name": "二弟", "sum_count": 1},
    {"challenge_id": 1593158120322061, "challenge_name": "面条🍜", "sum_count": 1},
    {
        "challenge_id": 1636291460599815,
        "challenge_name": "淮南交警vlog",
        "sum_count": 1,
    },
    {"challenge_id": 1588110644130845, "challenge_name": "郭京飞", "sum_count": 1},
    {"challenge_id": 1577055091555406, "challenge_name": "开学第一天", "sum_count": 1},
    {"challenge_id": 1676531842194509, "challenge_name": "apple2020", "sum_count": 1},
    {"challenge_id": 1578073549488157, "challenge_name": "小米", "sum_count": 1},
    {
        "challenge_id": 1677410156354574,
        "challenge_name": "徐峥新片预告",
        "sum_count": 1,
    },
    {"challenge_id": 1560045571001346, "challenge_name": "蹦迪", "sum_count": 1},
    {"challenge_id": 1594892141064206, "challenge_name": "霍启刚", "sum_count": 1},
    {"challenge_id": 1677870811959304, "challenge_name": "强迫他人", "sum_count": 1},
    {
        "challenge_id": 1669715754163208,
        "challenge_name": "我在抖瓜都动漫",
        "sum_count": 1,
    },
    {"challenge_id": 1628309420041220, "challenge_name": "抑郁症", "sum_count": 1},
    {"challenge_id": 1608911900488717, "challenge_name": "身材管理", "sum_count": 1},
    {
        "challenge_id": 1665696556659720,
        "challenge_name": "很哇塞的女人",
        "sum_count": 1,
    },
    {"challenge_id": 1674981802564615, "challenge_name": "默默团队", "sum_count": 1},
    {"challenge_id": 1612180629689351, "challenge_name": "炸厨房", "sum_count": 1},
    {"challenge_id": 1568544557198337, "challenge_name": "北海道", "sum_count": 1},
    {"challenge_id": 1592636211842056, "challenge_name": "奇葩亲戚", "sum_count": 1},
    {"challenge_id": 1633661740573699, "challenge_name": "玛莎和熊", "sum_count": 1},
    {"challenge_id": 1588496486919172, "challenge_name": "国球", "sum_count": 1},
    {"challenge_id": 1578666298013710, "challenge_name": "文章", "sum_count": 1},
    {"challenge_id": 1614388130793479, "challenge_name": "散装rap", "sum_count": 1},
    {"challenge_id": 1642927260825608, "challenge_name": "清网行动", "sum_count": 1},
    {"challenge_id": 1595024414973971, "challenge_name": "山海", "sum_count": 1},
    {"challenge_id": 1674082597191693, "challenge_name": "梵音年华", "sum_count": 1},
    {"challenge_id": 1579053392665613, "challenge_name": "养殖", "sum_count": 1},
    {"challenge_id": 1675549742092295, "challenge_name": "中天潮购", "sum_count": 1},
    {
        "challenge_id": 1638030278525955,
        "challenge_name": "717抖瓜嗨吃节",
        "sum_count": 1,
    },
    {"challenge_id": 1583194219327502, "challenge_name": "重庆大学城", "sum_count": 1},
    {"challenge_id": 1603508266643469, "challenge_name": "湖北高速", "sum_count": 1},
    {"challenge_id": 1580213348163598, "challenge_name": "探亲", "sum_count": 1},
    {
        "challenge_id": 1677682748723207,
        "challenge_name": "重启第二季开播",
        "sum_count": 1,
    },
    {"challenge_id": 1572892227475469, "challenge_name": "家人", "sum_count": 1},
    {
        "challenge_id": 1592527770535950,
        "challenge_name": "外国人看抖瓜",
        "sum_count": 1,
    },
    {"challenge_id": 1676697150098435, "challenge_name": "王在清", "sum_count": 1},
    {"challenge_id": 1574718665971725, "challenge_name": "谭松韵", "sum_count": 1},
    {"challenge_id": 1618889419552771, "challenge_name": "文昊", "sum_count": 1},
    {"challenge_id": 1624362684860488, "challenge_name": "canva", "sum_count": 1},
    {"challenge_id": 1544979526789121, "challenge_name": "哈哈哈哈哈", "sum_count": 1},
    {"challenge_id": 1586235561379854, "challenge_name": "t台走秀", "sum_count": 1},
    {
        "challenge_id": 1637292909692932,
        "challenge_name": "感谢官方大大推送",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677506209852424,
        "challenge_name": "老人卖粮款被偷",
        "sum_count": 1,
    },
    {"challenge_id": 1565481181932545, "challenge_name": "前方高能", "sum_count": 1},
    {"challenge_id": 1578692747274254, "challenge_name": "王宝强", "sum_count": 1},
    {"challenge_id": 1661045355972638, "challenge_name": "宫主切", "sum_count": 1},
    {"challenge_id": 1575313074761758, "challenge_name": "稻城亚丁", "sum_count": 1},
    {"challenge_id": 1563986486107138, "challenge_name": "开车", "sum_count": 1},
    {"challenge_id": 1599448214157389, "challenge_name": "集美", "sum_count": 1},
    {"challenge_id": 1601963171439623, "challenge_name": "李庚希", "sum_count": 1},
    {"challenge_id": 1584390730795021, "challenge_name": "孙膑", "sum_count": 1},
    {"challenge_id": 1609840812303432, "challenge_name": "拉黑删除", "sum_count": 1},
    {
        "challenge_id": 1622553413062980,
        "challenge_name": "初恋那件小事",
        "sum_count": 1,
    },
    {"challenge_id": 1559289151965185, "challenge_name": "手写", "sum_count": 1},
    {"challenge_id": 1599599087024141, "challenge_name": "解方程", "sum_count": 1},
    {"challenge_id": 1607122358097069, "challenge_name": "业主", "sum_count": 1},
    {"challenge_id": 1560034530455553, "challenge_name": "成都", "sum_count": 1},
    {"challenge_id": 1602773144185863, "challenge_name": "永不回头", "sum_count": 1},
    {"challenge_id": 1593010838601799, "challenge_name": "李浩菲", "sum_count": 1},
    {"challenge_id": 1608428009655300, "challenge_name": "养育女孩", "sum_count": 1},
    {
        "challenge_id": 1677827324154893,
        "challenge_name": "妈妈唠叨孩子50分钟把自己气跑",
        "sum_count": 1,
    },
    {"challenge_id": 1567811390931970, "challenge_name": "潜水", "sum_count": 1},
    {"challenge_id": 1601062764760071, "challenge_name": "侯梦莎", "sum_count": 1},
    {
        "challenge_id": 1677977854429192,
        "challenge_name": "大学女生在宿舍腌制糖醋姜",
        "sum_count": 1,
    },
    {
        "challenge_id": 1631649052171278,
        "challenge_name": "最强保镖天团",
        "sum_count": 1,
    },
    {"challenge_id": 1612211559179268, "challenge_name": "李民基", "sum_count": 1},
    {"challenge_id": 1583595187422350, "challenge_name": "科目二考试", "sum_count": 1},
    {"challenge_id": 1636327659377692, "challenge_name": "笑动欢乐秀", "sum_count": 1},
    {"challenge_id": 1586365039756318, "challenge_name": "长相守", "sum_count": 1},
    {"challenge_id": 1581135620239390, "challenge_name": "搞笑日常", "sum_count": 1},
    {
        "challenge_id": 1662663755205646,
        "challenge_name": "记录电影人的酸甜苦辣",
        "sum_count": 1,
    },
    {"challenge_id": 1659935844166660, "challenge_name": "球星", "sum_count": 1},
    {"challenge_id": 1655951651689480, "challenge_name": "杜卡迪", "sum_count": 1},
    {"challenge_id": 1577713096475661, "challenge_name": "三国", "sum_count": 1},
    {"challenge_id": 1588647137818628, "challenge_name": "母乳", "sum_count": 1},
    {
        "challenge_id": 1670551953451015,
        "challenge_name": "抖瓜奇妙好物节",
        "sum_count": 1,
    },
    {
        "challenge_id": 1645543382929422,
        "challenge_name": "挑战幼儿园老师",
        "sum_count": 1,
    },
    {"challenge_id": 1585881535148046, "challenge_name": "8090后", "sum_count": 1},
    {"challenge_id": 1586299512457293, "challenge_name": "感谢大家", "sum_count": 1},
    {"challenge_id": 1559915331061761, "challenge_name": "演技派", "sum_count": 1},
    {"challenge_id": 1610485479299075, "challenge_name": "跪坐", "sum_count": 1},
    {"challenge_id": 1580881561351182, "challenge_name": "马", "sum_count": 1},
    {"challenge_id": 1649434197898312, "challenge_name": "红糖小丸子", "sum_count": 1},
    {"challenge_id": 1586378479863837, "challenge_name": "暴富", "sum_count": 1},
    {"challenge_id": 1606143763738638, "challenge_name": "百花香", "sum_count": 1},
    {"challenge_id": 1565831000252433, "challenge_name": "爱的恰恰", "sum_count": 1},
    {"challenge_id": 1583757678399501, "challenge_name": "拼音", "sum_count": 1},
    {"challenge_id": 1587196934411277, "challenge_name": "农村美食", "sum_count": 1},
    {"challenge_id": 1676293182668811, "challenge_name": "konsn", "sum_count": 1},
    {"challenge_id": 1616157402092557, "challenge_name": "赤峰下雪了", "sum_count": 1},
    {"challenge_id": 1574078820326413, "challenge_name": "快递小哥", "sum_count": 1},
    {
        "challenge_id": 1668048489383949,
        "challenge_name": "普通食堂网红店",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677365645304855,
        "challenge_name": "宋丹丹粉色公主裙旧照",
        "sum_count": 1,
    },
    {"challenge_id": 1668927092622350, "challenge_name": "影视剪切师", "sum_count": 1},
    {"challenge_id": 1575861602181134, "challenge_name": "你怎么看", "sum_count": 1},
    {"challenge_id": 1610848415945742, "challenge_name": "盗墓", "sum_count": 1},
    {"challenge_id": 1579215271954461, "challenge_name": "丈母娘", "sum_count": 1},
    {"challenge_id": 1575369394224141, "challenge_name": "郑爽", "sum_count": 1},
    {"challenge_id": 1554416902162434, "challenge_name": "宝贝", "sum_count": 1},
    {
        "challenge_id": 1677869640669197,
        "challenge_name": "6000元离职赔偿金全是硬币",
        "sum_count": 1,
    },
    {"challenge_id": 1578584758711326, "challenge_name": "珍惜", "sum_count": 1},
    {"challenge_id": 1589034066467847, "challenge_name": "汽车小知识", "sum_count": 1},
    {"challenge_id": 1570173814554625, "challenge_name": "背影杀", "sum_count": 1},
    {"challenge_id": 1615490128907272, "challenge_name": "路易十三", "sum_count": 1},
    {"challenge_id": 1591958633563143, "challenge_name": "巩俐", "sum_count": 1},
    {
        "challenge_id": 1612554366616580,
        "challenge_name": "我和我的家乡",
        "sum_count": 1,
    },
    {"challenge_id": 1659021082383372, "challenge_name": "ios14来了", "sum_count": 1},
    {
        "challenge_id": 1674191592952839,
        "challenge_name": "杭城心动地图",
        "sum_count": 1,
    },
    {
        "challenge_id": 1620980854999053,
        "challenge_name": "八百里洞庭我的家",
        "sum_count": 1,
    },
    {"challenge_id": 1588835439117319, "challenge_name": "五迷三道", "sum_count": 1},
    {"challenge_id": 1553051327652865, "challenge_name": "欧美", "sum_count": 1},
    {"challenge_id": 1579705635002381, "challenge_name": "嫁给爱情", "sum_count": 1},
    {"challenge_id": 1576374277730317, "challenge_name": "华熙哥", "sum_count": 1},
    {"challenge_id": 1570521388208129, "challenge_name": "奔驰", "sum_count": 1},
    {"challenge_id": 1612592462954628, "challenge_name": "阿雅", "sum_count": 1},
    {"challenge_id": 1597785015461892, "challenge_name": "正念", "sum_count": 1},
    {"challenge_id": 1589646750185480, "challenge_name": "济南美食", "sum_count": 1},
    {"challenge_id": 1576102316028942, "challenge_name": "逆袭失败", "sum_count": 1},
    {"challenge_id": 1571501579261953, "challenge_name": "黑洞", "sum_count": 1},
    {
        "challenge_id": 1585840639121421,
        "challenge_name": "南方人北方人",
        "sum_count": 1,
    },
    {
        "challenge_id": 1672487422735374,
        "challenge_name": "全部都在那个地方",
        "sum_count": 1,
    },
    {"challenge_id": 1588140691710989, "challenge_name": "李子坝", "sum_count": 1},
    {"challenge_id": 1645092854816772, "challenge_name": "新兵", "sum_count": 1},
    {"challenge_id": 1586221525899278, "challenge_name": "屁", "sum_count": 1},
    {"challenge_id": 1574142789132302, "challenge_name": "短视频", "sum_count": 1},
    {
        "challenge_id": 1583368193224782,
        "challenge_name": "生活需要仪式感",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677548036602887,
        "challenge_name": "军训高校大比舞",
        "sum_count": 1,
    },
    {"challenge_id": 1663489014576132, "challenge_name": "阿森编舞", "sum_count": 1},
    {"challenge_id": 1609878026943502, "challenge_name": "胡慧中", "sum_count": 1},
    {"challenge_id": 1590731539234836, "challenge_name": "驴", "sum_count": 1},
    {
        "challenge_id": 1656860481080332,
        "challenge_name": "战胜疫情dou行动",
        "sum_count": 1,
    },
    {"challenge_id": 1659518605206536, "challenge_name": "网抑云", "sum_count": 1},
    {
        "challenge_id": 1625275798564932,
        "challenge_name": "阿尔法脑波音乐",
        "sum_count": 1,
    },
    {"challenge_id": 1565892214407169, "challenge_name": "后来", "sum_count": 1},
    {
        "challenge_id": 1589016552900616,
        "challenge_name": "中国人在非洲",
        "sum_count": 1,
    },
    {
        "challenge_id": 1672259762286606,
        "challenge_name": "发光的你与众不同",
        "sum_count": 1,
    },
    {"challenge_id": 1608857872691204, "challenge_name": "家暴", "sum_count": 1},
    {"challenge_id": 1584485929434125, "challenge_name": "文艺复兴", "sum_count": 1},
    {"challenge_id": 1584778872891405, "challenge_name": "悍匪", "sum_count": 1},
    {"challenge_id": 1640300629399559, "challenge_name": "轻知识计划", "sum_count": 1},
    {"challenge_id": 1659217199307787, "challenge_name": "原创者联盟", "sum_count": 1},
    {"challenge_id": 1652266872296455, "challenge_name": "成龙", "sum_count": 1},
    {
        "challenge_id": 1677959561693198,
        "challenge_name": "国庆大花篮预计9月25日亮相",
        "sum_count": 1,
    },
    {"challenge_id": 1615833098563741, "challenge_name": "酒驾", "sum_count": 1},
    {"challenge_id": 1628266163326983, "challenge_name": "末代溥仪", "sum_count": 1},
    {"challenge_id": 1579398594967662, "challenge_name": "农家小院", "sum_count": 1},
    {"challenge_id": 1608222952512590, "challenge_name": "玉米地", "sum_count": 1},
    {"challenge_id": 1586660674833518, "challenge_name": "刘邦", "sum_count": 1},
    {"challenge_id": 1627798126772236, "challenge_name": "逃学英雄传", "sum_count": 1},
    {
        "challenge_id": 1594433362720771,
        "challenge_name": "大山深处自然美",
        "sum_count": 1,
    },
    {"challenge_id": 1596051943036931, "challenge_name": "琉璃", "sum_count": 1},
    {
        "challenge_id": 1591850792327197,
        "challenge_name": "莱昂纳多迪卡普里奥",
        "sum_count": 1,
    },
    {"challenge_id": 1592534297609224, "challenge_name": "giao", "sum_count": 1},
    {"challenge_id": 1596540199302164, "challenge_name": "姜武", "sum_count": 1},
    {"challenge_id": 1571245293514754, "challenge_name": "牛", "sum_count": 1},
    {"challenge_id": 1644258138190862, "challenge_name": "鹤岗房子", "sum_count": 1},
    {"challenge_id": 1644075797125134, "challenge_name": "网安山东", "sum_count": 1},
    {
        "challenge_id": 1600312964157443,
        "challenge_name": "秦霄贤孙九香",
        "sum_count": 1,
    },
    {"challenge_id": 1552967203585026, "challenge_name": "00后", "sum_count": 1},
    {"challenge_id": 1613411803136003, "challenge_name": "申请热门", "sum_count": 1},
    {
        "challenge_id": 1579425824552974,
        "challenge_name": "农村人怎么了",
        "sum_count": 1,
    },
    {
        "challenge_id": 1608966790524932,
        "challenge_name": "持续关注持续精彩",
        "sum_count": 1,
    },
    {
        "challenge_id": 1648325059875851,
        "challenge_name": "不怕沙雕多就怕沙雕住一窝",
        "sum_count": 1,
    },
    {"challenge_id": 1571711435128833, "challenge_name": "情感电台", "sum_count": 1},
    {
        "challenge_id": 1677612249109512,
        "challenge_name": "青春无限♾青春之旅美好生活",
        "sum_count": 1,
    },
    {
        "challenge_id": 1675474719632396,
        "challenge_name": "铁雨2首脑峰会",
        "sum_count": 1,
    },
    {"challenge_id": 1590979013152781, "challenge_name": "高空挑战", "sum_count": 1},
    {"challenge_id": 1639829686000648, "challenge_name": "炮神", "sum_count": 1},
    {"challenge_id": 1574083285259341, "challenge_name": "第一段", "sum_count": 1},
    {"challenge_id": 1570980979267586, "challenge_name": "开心消消乐", "sum_count": 1},
    {"challenge_id": 1580986833528846, "challenge_name": "变老", "sum_count": 1},
    {"challenge_id": 1676446446547975, "challenge_name": "孙子大传", "sum_count": 1},
    {"challenge_id": 1591013283480589, "challenge_name": "细节操作", "sum_count": 1},
    {
        "challenge_id": 1677994182789127,
        "challenge_name": "成都软件园一盗贼专偷外卖冒菜",
        "sum_count": 1,
    },
    {"challenge_id": 1608870618100808, "challenge_name": "中国航天", "sum_count": 1},
    {"challenge_id": 1565524068230145, "challenge_name": "美发教学", "sum_count": 1},
    {"challenge_id": 1591499351969806, "challenge_name": "浴血黑帮", "sum_count": 1},
    {"challenge_id": 1669219873756163, "challenge_name": "底盘铲地球", "sum_count": 1},
    {"challenge_id": 1628918765741059, "challenge_name": "旅游", "sum_count": 1},
    {
        "challenge_id": 1600604121323544,
        "challenge_name": "每日亲子陪伴",
        "sum_count": 1,
    },
    {
        "challenge_id": 1628804425315341,
        "challenge_name": "王牌特工特工学院",
        "sum_count": 1,
    },
    {"challenge_id": 1677801620133901, "challenge_name": "女生好s", "sum_count": 1},
    {
        "challenge_id": 1585127360394381,
        "challenge_name": "大话西游之大圣娶亲",
        "sum_count": 1,
    },
    {"challenge_id": 1553606972212226, "challenge_name": "小可爱", "sum_count": 1},
    {"challenge_id": 1629769441716232, "challenge_name": "胡教练", "sum_count": 1},
    {"challenge_id": 1605932028987422, "challenge_name": "虎头蜂", "sum_count": 1},
    {"challenge_id": 1633497633660941, "challenge_name": "反对家暴", "sum_count": 1},
    {"challenge_id": 1600164474695684, "challenge_name": "逆缘", "sum_count": 1},
    {"challenge_id": 1569782244524033, "challenge_name": "婚纱照", "sum_count": 1},
    {
        "challenge_id": 1627275015135245,
        "challenge_name": "超自然事件之坠龙事件",
        "sum_count": 1,
    },
    {"challenge_id": 1668471474695243, "challenge_name": "宅间轻喜剧", "sum_count": 1},
    {"challenge_id": 1623167611284643, "challenge_name": "恶魔蜘蛛王", "sum_count": 1},
    {"challenge_id": 1577683473092622, "challenge_name": "新生儿", "sum_count": 1},
    {"challenge_id": 1583589310753806, "challenge_name": "抢车位", "sum_count": 1},
    {"challenge_id": 1614726028664840, "challenge_name": "八佰", "sum_count": 1},
    {"challenge_id": 1594929080971268, "challenge_name": "焊工", "sum_count": 1},
    {"challenge_id": 1642665199412238, "challenge_name": "小欢喜2", "sum_count": 1},
    {
        "challenge_id": 1677448679876620,
        "challenge_name": "送到女儿学校",
        "sum_count": 1,
    },
    {"challenge_id": 1653695079982092, "challenge_name": "duo小助手", "sum_count": 1},
    {"challenge_id": 1665901656861709, "challenge_name": "男科医生", "sum_count": 1},
    {
        "challenge_id": 1655951003594766,
        "challenge_name": "2020dou上热门",
        "sum_count": 1,
    },
    {
        "challenge_id": 1651350737173508,
        "challenge_name": "明星怼人合集",
        "sum_count": 1,
    },
    {"challenge_id": 1571082277522434, "challenge_name": "坑娃", "sum_count": 1},
    {"challenge_id": 1588537977068552, "challenge_name": "痛风", "sum_count": 1},
    {"challenge_id": 1673565308310536, "challenge_name": "乡村新景象", "sum_count": 1},
    {"challenge_id": 1574954750868494, "challenge_name": "网瘾少年", "sum_count": 1},
    {
        "challenge_id": 1677997772205069,
        "challenge_name": "穗禾锦觅开战",
        "sum_count": 1,
    },
    {"challenge_id": 1591210331884551, "challenge_name": "养猪人", "sum_count": 1},
    {"challenge_id": 1563668798581762, "challenge_name": "男朋友", "sum_count": 1},
    {"challenge_id": 1585676977298510, "challenge_name": "室友日常", "sum_count": 1},
    {"challenge_id": 1622457289260039, "challenge_name": "天才枪手", "sum_count": 1},
    {
        "challenge_id": 1555878998362113,
        "challenge_name": "巴啦啦小魔仙",
        "sum_count": 1,
    },
    {"challenge_id": 1603543547993101, "challenge_name": "湖南沅江", "sum_count": 1},
    {"challenge_id": 1584493793569869, "challenge_name": "英超", "sum_count": 1},
    {"challenge_id": 1618115996383236, "challenge_name": "秋梨膏", "sum_count": 1},
    {"challenge_id": 1557493523404802, "challenge_name": "推荐", "sum_count": 1},
    {"challenge_id": 1590727328633869, "challenge_name": "抖瓜小助", "sum_count": 1},
    {"challenge_id": 1572190925510669, "challenge_name": "骗子", "sum_count": 1},
    {
        "challenge_id": 1675551209467916,
        "challenge_name": "我和我的家乡味",
        "sum_count": 1,
    },
    {"challenge_id": 1663359081850888, "challenge_name": "伟壹", "sum_count": 1},
    {"challenge_id": 1585013108852813, "challenge_name": "谁是卧底", "sum_count": 1},
    {
        "challenge_id": 1642978158987272,
        "challenge_name": "带孩子太难了",
        "sum_count": 1,
    },
    {"challenge_id": 1607940554062852, "challenge_name": "母婴店", "sum_count": 1},
    {
        "challenge_id": 1677422862199821,
        "challenge_name": "传统古井烧猪",
        "sum_count": 1,
    },
    {"challenge_id": 1663659493911563, "challenge_name": "小姐姐", "sum_count": 1},
    {"challenge_id": 1665940721062926, "challenge_name": "r法催眠", "sum_count": 1},
    {
        "challenge_id": 1677704248546312,
        "challenge_name": "347国道发生大面积山体垮塌",
        "sum_count": 1,
    },
    {"challenge_id": 1564190844956674, "challenge_name": "男装", "sum_count": 1},
    {"challenge_id": 1587900223097869, "challenge_name": "蜜雪冰城", "sum_count": 1},
    {
        "challenge_id": 1607764214653959,
        "challenge_name": "危险动作请勿模仿",
        "sum_count": 1,
    },
    {"challenge_id": 1645997237147659, "challenge_name": "男科", "sum_count": 1},
    {
        "challenge_id": 1671424452809741,
        "challenge_name": "李有财造飞机",
        "sum_count": 1,
    },
    {"challenge_id": 1581202617364493, "challenge_name": "五菱之光", "sum_count": 1},
    {"challenge_id": 1580116381965325, "challenge_name": "城市建设", "sum_count": 1},
    {
        "challenge_id": 1635700626272264,
        "challenge_name": "亲爱的热爱的",
        "sum_count": 1,
    },
    {"challenge_id": 1652260536571911, "challenge_name": "轩辕大帝", "sum_count": 1},
    {
        "challenge_id": 1565705888506882,
        "challenge_name": "你是风儿我是沙",
        "sum_count": 1,
    },
    {"challenge_id": 1601051713037326, "challenge_name": "派出所", "sum_count": 1},
    {"challenge_id": 1566926248364034, "challenge_name": "美食分享", "sum_count": 1},
    {"challenge_id": 1659513056397320, "challenge_name": "百亿好剧", "sum_count": 1},
    {"challenge_id": 1578664801646622, "challenge_name": "月饼", "sum_count": 1},
    {"challenge_id": 1614116465033357, "challenge_name": "猫鼠游戏", "sum_count": 1},
    {
        "challenge_id": 1608401017931789,
        "challenge_name": "冰天雪地任我行",
        "sum_count": 1,
    },
    {"challenge_id": 1560535746018306, "challenge_name": "老师", "sum_count": 1},
    {"challenge_id": 1565553933189122, "challenge_name": "深圳", "sum_count": 1},
    {
        "challenge_id": 1677970801234957,
        "challenge_name": "小鬼黄鸿升去世原因公布",
        "sum_count": 1,
    },
    {"challenge_id": 1622354945002499, "challenge_name": "赵亮", "sum_count": 1},
    {"challenge_id": 1636325168600068, "challenge_name": "工业机械", "sum_count": 1},
    {"challenge_id": 1608703107038221, "challenge_name": "胸肌训练", "sum_count": 1},
    {"challenge_id": 1613644615706632, "challenge_name": "爱商", "sum_count": 1},
    {"challenge_id": 1583122900064270, "challenge_name": "任达华", "sum_count": 1},
    {"challenge_id": 1582284554587150, "challenge_name": "奔跑的少年", "sum_count": 1},
    {"challenge_id": 1656436228311047, "challenge_name": "小宝茶", "sum_count": 1},
    {"challenge_id": 1613770438473735, "challenge_name": "老房改造", "sum_count": 1},
    {"challenge_id": 1611423829680131, "challenge_name": "台海", "sum_count": 1},
    {"challenge_id": 1657017434406925, "challenge_name": "张定宇", "sum_count": 1},
    {"challenge_id": 1568980017052674, "challenge_name": "小猫咪", "sum_count": 1},
    {"challenge_id": 1588202363174926, "challenge_name": "甜剧", "sum_count": 1},
    {"challenge_id": 1583121307960334, "challenge_name": "重男轻女", "sum_count": 1},
    {
        "challenge_id": 1658678512169991,
        "challenge_name": "抖瓜好货推荐给你",
        "sum_count": 1,
    },
    {"challenge_id": 1620202558885892, "challenge_name": "李幼斌", "sum_count": 1},
    {"challenge_id": 1561383677659138, "challenge_name": "画", "sum_count": 1},
    {"challenge_id": 1572558994881549, "challenge_name": "抖瓜游戏", "sum_count": 1},
    {"challenge_id": 1597624168204291, "challenge_name": "电影推荐", "sum_count": 1},
    {
        "challenge_id": 1677892094793741,
        "challenge_name": "2020年只剩3个月了",
        "sum_count": 1,
    },
    {"challenge_id": 1579656970466318, "challenge_name": "雪景", "sum_count": 1},
    {"challenge_id": 1583592099941389, "challenge_name": "报恩", "sum_count": 1},
    {
        "challenge_id": 1586303608551501,
        "challenge_name": "胖子都是潜力股",
        "sum_count": 1,
    },
    {
        "challenge_id": 1613852073929731,
        "challenge_name": "爱情婚姻家庭",
        "sum_count": 1,
    },
    {"challenge_id": 1644541644214286, "challenge_name": "911事件", "sum_count": 1},
    {
        "challenge_id": 1677426437089292,
        "challenge_name": "吉林复读生为凑学费偷电瓶",
        "sum_count": 1,
    },
    {"challenge_id": 1648371942519816, "challenge_name": "圣殇", "sum_count": 1},
    {"challenge_id": 1575983798856733, "challenge_name": "高速", "sum_count": 1},
    {
        "challenge_id": 1587498228710414,
        "challenge_name": "跳舞使我快乐",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677535476530248,
        "challenge_name": "911事件19周年旧址点亮光柱",
        "sum_count": 1,
    },
    {"challenge_id": 1588584855328782, "challenge_name": "红旗", "sum_count": 1},
    {"challenge_id": 1591380902636552, "challenge_name": "汤唯", "sum_count": 1},
    {
        "challenge_id": 1678001781079053,
        "challenge_name": "11岁骑车女生被人恶意拉倒",
        "sum_count": 1,
    },
    {"challenge_id": 1562187648336897, "challenge_name": "男士发型", "sum_count": 1},
    {"challenge_id": 1666189726534663, "challenge_name": "魔幻剧场", "sum_count": 1},
    {
        "challenge_id": 1674622886439943,
        "challenge_name": "王牌主播请出战",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677144000920589,
        "challenge_name": "笑容丝滑愉悦开挂",
        "sum_count": 1,
    },
    {"challenge_id": 1576155270937645, "challenge_name": "模拟游戏", "sum_count": 1},
    {"challenge_id": 1663853183150092, "challenge_name": "小影夫妇", "sum_count": 1},
    {"challenge_id": 1588486474216455, "challenge_name": "国产神剧", "sum_count": 1},
    {"challenge_id": 1618895028044803, "challenge_name": "养父母", "sum_count": 1},
    {"challenge_id": 1564252858121218, "challenge_name": "模仿", "sum_count": 1},
    {"challenge_id": 1573548836325390, "challenge_name": "狂欢", "sum_count": 1},
    {"challenge_id": 1646884809810948, "challenge_name": "最强蜗牛", "sum_count": 1},
    {"challenge_id": 1668279142359048, "challenge_name": "李泰林", "sum_count": 1},
    {
        "challenge_id": 1677908858535943,
        "challenge_name": "加优跨次元歌者",
        "sum_count": 1,
    },
    {
        "challenge_id": 1641649362983944,
        "challenge_name": "婚姻爱情保卫战",
        "sum_count": 1,
    },
    {"challenge_id": 1577070421094413, "challenge_name": "赵奕欢", "sum_count": 1},
    {
        "challenge_id": 1677816329351176,
        "challenge_name": "我喜欢你订档",
        "sum_count": 1,
    },
    {"challenge_id": 1677160007808011, "challenge_name": "派出所蹲点", "sum_count": 1},
    {"challenge_id": 1623998763217927, "challenge_name": "抗战神剧", "sum_count": 1},
    {"challenge_id": 1580905686601741, "challenge_name": "大火", "sum_count": 1},
    {"challenge_id": 1585840279782414, "challenge_name": "梅艳芳", "sum_count": 1},
    {
        "challenge_id": 1677862456652807,
        "challenge_name": "韩国女星吴仁惠去世",
        "sum_count": 1,
    },
    {
        "challenge_id": 1586226599503965,
        "challenge_name": "康熙微服私访记",
        "sum_count": 1,
    },
    {"challenge_id": 1573054489165838, "challenge_name": "感悟", "sum_count": 1},
    {"challenge_id": 1563226140259329, "challenge_name": "热门音乐", "sum_count": 1},
    {
        "challenge_id": 1677323659416589,
        "challenge_name": "我二兄弟也来跟个风",
        "sum_count": 1,
    },
    {"challenge_id": 1582299757202509, "challenge_name": "神雕侠侣", "sum_count": 1},
    {"challenge_id": 1596969382013960, "challenge_name": "秦海璐", "sum_count": 1},
    {"challenge_id": 1610841088134148, "challenge_name": "婴儿成长", "sum_count": 1},
    {"challenge_id": 1677705427025933, "challenge_name": "总裁崩溃", "sum_count": 1},
    {"challenge_id": 1594627956941831, "challenge_name": "武松", "sum_count": 1},
    {"challenge_id": 1617178489624583, "challenge_name": "巴布豆童鞋", "sum_count": 1},
    {"challenge_id": 1677789567607821, "challenge_name": "菅义伟", "sum_count": 1},
    {"challenge_id": 1591111607478302, "challenge_name": "中年", "sum_count": 1},
    {"challenge_id": 1571738776766466, "challenge_name": "纽约", "sum_count": 1},
    {"challenge_id": 1567482989331458, "challenge_name": "每天一笑", "sum_count": 1},
    {
        "challenge_id": 1657497641070595,
        "challenge_name": "新冠肺炎疫情",
        "sum_count": 1,
    },
    {"challenge_id": 1600236256103431, "challenge_name": "核爆", "sum_count": 1},
    {"challenge_id": 1587920809401421, "challenge_name": "洗浴中心", "sum_count": 1},
    {"challenge_id": 1619277860613197, "challenge_name": "智勋", "sum_count": 1},
    {
        "challenge_id": 1623971651070990,
        "challenge_name": "记录我的农村生活",
        "sum_count": 1,
    },
    {"challenge_id": 1611773240568840, "challenge_name": "沙雕闺蜜", "sum_count": 1},
    {"challenge_id": 1584324669784078, "challenge_name": "推理", "sum_count": 1},
    {"challenge_id": 1580696632184861, "challenge_name": "孝心", "sum_count": 1},
    {"challenge_id": 1600246427211779, "challenge_name": "功能沙发", "sum_count": 1},
    {"challenge_id": 1563738987591682, "challenge_name": "北京", "sum_count": 1},
    {"challenge_id": 1591025235673092, "challenge_name": "郭麒麟", "sum_count": 1},
    {"challenge_id": 1581320898713645, "challenge_name": "肚子", "sum_count": 1},
    {"challenge_id": 1570426482584578, "challenge_name": "好友", "sum_count": 1},
    {"challenge_id": 1600131351015427, "challenge_name": "争执", "sum_count": 1},
    {"challenge_id": 1566087130744833, "challenge_name": "造型", "sum_count": 1},
    {"challenge_id": 1566395514126354, "challenge_name": "大学宿舍", "sum_count": 1},
    {"challenge_id": 1578084874670093, "challenge_name": "中国历史", "sum_count": 1},
    {"challenge_id": 1567277130726402, "challenge_name": "童年回忆", "sum_count": 1},
    {"challenge_id": 1607467553843268, "challenge_name": "老师想你了", "sum_count": 1},
    {"challenge_id": 1603256533442573, "challenge_name": "纹身师", "sum_count": 1},
    {"challenge_id": 1579796389787661, "challenge_name": "发饰", "sum_count": 1},
    {
        "challenge_id": 1674810012655623,
        "challenge_name": "急诊花帽小护士",
        "sum_count": 1,
    },
    {"challenge_id": 1555403246990338, "challenge_name": "双胞胎", "sum_count": 1},
    {"challenge_id": 1575985882266654, "challenge_name": "黄渤", "sum_count": 1},
    {
        "challenge_id": 1676692906416135,
        "challenge_name": "自热小火锅就吃自嗨锅",
        "sum_count": 1,
    },
    {"challenge_id": 1612138665803815, "challenge_name": "普法小剧场", "sum_count": 1},
    {"challenge_id": 1661132764788748, "challenge_name": "陕西吃啥", "sum_count": 1},
    {"challenge_id": 1576794974985229, "challenge_name": "一年级", "sum_count": 1},
    {"challenge_id": 1600979165590599, "challenge_name": "纹身刺青", "sum_count": 1},
    {"challenge_id": 1611819994148867, "challenge_name": "左下角", "sum_count": 1},
    {"challenge_id": 1569612383333377, "challenge_name": "螃蟹", "sum_count": 1},
    {"challenge_id": 1585773422203917, "challenge_name": "眼光", "sum_count": 1},
    {"challenge_id": 1593360069113863, "challenge_name": "微胖女生", "sum_count": 1},
    {"challenge_id": 1582287894079502, "challenge_name": "最可爱的人", "sum_count": 1},
    {"challenge_id": 1565826299223042, "challenge_name": "桃总", "sum_count": 1},
    {
        "challenge_id": 1675859474872332,
        "challenge_name": "每日精选好剧",
        "sum_count": 1,
    },
    {"challenge_id": 1565366135950338, "challenge_name": "我要上推荐", "sum_count": 1},
    {"challenge_id": 1561584511636481, "challenge_name": "武汉", "sum_count": 1},
    {"challenge_id": 1553120460998657, "challenge_name": "可爱", "sum_count": 1},
    {"challenge_id": 1637739690380292, "challenge_name": "谦酒", "sum_count": 1},
    {"challenge_id": 1582310445641741, "challenge_name": "牙齿", "sum_count": 1},
    {"challenge_id": 1576734679761934, "challenge_name": "好兄弟", "sum_count": 1},
    {"challenge_id": 1623836144134151, "challenge_name": "捐献", "sum_count": 1},
    {"challenge_id": 1597903469607971, "challenge_name": "杨紫李现", "sum_count": 1},
    {"challenge_id": 1564355286200321, "challenge_name": "victory", "sum_count": 1},
    {"challenge_id": 1573873261129741, "challenge_name": "甜品", "sum_count": 1},
    {"challenge_id": 1640395304947723, "challenge_name": "撞邪先生", "sum_count": 1},
    {"challenge_id": 1638734146822148, "challenge_name": "最美女兵", "sum_count": 1},
    {
        "challenge_id": 1591122149099528,
        "challenge_name": "你猜不到的结局",
        "sum_count": 1,
    },
    {"challenge_id": 1607799206402056, "challenge_name": "太古里街拍", "sum_count": 1},
    {"challenge_id": 1564001790576642, "challenge_name": "神奇的肚子", "sum_count": 1},
    {
        "challenge_id": 1677268298126344,
        "challenge_name": "领导护犊子是什么体验",
        "sum_count": 1,
    },
    {"challenge_id": 1571517182097410, "challenge_name": "碰瓷", "sum_count": 1},
    {"challenge_id": 1609469406354567, "challenge_name": "唐老师日常", "sum_count": 1},
    {
        "challenge_id": 1593647413242887,
        "challenge_name": "徒手掰苹果挑战",
        "sum_count": 1,
    },
    {"challenge_id": 1567822906610690, "challenge_name": "上海迪士尼", "sum_count": 1},
    {"challenge_id": 1614395633233928, "challenge_name": "患者", "sum_count": 1},
    {
        "challenge_id": 1593351384951815,
        "challenge_name": "好看的电视剧",
        "sum_count": 1,
    },
    {"challenge_id": 1582470770924558, "challenge_name": "大鹏", "sum_count": 1},
    {"challenge_id": 1616526307560536, "challenge_name": "雷神电脑", "sum_count": 1},
    {"challenge_id": 1622630551428104, "challenge_name": "姜贞羽", "sum_count": 1},
    {"challenge_id": 1628892655780877, "challenge_name": "代号生机", "sum_count": 1},
    {
        "challenge_id": 1597197008174094,
        "challenge_name": "成年人的世界",
        "sum_count": 1,
    },
    {"challenge_id": 1573897210178574, "challenge_name": "洪崖洞", "sum_count": 1},
    {"challenge_id": 1588270668149763, "challenge_name": "年龄差", "sum_count": 1},
    {
        "challenge_id": 1674700166464516,
        "challenge_name": "广西新闻早知道",
        "sum_count": 1,
    },
    {"challenge_id": 1552386123095041, "challenge_name": "吴亦凡", "sum_count": 1},
    {
        "challenge_id": 1623175739432967,
        "challenge_name": "这条街最亮的仔",
        "sum_count": 1,
    },
    {"challenge_id": 1577837655597070, "challenge_name": "考驾照", "sum_count": 1},
    {"challenge_id": 1611309589793860, "challenge_name": "成功学", "sum_count": 1},
    {"challenge_id": 1569810761985025, "challenge_name": "伞", "sum_count": 1},
    {"challenge_id": 1598130235290638, "challenge_name": "农忙季节", "sum_count": 1},
    {"challenge_id": 1575074513291277, "challenge_name": "育儿", "sum_count": 1},
    {"challenge_id": 1580420785864797, "challenge_name": "婆婆", "sum_count": 1},
    {"challenge_id": 1574231426382862, "challenge_name": "脱单", "sum_count": 1},
    {"challenge_id": 1649547991217160, "challenge_name": "测评打假", "sum_count": 1},
    {"challenge_id": 1568886956459010, "challenge_name": "烤肉", "sum_count": 1},
    {"challenge_id": 1676784772459532, "challenge_name": "玩转好声音", "sum_count": 1},
    {"challenge_id": 1628665998113800, "challenge_name": "倚天屠龙记_", "sum_count": 1},
    {"challenge_id": 1594195711440903, "challenge_name": "苗侨伟", "sum_count": 1},
    {"challenge_id": 1606928965228548, "challenge_name": "别屏蔽", "sum_count": 1},
    {
        "challenge_id": 1656666038590475,
        "challenge_name": "荒野求生第三季",
        "sum_count": 1,
    },
    {
        "challenge_id": 1625430245483523,
        "challenge_name": "下雪就得这样玩",
        "sum_count": 1,
    },
    {"challenge_id": 1662831692030980, "challenge_name": "影视风云", "sum_count": 1},
    {
        "challenge_id": 1615008005051406,
        "challenge_name": "全世界最可爱的噗噗",
        "sum_count": 1,
    },
    {"challenge_id": 1578422296238094, "challenge_name": "活佛济公", "sum_count": 1},
    {"challenge_id": 1557054362274818, "challenge_name": "段子", "sum_count": 1},
    {"challenge_id": 1603498176704516, "challenge_name": "张小龙", "sum_count": 1},
    {"challenge_id": 1606770143682579, "challenge_name": "和睦家庭", "sum_count": 1},
    {"challenge_id": 1586854430479374, "challenge_name": "搞笑大师", "sum_count": 1},
    {"challenge_id": 1569317660421121, "challenge_name": "文明出行", "sum_count": 1},
    {
        "challenge_id": 1677893107969032,
        "challenge_name": "抖in北京#抖incity美好生活节",
        "sum_count": 1,
    },
    {"challenge_id": 1554119263548418, "challenge_name": "刷牙", "sum_count": 1},
    {"challenge_id": 1602335893123080, "challenge_name": "打黑", "sum_count": 1},
    {"challenge_id": 1583314560927757, "challenge_name": "青丘狐传说", "sum_count": 1},
    {"challenge_id": 1588467836491779, "challenge_name": "武动乾坤", "sum_count": 1},
    {"challenge_id": 1598834307560461, "challenge_name": "兴化", "sum_count": 1},
    {"challenge_id": 1615351904830488, "challenge_name": "玉石", "sum_count": 1},
    {"challenge_id": 1631236218335236, "challenge_name": "一车一江湖", "sum_count": 1},
    {"challenge_id": 1651083148338190, "challenge_name": "命案十三宗", "sum_count": 1},
    {"challenge_id": 1614949987486724, "challenge_name": "汽车文化", "sum_count": 1},
    {"challenge_id": 1556856443091969, "challenge_name": "周杰伦", "sum_count": 1},
    {"challenge_id": 1574174143975437, "challenge_name": "奥迪", "sum_count": 1},
    {
        "challenge_id": 1644033553605646,
        "challenge_name": "每一帧都是热爱",
        "sum_count": 1,
    },
    {"challenge_id": 1574070346729550, "challenge_name": "好物推荐", "sum_count": 1},
    {"challenge_id": 1638374406601739, "challenge_name": "利刃出鞘", "sum_count": 1},
    {"challenge_id": 1654583391575051, "challenge_name": "品鉴彭州", "sum_count": 1},
    {"challenge_id": 1568968940120066, "challenge_name": "儿童", "sum_count": 1},
    {
        "challenge_id": 1677812835643406,
        "challenge_name": "电子科技大学开学典礼",
        "sum_count": 1,
    },
    {"challenge_id": 1617538819795975, "challenge_name": "天堂回信", "sum_count": 1},
    {"challenge_id": 1587392320444429, "challenge_name": "娘家", "sum_count": 1},
    {
        "challenge_id": 1605527650724888,
        "challenge_name": "记录宝宝成长的每一天",
        "sum_count": 1,
    },
    {
        "challenge_id": 1675423522696205,
        "challenge_name": "郭论谦道花式开箱",
        "sum_count": 1,
    },
    {"challenge_id": 1596912499578903, "challenge_name": "名场面", "sum_count": 1},
    {"challenge_id": 1675898188874755, "challenge_name": "参加献x活动", "sum_count": 1},
    {"challenge_id": 1606162495645700, "challenge_name": "梳头姐弟", "sum_count": 1},
    {"challenge_id": 1582301101844494, "challenge_name": "军舰岛", "sum_count": 1},
    {"challenge_id": 1668291124803591, "challenge_name": "云上科普日", "sum_count": 1},
    {"challenge_id": 1574626737582125, "challenge_name": "单身的原因", "sum_count": 1},
    {"challenge_id": 1589412185820173, "challenge_name": "独生子女", "sum_count": 1},
    {"challenge_id": 1564299357263873, "challenge_name": "外卖", "sum_count": 1},
    {"challenge_id": 1666438297361416, "challenge_name": "吴中天", "sum_count": 1},
    {"challenge_id": 1611725973256206, "challenge_name": "长安诺", "sum_count": 1},
    {"challenge_id": 1639761233340427, "challenge_name": "知识充电站", "sum_count": 1},
    {"challenge_id": 1677328466790414, "challenge_name": "美味卷排骨", "sum_count": 1},
    {"challenge_id": 1572339587976205, "challenge_name": "摄影师", "sum_count": 1},
    {"challenge_id": 1591711534215176, "challenge_name": "韩剧推荐", "sum_count": 1},
    {"challenge_id": 1570750369816578, "challenge_name": "家乡", "sum_count": 1},
    {"challenge_id": 1640994071284739, "challenge_name": "军事", "sum_count": 1},
    {"challenge_id": 1574272490583069, "challenge_name": "按摩", "sum_count": 1},
    {"challenge_id": 1581492753429517, "challenge_name": "姚明", "sum_count": 1},
    {"challenge_id": 1629134389170188, "challenge_name": "童鞋厂家", "sum_count": 1},
    {"challenge_id": 1583106210907150, "challenge_name": "素质", "sum_count": 1},
    {"challenge_id": 1580315652631566, "challenge_name": "军训时光", "sum_count": 1},
    {"challenge_id": 1566722653623298, "challenge_name": "拍戏", "sum_count": 1},
    {"challenge_id": 1571257555315714, "challenge_name": "老九门", "sum_count": 1},
    {"challenge_id": 1568611992448002, "challenge_name": "精灵宝可梦", "sum_count": 1},
    {"challenge_id": 1587746734672910, "challenge_name": "伍佰", "sum_count": 1},
    {"challenge_id": 1595993623472136, "challenge_name": "鹿晗邓超", "sum_count": 1},
    {"challenge_id": 1567098492881922, "challenge_name": "真爱", "sum_count": 1},
    {"challenge_id": 1585015473368077, "challenge_name": "挽回", "sum_count": 1},
    {"challenge_id": 1604354969179143, "challenge_name": "大学开学", "sum_count": 1},
    {"challenge_id": 1667851982065672, "challenge_name": "斗破乱世情", "sum_count": 1},
    {"challenge_id": 1604260854383629, "challenge_name": "王耀庆", "sum_count": 1},
    {"challenge_id": 1592304500408334, "challenge_name": "袁隆平", "sum_count": 1},
    {"challenge_id": 1575074180823053, "challenge_name": "矮脚猫", "sum_count": 1},
    {"challenge_id": 1550769196968961, "challenge_name": "尴尬", "sum_count": 1},
    {
        "challenge_id": 1625613090686987,
        "challenge_name": "感谢抖瓜我要热门",
        "sum_count": 1,
    },
    {"challenge_id": 1647456595878919, "challenge_name": "创作人计划", "sum_count": 1},
    {"challenge_id": 1654891320307720, "challenge_name": "皮卡大叔", "sum_count": 1},
    {"challenge_id": 1565910651251713, "challenge_name": "我的世界", "sum_count": 1},
    {
        "challenge_id": 1596008877671437,
        "challenge_name": "女儿是爸爸的小棉袄",
        "sum_count": 1,
    },
    {"challenge_id": 1650603431386119, "challenge_name": "殴打", "sum_count": 1},
    {"challenge_id": 1563656529006593, "challenge_name": "岳云鹏", "sum_count": 1},
    {"challenge_id": 1583042891615246, "challenge_name": "乔欣", "sum_count": 1},
    {"challenge_id": 1564937801376834, "challenge_name": "衣服", "sum_count": 1},
    {"challenge_id": 1668573286215687, "challenge_name": "无用的知识", "sum_count": 1},
    {"challenge_id": 1601145958245379, "challenge_name": "钓鱼那些事", "sum_count": 1},
    {"challenge_id": 1579264536548366, "challenge_name": "拯救不开心", "sum_count": 1},
    {"challenge_id": 1674074185015310, "challenge_name": "安徽观察", "sum_count": 1},
    {"challenge_id": 1575780116911118, "challenge_name": "创意diy", "sum_count": 1},
    {
        "challenge_id": 1668662691087373,
        "challenge_name": "花西子蚕丝蜜粉饼",
        "sum_count": 1,
    },
    {
        "challenge_id": 1625992682089475,
        "challenge_name": "当下热门话题",
        "sum_count": 1,
    },
    {"challenge_id": 1573372737495054, "challenge_name": "来了", "sum_count": 1},
    {"challenge_id": 1587732686155821, "challenge_name": "郑秀文", "sum_count": 1},
    {
        "challenge_id": 1677155037307907,
        "challenge_name": "圣罗兰黑白无界",
        "sum_count": 1,
    },
    {"challenge_id": 1580977453876237, "challenge_name": "林正英", "sum_count": 1},
    {"challenge_id": 1603942096703501, "challenge_name": "晴雨两用伞", "sum_count": 1},
    {"challenge_id": 1572529934098445, "challenge_name": "珍爱生命", "sum_count": 1},
    {"challenge_id": 1612731880805390, "challenge_name": "广西土猎", "sum_count": 1},
    {
        "challenge_id": 1635030668863501,
        "challenge_name": "建设美好生活ing",
        "sum_count": 1,
    },
    {"challenge_id": 1566293521774658, "challenge_name": "迪丽热巴", "sum_count": 1},
    {
        "challenge_id": 1630040738892814,
        "challenge_name": "抖瓜大大给个热门吧",
        "sum_count": 1,
    },
    {"challenge_id": 1657940070453256, "challenge_name": "喂鸡神器", "sum_count": 1},
    {"challenge_id": 1586595426017294, "challenge_name": "头条", "sum_count": 1},
    {"challenge_id": 1583555778716686, "challenge_name": "辽宁", "sum_count": 1},
    {
        "challenge_id": 1677699250549790,
        "challenge_name": "伍佰坐轮椅开唱",
        "sum_count": 1,
    },
    {
        "challenge_id": 1608665399364686,
        "challenge_name": "身材好就是好",
        "sum_count": 1,
    },
    {"challenge_id": 1621353881698333, "challenge_name": "钞能力", "sum_count": 1},
    {"challenge_id": 1575705125389326, "challenge_name": "美人", "sum_count": 1},
    {"challenge_id": 1652806277463044, "challenge_name": "远离家暴男", "sum_count": 1},
    {"challenge_id": 1619008421086211, "challenge_name": "爆炸", "sum_count": 1},
    {"challenge_id": 1585547389384717, "challenge_name": "现场直播", "sum_count": 1},
    {"challenge_id": 1570370216534018, "challenge_name": "等着你", "sum_count": 1},
    {"challenge_id": 1579060689456206, "challenge_name": "改编音乐", "sum_count": 1},
    {"challenge_id": 1595153349741572, "challenge_name": "静安寺", "sum_count": 1},
    {"challenge_id": 1677371187758094, "challenge_name": "李有财农具", "sum_count": 1},
    {"challenge_id": 1618746894366724, "challenge_name": "网恋奔现", "sum_count": 1},
    {"challenge_id": 1629481541809166, "challenge_name": "d位出道", "sum_count": 1},
    {"challenge_id": 1670365038337031, "challenge_name": "李响传媒", "sum_count": 1},
    {"challenge_id": 1677631913847811, "challenge_name": "无心街坊", "sum_count": 1},
    {
        "challenge_id": 1626680443255819,
        "challenge_name": "大头biubiubiu",
        "sum_count": 1,
    },
    {"challenge_id": 1576885881009181, "challenge_name": "我想上推荐", "sum_count": 1},
    {
        "challenge_id": 1677532093074436,
        "challenge_name": "冠军陈更单挑四个小学生",
        "sum_count": 1,
    },
    {"challenge_id": 1602244482891783, "challenge_name": "法医", "sum_count": 1},
    {
        "challenge_id": 1653962626043912,
        "challenge_name": "我的养龟日记",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677506828729351,
        "challenge_name": "懂车的小同学",
        "sum_count": 1,
    },
    {"challenge_id": 1674150487094296, "challenge_name": "电视剧琉璃", "sum_count": 1},
    {"challenge_id": 1602775492824094, "challenge_name": "居家好物", "sum_count": 1},
    {"challenge_id": 1574533455391758, "challenge_name": "水浒传", "sum_count": 1},
    {"challenge_id": 1661672750623752, "challenge_name": "精神过头", "sum_count": 1},
    {
        "challenge_id": 1677264336503812,
        "challenge_name": "孙弈秋郝帅天台打架",
        "sum_count": 1,
    },
    {
        "challenge_id": 1580667354053645,
        "challenge_name": "不能说的秘密",
        "sum_count": 1,
    },
    {"challenge_id": 1677371056041991, "challenge_name": "割玉米专用", "sum_count": 1},
    {"challenge_id": 1579578896252941, "challenge_name": "过把瘾", "sum_count": 1},
    {
        "challenge_id": 1665194959381511,
        "challenge_name": "把你宠在手心",
        "sum_count": 1,
    },
    {"challenge_id": 1573171437990925, "challenge_name": "名人", "sum_count": 1},
    {"challenge_id": 1582961471341581, "challenge_name": "超模", "sum_count": 1},
    {"challenge_id": 1629946584029187, "challenge_name": "nba季后赛", "sum_count": 1},
    {"challenge_id": 1596686386095134, "challenge_name": "文森特", "sum_count": 1},
    {
        "challenge_id": 1674092342988814,
        "challenge_name": "资本市场三十年",
        "sum_count": 1,
    },
    {"challenge_id": 1576053374275614, "challenge_name": "鲁班锁", "sum_count": 1},
    {"challenge_id": 1594197527162887, "challenge_name": "感谢关注", "sum_count": 1},
    {"challenge_id": 1581157635157005, "challenge_name": "倪妮", "sum_count": 1},
    {"challenge_id": 1612502240242695, "challenge_name": "急诊护士", "sum_count": 1},
    {
        "challenge_id": 1677981362427912,
        "challenge_name": "七开头的车太牛逼",
        "sum_count": 1,
    },
    {"challenge_id": 1595353735032856, "challenge_name": "台湾人", "sum_count": 1},
    {"challenge_id": 1586288937230350, "challenge_name": "乡村美食", "sum_count": 1},
    {"challenge_id": 1603596646665224, "challenge_name": "北京卫视", "sum_count": 1},
    {"challenge_id": 1582400515252238, "challenge_name": "奚梦瑶", "sum_count": 1},
    {"challenge_id": 1672198277140488, "challenge_name": "今晚看点啥", "sum_count": 1},
    {"challenge_id": 1630183793301517, "challenge_name": "jnk", "sum_count": 1},
    {"challenge_id": 1583078861298701, "challenge_name": "私人飞机", "sum_count": 1},
    {"challenge_id": 1582100926906381, "challenge_name": "趣味", "sum_count": 1},
    {
        "challenge_id": 1677445463048196,
        "challenge_name": "我的盟友你是谁",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677789580033038,
        "challenge_name": "男子当众抢夺女童被饭店老板制服",
        "sum_count": 1,
    },
    {"challenge_id": 1638734089995268, "challenge_name": "古丽讲西部", "sum_count": 1},
    {"challenge_id": 1603758796182600, "challenge_name": "汽车知识", "sum_count": 1},
    {"challenge_id": 1573726718016526, "challenge_name": "兜风", "sum_count": 1},
    {"challenge_id": 1590339862235207, "challenge_name": "钢结构", "sum_count": 1},
    {"challenge_id": 1583552725006349, "challenge_name": "杜高犬", "sum_count": 1},
    {"challenge_id": 1596953771948039, "challenge_name": "楚汉传奇", "sum_count": 1},
    {
        "challenge_id": 1633045749074955,
        "challenge_name": "千里马遇伯乐",
        "sum_count": 1,
    },
    {"challenge_id": 1626610456773644, "challenge_name": "dou小助手", "sum_count": 1},
    {"challenge_id": 1564599163399170, "challenge_name": "tattoo", "sum_count": 1},
    {"challenge_id": 1656940481593357, "challenge_name": "疫情快报", "sum_count": 1},
    {"challenge_id": 1571970061487122, "challenge_name": "神秘博士", "sum_count": 1},
    {"challenge_id": 1570876411961346, "challenge_name": "甘肃", "sum_count": 1},
    {"challenge_id": 1590235339528195, "challenge_name": "沙溢", "sum_count": 1},
    {"challenge_id": 1567110187537410, "challenge_name": "怀旧经典", "sum_count": 1},
    {"challenge_id": 1620456395930637, "challenge_name": "虎子哥", "sum_count": 1},
    {"challenge_id": 1583324255251550, "challenge_name": "宁静", "sum_count": 1},
    {"challenge_id": 1610947503030286, "challenge_name": "李兆基", "sum_count": 1},
    {"challenge_id": 1572672048495630, "challenge_name": "农村老家", "sum_count": 1},
    {"challenge_id": 1554229606605825, "challenge_name": "新人报道", "sum_count": 1},
    {"challenge_id": 1625725053004808, "challenge_name": "熬夜", "sum_count": 1},
    {
        "challenge_id": 1677409754578958,
        "challenge_name": "这样装没问题👌",
        "sum_count": 1,
    },
    {"challenge_id": 1634771076474888, "challenge_name": "护童联盟", "sum_count": 1},
    {"challenge_id": 1592007484804100, "challenge_name": "肩周炎", "sum_count": 1},
    {"challenge_id": 1589204573906967, "challenge_name": "养猪", "sum_count": 1},
    {
        "challenge_id": 1673993665429511,
        "challenge_name": "团圆直播认亲",
        "sum_count": 1,
    },
    {"challenge_id": 1567359097185282, "challenge_name": "奶茶", "sum_count": 1},
    {"challenge_id": 1583866543616014, "challenge_name": "屋顶", "sum_count": 1},
    {"challenge_id": 1582935166874637, "challenge_name": "包工头", "sum_count": 1},
    {
        "challenge_id": 1677911863907335,
        "challenge_name": "女子与丈夫吵架后离家失联",
        "sum_count": 1,
    },
    {"challenge_id": 1583125640459278, "challenge_name": "六胞胎", "sum_count": 1},
    {"challenge_id": 1576230229140494, "challenge_name": "牛排", "sum_count": 1},
    {"challenge_id": 1579560185909262, "challenge_name": "十一", "sum_count": 1},
    {"challenge_id": 1588226722494494, "challenge_name": "石油", "sum_count": 1},
    {"challenge_id": 1675252747844615, "challenge_name": "脸部变焦", "sum_count": 1},
    {"challenge_id": 1584737689437198, "challenge_name": "带鱼", "sum_count": 1},
    {
        "challenge_id": 1677730967865351,
        "challenge_name": "天天向上金靖",
        "sum_count": 1,
    },
    {"challenge_id": 1576032843484238, "challenge_name": "魏大勋", "sum_count": 1},
    {"challenge_id": 1572818441355277, "challenge_name": "暖心", "sum_count": 1},
    {"challenge_id": 1565827666711553, "challenge_name": "抓鱼", "sum_count": 1},
    {"challenge_id": 1603114144781326, "challenge_name": "打针不哭", "sum_count": 1},
    {
        "challenge_id": 1663964234839047,
        "challenge_name": "小伙子全程蒙圈状态",
        "sum_count": 1,
    },
    {"challenge_id": 1656138914810894, "challenge_name": "江玉郎", "sum_count": 1},
    {"challenge_id": 1666642550330371, "challenge_name": "妻子不忠", "sum_count": 1},
    {
        "challenge_id": 1641292794003468,
        "challenge_name": "今日迷惑行为",
        "sum_count": 1,
    },
    {"challenge_id": 1562308352218113, "challenge_name": "90后", "sum_count": 1},
    {"challenge_id": 1615377490839636, "challenge_name": "养猪技术", "sum_count": 1},
    {
        "challenge_id": 1676277948057613,
        "challenge_name": "电影死无对证",
        "sum_count": 1,
    },
    {"challenge_id": 1676326998159374, "challenge_name": "兰蔻888", "sum_count": 1},
    {"challenge_id": 1611766356087812, "challenge_name": "crc", "sum_count": 1},
    {
        "challenge_id": 1677970726907975,
        "challenge_name": "杨丞琳演唱邀请初恋小鬼泪奔",
        "sum_count": 1,
    },
    {"challenge_id": 1582117337036813, "challenge_name": "多年以后", "sum_count": 1},
    {"challenge_id": 1582680000508942, "challenge_name": "作文", "sum_count": 1},
    {"challenge_id": 1675791473436686, "challenge_name": "黎明觉醒", "sum_count": 1},
    {"challenge_id": 1649290830526477, "challenge_name": "阳光抑郁", "sum_count": 1},
    {
        "challenge_id": 1677859979017230,
        "challenge_name": "男子南京地铁喝奶茶被逮罚款",
        "sum_count": 1,
    },
    {"challenge_id": 1676818157004808, "challenge_name": "俪刻报导", "sum_count": 1},
    {"challenge_id": 1570269194376193, "challenge_name": "爱心传递", "sum_count": 1},
    {"challenge_id": 1612035551475783, "challenge_name": "江别鹤", "sum_count": 1},
    {"challenge_id": 1630954077332484, "challenge_name": "无痕挂钩", "sum_count": 1},
    {"challenge_id": 1590843170959380, "challenge_name": "云南普洱", "sum_count": 1},
    {"challenge_id": 1571365172981761, "challenge_name": "报复", "sum_count": 1},
    {"challenge_id": 1649018213993480, "challenge_name": "黑帮", "sum_count": 1},
    {"challenge_id": 1595303061068813, "challenge_name": "王雷", "sum_count": 1},
    {
        "challenge_id": 1677426581310471,
        "challenge_name": "19岁女孩借用药店厕所产子后偷跑",
        "sum_count": 1,
    },
    {"challenge_id": 1647366104359940, "challenge_name": "正能量短剧", "sum_count": 1},
    {
        "challenge_id": 1603867167899652,
        "challenge_name": "不信谣不传谣",
        "sum_count": 1,
    },
    {"challenge_id": 1609947391179779, "challenge_name": "五粮液", "sum_count": 1},
    {
        "challenge_id": 1575122910910477,
        "challenge_name": "王者荣耀搞笑",
        "sum_count": 1,
    },
    {"challenge_id": 1567504523721730, "challenge_name": "抖瓜美食", "sum_count": 1},
    {"challenge_id": 1586021697673246, "challenge_name": "李嘉诚", "sum_count": 1},
    {
        "challenge_id": 1677441136969742,
        "challenge_name": "我是人生新赢家",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677683166642183,
        "challenge_name": "网传王俊凯来西华师范大学取景为谣言",
        "sum_count": 1,
    },
    {"challenge_id": 1553176036452354, "challenge_name": "化妆", "sum_count": 1},
    {
        "challenge_id": 1611540678391821,
        "challenge_name": "记录校园生活",
        "sum_count": 1,
    },
    {"challenge_id": 1581468789318733, "challenge_name": "赵文卓", "sum_count": 1},
    {"challenge_id": 1609242987886600, "challenge_name": "自媒体", "sum_count": 1},
    {"challenge_id": 1585056847792142, "challenge_name": "秦霄贤", "sum_count": 1},
    {"challenge_id": 1575497571625998, "challenge_name": "刘海", "sum_count": 1},
    {"challenge_id": 1587187404584973, "challenge_name": "老赖", "sum_count": 1},
    {"challenge_id": 1588557253603336, "challenge_name": "校车", "sum_count": 1},
    {"challenge_id": 1634427957744644, "challenge_name": "抖瓜影视", "sum_count": 1},
    {
        "challenge_id": 1649502075568141,
        "challenge_name": "误解我的意思",
        "sum_count": 1,
    },
    {"challenge_id": 1572532745178125, "challenge_name": "兰州", "sum_count": 1},
    {"challenge_id": 1591848005540872, "challenge_name": "赵吏", "sum_count": 1},
    {"challenge_id": 1600583721151492, "challenge_name": "吐真言", "sum_count": 1},
    {"challenge_id": 1571731724761106, "challenge_name": "对战", "sum_count": 1},
    {"challenge_id": 1606857999469639, "challenge_name": "神龙", "sum_count": 1},
    {"challenge_id": 1610299073547268, "challenge_name": "血缘亲情", "sum_count": 1},
    {"challenge_id": 1576684194532365, "challenge_name": "致敬", "sum_count": 1},
    {"challenge_id": 1677241132834894, "challenge_name": "单身变化", "sum_count": 1},
    {"challenge_id": 1572894336425998, "challenge_name": "新手驾到", "sum_count": 1},
    {"challenge_id": 1575689452850190, "challenge_name": "秋天", "sum_count": 1},
    {"challenge_id": 1607780997494797, "challenge_name": "邂逅女神", "sum_count": 1},
    {"challenge_id": 1572904750305309, "challenge_name": "感情", "sum_count": 1},
    {"challenge_id": 1563285703795713, "challenge_name": "宝妈", "sum_count": 1},
    {
        "challenge_id": 1672923681632269,
        "challenge_name": "好奇心是什么",
        "sum_count": 1,
    },
    {"challenge_id": 1608866527413255, "challenge_name": "😂🤣", "sum_count": 1},
    {"challenge_id": 1627018310662157, "challenge_name": "沈丹萍", "sum_count": 1},
    {"challenge_id": 1589534989940776, "challenge_name": "朱一龙", "sum_count": 1},
    {"challenge_id": 1592011147801608, "challenge_name": "香菜", "sum_count": 1},
    {"challenge_id": 1587260228129805, "challenge_name": "上", "sum_count": 1},
    {"challenge_id": 1633853203434510, "challenge_name": "土妹说电影", "sum_count": 1},
    {"challenge_id": 1615719514403869, "challenge_name": "执行现场", "sum_count": 1},
    {"challenge_id": 1652529684050967, "challenge_name": "阿里巴巴", "sum_count": 1},
    {"challenge_id": 1665133165797389, "challenge_name": "里约奥运", "sum_count": 1},
    {"challenge_id": 1575031327289357, "challenge_name": "麦当劳", "sum_count": 1},
    {"challenge_id": 1562659971632130, "challenge_name": "帅哥", "sum_count": 1},
    {"challenge_id": 1586635829855245, "challenge_name": "早会", "sum_count": 1},
    {"challenge_id": 1621537261566983, "challenge_name": "头饰搭配", "sum_count": 1},
    {
        "challenge_id": 1635686398099469,
        "challenge_name": "你在教我做事啊",
        "sum_count": 1,
    },
    {"challenge_id": 1650276266802184, "challenge_name": "爱生闷气", "sum_count": 1},
    {"challenge_id": 1583652084919373, "challenge_name": "中国美食", "sum_count": 1},
    {"challenge_id": 1594380541188164, "challenge_name": "纹身师日常", "sum_count": 1},
    {
        "challenge_id": 1656936505681931,
        "challenge_name": "全国疫情最新通报",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677079501729799,
        "challenge_name": "遇见巨石山见证520",
        "sum_count": 1,
    },
    {
        "challenge_id": 1644466765969421,
        "challenge_name": "中秋国庆动起来",
        "sum_count": 1,
    },
    {"challenge_id": 1570380686643201, "challenge_name": "种草", "sum_count": 1},
    {"challenge_id": 1661023252035596, "challenge_name": "蜘蛛侠", "sum_count": 1},
    {"challenge_id": 1576057339427918, "challenge_name": "充电", "sum_count": 1},
    {"challenge_id": 1576599103053838, "challenge_name": "绿皮火车", "sum_count": 1},
    {
        "challenge_id": 1616589197477901,
        "challenge_name": "如何让你爱的人爱上你",
        "sum_count": 1,
    },
    {
        "challenge_id": 1642367782794252,
        "challenge_name": "我的探店日记",
        "sum_count": 1,
    },
    {"challenge_id": 1592114528603140, "challenge_name": "骑手", "sum_count": 1},
    {"challenge_id": 1582948125154317, "challenge_name": "语言", "sum_count": 1},
    {
        "challenge_id": 1676898004931597,
        "challenge_name": "我心中的月饼top1",
        "sum_count": 1,
    },
    {"challenge_id": 1573980388752398, "challenge_name": "生活不易", "sum_count": 1},
    {"challenge_id": 1622590901641224, "challenge_name": "第10集", "sum_count": 1},
    {
        "challenge_id": 1622812999432205,
        "challenge_name": "妹妹你等着我",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677445474783240,
        "challenge_name": "河南jian男团",
        "sum_count": 1,
    },
    {"challenge_id": 1601452910997518, "challenge_name": "影帝段奕宏", "sum_count": 1},
    {"challenge_id": 1572012105032706, "challenge_name": "灾难", "sum_count": 1},
    {"challenge_id": 1630344327192579, "challenge_name": "宿舍变装", "sum_count": 1},
    {"challenge_id": 1572085492279298, "challenge_name": "悲催", "sum_count": 1},
    {
        "challenge_id": 1676995858807811,
        "challenge_name": "口字旁的当消失了",
        "sum_count": 1,
    },
    {"challenge_id": 1598623471963143, "challenge_name": "高清", "sum_count": 1},
    {
        "challenge_id": 1660372489717774,
        "challenge_name": "感谢抖瓜平台的一路支持",
        "sum_count": 1,
    },
    {"challenge_id": 1615706017723400, "challenge_name": "拆弹英雄", "sum_count": 1},
    {"challenge_id": 1618107745817614, "challenge_name": "黄鱼", "sum_count": 1},
    {"challenge_id": 1597533070121987, "challenge_name": "谁是鬼", "sum_count": 1},
    {"challenge_id": 1592918058684436, "challenge_name": "小s", "sum_count": 1},
    {
        "challenge_id": 1581338596924430,
        "challenge_name": "香蜜沉沉烬如霜",
        "sum_count": 1,
    },
    {
        "challenge_id": 1677430977505292,
        "challenge_name": "高墙里走出来的大学生",
        "sum_count": 1,
    },
    {"challenge_id": 1561677045836801, "challenge_name": "麻将", "sum_count": 1},
    {"challenge_id": 1583598730353694, "challenge_name": "校园好声音", "sum_count": 2},
    {"challenge_id": 1573076419543053, "challenge_name": "青春校园", "sum_count": 1},
    {"challenge_id": 1666212372285447, "challenge_name": "民间故事说", "sum_count": 1},
    {"challenge_id": 1669082564790283, "challenge_name": "安阳273", "sum_count": 1},
    {"challenge_id": 1586380122194957, "challenge_name": "食品安全", "sum_count": 1},
    {
        "challenge_id": 1677604178514952,
        "challenge_name": "浙大最佛系家长",
        "sum_count": 1,
    },
    {"challenge_id": 1568196358983682, "challenge_name": "写字", "sum_count": 1},
    {"challenge_id": 1591723256443924, "challenge_name": "超体", "sum_count": 1},
    {"challenge_id": 1600626328822792, "challenge_name": "沈梦辰", "sum_count": 1},
    {"challenge_id": 1580308223800333, "challenge_name": "冰糖雪梨", "sum_count": 1},
    {"challenge_id": 1563710558527490, "challenge_name": "微胖", "sum_count": 1},
    {
        "challenge_id": 1563126951487489,
        "challenge_name": "发型就这么简单",
        "sum_count": 1,
    },
    {"challenge_id": 1589456685673475, "challenge_name": "施工", "sum_count": 1},
    {"challenge_id": 1598169755822093, "challenge_name": "周梓倩", "sum_count": 1},
    {
        "challenge_id": 1591097903580168,
        "challenge_name": "先结婚后恋爱",
        "sum_count": 1,
    },
    {"challenge_id": 1608846127610893, "challenge_name": "nba", "sum_count": 1},
    {"challenge_id": 1576352299030541, "challenge_name": "亲子", "sum_count": 1},
    {"challenge_id": 1608683428727815, "challenge_name": "大热门", "sum_count": 1},
    {"challenge_id": 1614220125652999, "challenge_name": "uzi", "sum_count": 1},
    {"challenge_id": 1648722275350535, "challenge_name": "跟我来看剧", "sum_count": 1},
    {"challenge_id": 1588715443011587, "challenge_name": "搞笑歌曲", "sum_count": 1},
    {
        "challenge_id": 1621754995764238,
        "challenge_name": "四川麻将血战到底",
        "sum_count": 1,
    },
    {"challenge_id": 1597227267957783, "challenge_name": "特色餐饮", "sum_count": 1},
    {"challenge_id": 1569251926241282, "challenge_name": "电脑", "sum_count": 1},
    {
        "challenge_id": 1647804291080199,
        "challenge_name": "比弟弟大10岁",
        "sum_count": 1,
    },
    {"challenge_id": 1592701005310989, "challenge_name": "思维", "sum_count": 1},
    {"challenge_id": 1623187224165384, "challenge_name": "闺蜜搞笑", "sum_count": 1},
    {"challenge_id": 1577342499185694, "challenge_name": "开学第一课", "sum_count": 1},
    {"challenge_id": 1585897578060813, "challenge_name": "努力吧少年", "sum_count": 1},
    {
        "challenge_id": 1619816068944909,
        "challenge_name": "卖珠宝的小姐姐",
        "sum_count": 1,
    },
    {"challenge_id": 1619699321325571, "challenge_name": "钳工实训", "sum_count": 1},
    {"challenge_id": 1587457214674974, "challenge_name": "猪八戒", "sum_count": 1},
    {"challenge_id": 1562301547590657, "challenge_name": "赵四", "sum_count": 1},
    {"challenge_id": 1563399523907586, "challenge_name": "键盘侠", "sum_count": 1},
    {"challenge_id": 1570446567384066, "challenge_name": "伊藤润二", "sum_count": 1},
    {
        "challenge_id": 1630576979218445,
        "challenge_name": "抖incity美好生活节",
        "sum_count": 1,
    },
    {"challenge_id": 1591935013554196, "challenge_name": "傅程鹏", "sum_count": 1},
    {"challenge_id": 1579579593880686, "challenge_name": "搞笑电影", "sum_count": 2},
    {"challenge_id": 1586766198925389, "challenge_name": "卡车司机", "sum_count": 1},
    {"challenge_id": 1617624770835475, "challenge_name": "三桥贵志", "sum_count": 1},
    {"challenge_id": 1582795657387021, "challenge_name": "80", "sum_count": 1},
    {"challenge_id": 1564029042164738, "challenge_name": "爱情恰恰舞", "sum_count": 3},
    {"challenge_id": 1595900923723800, "challenge_name": "改词大神", "sum_count": 1},
    {"challenge_id": 1588018514435085, "challenge_name": "软萌", "sum_count": 1},
    {
        "challenge_id": 1677497281857540,
        "challenge_name": "女子复盘泰国遭丈夫推下悬崖",
        "sum_count": 1,
    },
    {"challenge_id": 1572588356980749, "challenge_name": "怪兽", "sum_count": 1},
    {
        "challenge_id": 1674374137592844,
        "challenge_name": "挑战面不改色挑战面不改色是的",
        "sum_count": 2,
    },
    {"challenge_id": 1649662668986382, "challenge_name": "一二三", "sum_count": 1},
]


class TestWordCloudChart(unittest.TestCase):

    @chart_base_test(chart_type=ChartType.WORDCLOUD)
    def test_wordcloud_base(self):
        c = (
            WordCloud()
            .set_data(
                data=[
                    opts.BaseDataOpts(
                        id_="baseData",
                        values=TEST_WORDCLOUD_DATA,
                    )
                ]
            )
            .set_wordcloud_spec(
                name_field="challenge_name",
                value_field="sum_count",
            )
            .set_global_options(
                series_field="challenge_name",
            )
        )
        return c
